// dbblk.cpp    Member functions for DbBlock

/*
* Send comments to:
*   Dave Kuhlman
*   Reify
*   426 Madison Ave #9
*   Redwood City, CA 94061
*   415-368-8450
*   Internet: dkuhlman@netcom.com
*
* Copyright (c) 1994 G. David Kuhlman.
* All rights reserved.
*
* Permission is hereby granted, without written agreement and without
* license or royalty fees, to use, copy, modify, and distribute this
* software and its documentation for any purpose, provided that the
* above copyright notice and the following two paragraphs appear in
* all copies of this software.
* 
* IN NO EVENT SHALL G. DAVID KUHLMAN BE LIABLE TO ANY PARTY FOR
* DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
* OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF G. DAVID KUHLMAN
* HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* G. DAVID KUHLMAN SPECIFICALLY DISCLAIMS ANY WARRANTIES,
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
* ON AN "AS IS" BASIS, AND G. DAVID KUHLMAN HAS NO OBLIGATION TO
* PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

#include <string.h>

#include <mdlsyn.hpp>


#include "mdl.hpp"
#include "..\sup\sup.hpp"
#include "db.hpp"
#include "dbblk.hpp"
#include "bt.h"

// --------------------------------------------------------

// Private constants:

// Private globals:

// Private function prototypes:

// --------------------------------------------------------

DbBlock::DbBlock()
    : blkIndex(0)
{
    data = new BlkBlkRec;
    memset(data, 0, sizeof(BlkBlkRec));
    initBlk();
} // DbBlock::DbBlock

// --------------------------------------------------------

// The copy constructor is private so no one can use it.
DbBlock::DbBlock(const DbBlock& rhs)
    : data(NULL), blkIndex(0)
{
    if (this == &rhs)
    {
        return;
    } // if
    memset(data, 0, sizeof(BlkBlkRec));
    initBlk();
} // DbBlock::DbBlock

// --------------------------------------------------------

DbBlock::~DbBlock()
{

} // DbBlock::~DbBlock

// --------------------------------------------------------

// The assignment operator is private so no one can use it.
DbBlock &DbBlock::operator =(const DbBlock& rhs)
{
    if (this == &rhs)
    {
        return *this;
    } // if
    return *this;
} // DbBlock::operator =

// --------------------------------------------------------

// Add a record to the current block.
//  If the current block is full, write it out and initialize another.
//  Generate an error if we exceed the maximum number of blocks.

// If descriptor is True, then create a indirection/descriptor record
//  and return its record number.

Db::RecNumber DbBlock::AddDbRec(
    void    *pRec,
    short   len,
    Boolean descriptor,     // True=create indirection/descriptor record
    char * className
)
{
    Boolean         needNewBuf;
    Db::RecNumber   returnRecno;
    short           pinNo;
    BlkRecnoRec     blkRecno;
    short           sizeStore;
    BlkBlkRecPtr    pBlk;

    sizeStore = BLKSIZESTORE;

    // Do we need a new block?
    needNewBuf = False;

    // Is there a free pin?
    if ( data->h.nextFreePin == BLKNULLPIN )
    {
        needNewBuf = True;
    } // if
    // Is there is not enough room for the string in the current block.
    //  Use 'len + 2' because the length is stored as a short.
    else if ( ( sizeStore - data->h.nextFreeStore ) < (short)( alignLength(len + 2) ) )
    {
        needNewBuf = True;
    } // if

    // If we cannot put it in the current block, then bump the current
    //  block pointer to the next block in the array of blocks.
    if (needNewBuf)
    {
        pBlk = data;
        data = new BlkBlkRec;
        memset(data, 0, sizeof(BlkBlkRec));
        initBlk();
        data->next = pBlk;
        blkIndex++;
    } // if

    // If it still doesn't fit, then it's too big, period.
    //  Give up and go home.
    if ( ( sizeStore - data->h.nextFreeStore ) < (short)( alignLength(len + 2) ) )
    {
        MdlSupport::ReportError( "(DbBlock::MakeDbRec) Record too big" ,
            MdlSupport::MDL_ERROR_LEVEL_1);
    } // if

    // Copy the string into the current block and get a pin number.
    pinNo = addRecToBlk(pRec, len);

    blkRecno.blkNo = (short)(blkIndex + 1);
    blkRecno.pinNo = (short)(pinNo + 1);

    memcpy(&returnRecno, &blkRecno, sizeof(Db::RecNumber));

    if (descriptor)
    {
        Db::DescriptorRec desc;
        desc.data = returnRecno;
        desc.className = AddDbString(className);
        returnRecno = AddDbRec(&desc, sizeof(desc));
        // Make the block number in the record number negative to
        //  mark this as a descriptor record.
        BlkRecnoRecPtr pBlkRecno = (BlkRecnoRecPtr)&returnRecno;
        pBlkRecno->blkNo = 0 - pBlkRecno->blkNo;
    } // if
    return returnRecno;

} // DbBlock::AddDbRec

// --------------------------------------------------------

// Add a string to the current block.
//  If the current block is full, write it out and initialize another.
//  Generate an error if we exceed the maximum number of blocks.

Db::RecNumber DbBlock::AddDbString(const char *str)
{
    Boolean         needNewBuf;
    int             len;
    Db::RecNumber   returnRecno;
    short           pinNo;
    BlkRecnoRec     blkRecno;
    short           sizeStore;
    BlkBlkRecPtr    pBlk;

    sizeStore = BLKSIZESTORE;

    len = strlen(str);

    // Do we need a new block?
    needNewBuf = False;

    // Is there a free pin?
    if (data->h.nextFreePin == BLKNULLPIN)
    {
        needNewBuf = True;
    } // if
    // Is there is not enough room for the string in the current block.
    //  Use 'len + 1' because we have to store the null terminator, too.
    else if ((sizeStore - data->h.nextFreeStore ) < (short)( alignLength(len + 1)))
    {
        needNewBuf = True;
    } // if

    // If we cannot put it in the current block, then
    //  write out the current block and clear it.
    if (needNewBuf)
    {
        pBlk = data;
        data = new BlkBlkRec;
        memset(data, 0, sizeof(BlkBlkRec));
        initBlk();
        data->next = pBlk;
        blkIndex++;
    } // if

    // If it still doesn't fit, then it's too big, period.
    //  Give up and go home.
    if ((sizeStore - data->h.nextFreeStore) < (short)(alignLength(len + 1)))
    {
        MdlSupport::ReportError("(DbBlock::MakeDbString) String too big" ,
            MdlSupport::MDL_ERROR_LEVEL_1);
    } // if

    // Copy the string into the current block and get a pin number.
    pinNo = addStrToBlk(str);

    blkRecno.blkNo = (short)(blkIndex + 1);
    blkRecno.pinNo = (short)(pinNo + 1);

    memcpy(&returnRecno, &blkRecno, sizeof(Db::RecNumber));
    return returnRecno;
} // DbBlock::AddDbString

// --------------------------------------------------------

Db::RecNumber DbBlock::AddDbString(char * str)
{
    return AddDbString((const char *)str);
} // DbBlock::AddDbString

// --------------------------------------------------------

// Flush the DB blocks in the array to disk and return the record number of
//  the first block in the array.

Db::RecNumber DbBlock::FlushBlocks(Db::RecNumber headRecno)
{
    Db::RecNumber   recno = Db::DB_NULL_RECID;
    BlkBlkRecPtr    pBlk;
    BlkBlkRecPtr    pBlk1;
    BlkBlkRecPtr    *pPtr;

    if (data == NULL)
    {
        return recno;
    } // if

    data->h.headRecno = headRecno;

    pBlk = data;
    while (pBlk != NULL)
    {
        pBlk->h.nextBlk = recno;
        recno = btAddRec(pBlk, sizeof(BlkBlkRec));
        pBlk1 = pBlk->next;
        delete pBlk;
        pBlk = pBlk1;
    } // for

    data = NULL;
    return( recno );
} // DbBlock::FlushBlocks

// --------------------------------------------------------

// Return a pointer to a string within an array of blocks that
//  have been loaded from the database.

char * DbBlock::GetStr(Db::RecNumber strId)
{
    short           blkNo;
    short           pinNo;
    short           offset;
    BlkRecnoRecPtr  pRecno;
    BlkBlkRecPtr    pBlk = data;
    int             idx;

    pRecno = (BlkRecnoRecPtr)&strId;

    // The pin and block numbers are relative to 0 but we saved them
    //  relative to 1.  Adjust them by subtracting 1.
    blkNo = (short)(pRecno->blkNo - 1);
    pinNo = (short)(pRecno->pinNo - 1);

    for (idx = 0; idx < blkNo; idx++)
    {
        pBlk = pBlk->next;
    } // for

    offset = pBlk->h.pins[pinNo];

    return &pBlk->store[offset];
} // DbBlock::GetStr

// --------------------------------------------------------

/* Find a record within an array of blocks that
*   have been loaded from the database.
*   Copy the record into the requestor's buffer.
*/
void DbBlock::GetRec(
    Db::RecNumber   recId,      // IN: String block record ID
    void        *pBuf       // OUT: Destination for data
)
{
    short           blkNo;
    short           pinNo;
    short           offset;
    BlkRecnoRecPtr  pRecno;
    BlkBlkRecPtr    pBlk = data;
    BlkStoreRecPtr  pData;
    int             idx;

    pRecno = (BlkRecnoRecPtr)&recId;

    // The pin and block numbers are relative to 0 but we saved them
    //  relative to 1.  Adjust them by subtracting 1.
    blkNo = (short)(pRecno->blkNo - 1);
    pinNo = (short)(pRecno->pinNo - 1);

    for (idx = 0; idx < blkNo; idx++)
    {
        pBlk = pBlk->next;
    } // for

    offset = pBlk->h.pins[pinNo];
    pData = (BlkStoreRecPtr)&pBlk->store[offset];

    // Now we know where the data is.  Copy it into the requestor's buffer.
    //  The length is in the first two bytes.
    memcpy(pBuf, pData->data, pData->size);

} // DbBlock::GetRec

// --------------------------------------------------------

// Write  a record back into an array of DB blocks.

void DbBlock::WriteDbRec(
    Db::RecNumber   recId,      // IN: Block record ID
    void        *pBuf,      // IN: Ptr to data
    short       size        // IN: Size of data.  Ignored.  Use old size
)
{
    short           blkNo;
    short           pinNo;
    short           offset;
    BlkRecnoRecPtr  pRecno;
    BlkBlkRecPtr    pBlk;
    BlkStoreRecPtr  pData;
    int             idx;

    pRecno = (BlkRecnoRecPtr)&recId;

    // The pin and block numbers are relative to 0 but we saved them
    //  relative to 1.  Adjust them by subtracting 1.
    blkNo = (short)(pRecno->blkNo - 1);
    pinNo = (short)(pRecno->pinNo - 1);

    for (idx = 0; idx < blkNo; idx++)
    {
        pBlk = pBlk->next;
    } // for
    offset = pBlk->h.pins[pinNo];
    pData = (BlkStoreRecPtr)&pBlk->store[offset];

    // Now we know where the data is and how big it is.
    //  Copy it into the requestor over the top of the old data in the block.
    //  The length is in the first two bytes.
    memcpy(pData->data, pBuf, pData->size);
} // DbBlock::WriteDbRec

// --------------------------------------------------------

/* Delete a set of (chained) blocks from the database.
*/
void DbBlock::DeleteDbRecs(
    Db::RecNumber   recId       // IN: Record ID of first block
)
{
    Db::RecNumber   recno;
    BlkBlkRec   dbrec;

    recno = recId;
    while ( recno != Db::DB_NULL_RECID )
    {
        btReadRec(recno, &dbrec);
        btDeleteRec( recno );
        recno = dbrec.h.nextBlk;
    } // while
} // DbBlock::DeleteDbRecs

// --------------------------------------------------------

// Load a list of blocks into an array of blocks.
//  This array of blocks is the array in which FindStr() searches.

Db::RecNumber DbBlock::LoadBlocks(Db::RecNumber firstRecno)
{
    int             idx;
    Db::RecNumber   recno;
    BlkBlkRecPtr    pDbRec;
    BlkBlkRecPtr    pDbRec1;
    BlkBlkRecPtr    pPrevDbRec;

    recno = firstRecno;

    pDbRec1 = NULL;
    for (idx = 0; recno != Db::DB_NULL_RECID; idx++)
    {
        pDbRec = new BlkBlkRec;
        btReadRec(recno, pDbRec);
        pDbRec->next = pDbRec1;
        pDbRec1 = pDbRec;
        recno = pDbRec->h.nextBlk;
    } // for
    blkIndex = idx - 1;
    recno = pDbRec->h.headRecno;

    // When we created the list of blocks, we pre-pended each new record
    //  to the beginning (not the end) of the list.  Then we stored the
    //  records in reverse order and read them in in reverse order.
    //  Now we have to reverse the order of the list.
    pPrevDbRec = NULL;
    while (pDbRec != NULL)
    {
        pDbRec1 = pDbRec->next;
        pDbRec->next = pPrevDbRec;
        pPrevDbRec = pDbRec;
        pDbRec = pDbRec1;
    } // while

    data = pPrevDbRec;
    return recno;
} // DbBlock::LoadBlocks

// --------------------------------------------------------

void DbBlock::initBlk(void)
{
    int   idx;

    data->next = NULL;
    data->h.nextFreePin = 0;
    for ( idx = 0; idx < BLKNOPINS; idx++ )
    {
        data->h.pins[idx] = (short)(idx + 1);
    } /* for */
    data->h.pins[BLKNOPINS - 1] = BLKNULLPIN;

    data->h.nextFreeStore = 0;
    data->h.nextBlk =       Db::DB_NULL_RECID;
    data->h.headRecno =     Db::DB_NULL_RECID;

} // DbBlock::initBlk

// --------------------------------------------------------

/* Add a block to a block.
*   Assume that the caller has checked to make sure that there is still
*   an available pin and are that the store in the block has enough room
*   for the block.
*/
short DbBlock::addRecToBlk(void *pRec, short len)
{
    short           pinNo;
    BlkStoreRecPtr  pData;

    pinNo = data->h.nextFreePin;
    data->h.nextFreePin = data->h.pins[pinNo];

    pData = (BlkStoreRecPtr)&data->store[data->h.nextFreeStore];
    pData->size = len;
    memcpy(pData->data, pRec, len);

    data->h.pins[pinNo] = data->h.nextFreeStore;

    data->h.nextFreeStore += alignLength(len + 2);

    return pinNo;
} // DbBlock::addRecToBlk

// --------------------------------------------------------

// Add a string to a block.
//  Assume that the caller has checked to make sure that there is still
//  an available pin and are that the store in the block has enough room
//  for the string.

short DbBlock::addStrToBlk(const char *pStr)
{
    short   pinNo;
    short   len;

    len = (short)strlen( pStr );
    pinNo = data->h.nextFreePin;
    data->h.nextFreePin = data->h.pins[ pinNo ];

    strcpy( &data->store[ data->h.nextFreeStore ], pStr );
    data->h.pins[ pinNo ] = data->h.nextFreeStore;
    data->h.nextFreeStore += alignLength(len + 1);

    return( pinNo );
} // DbBlock::addStrToBlk

// --------------------------------------------------------

// Align size on 4 btye boundry for the sun

short DbBlock::alignLength(short len)
{
    short alignSize;

#ifdef SUNSPARC
    if (len%4 == 0)
        alignSize = len;
    else
        alignSize = len + 4 - (len%4);
#else
    alignSize = len;
#endif

    return(alignSize);
} // DbBlock::alignLength

// --------------------------------------------------------

Boolean DbBlock::IsDescriptor(Db::RecNumber recno, Db::RecNumber &convertedRecno)
{
    BlkRecnoRecPtr pBlkRecno = (BlkRecnoRecPtr)&recno;
    BlkRecnoRecPtr pConvertedBlkRecno = (BlkRecnoRecPtr)&convertedRecno;

    if (pBlkRecno->blkNo < 0)
    {
        pConvertedBlkRecno->blkNo = 0 - pBlkRecno->blkNo;
        pConvertedBlkRecno->pinNo = pBlkRecno->pinNo;
        return True;
    } // if
    else
    {
        convertedRecno = recno;
        return False;
    } // else
} // DbBlock::IsDescriptor

// --------------------------------------------------------

