// dbdir.hpp    Class definitions for DbDirDirectory, DbDirModel

/*
* Send comments to:
*   Dave Kuhlman
*   Reify
*   426 Madison Ave #9
*   Redwood City, CA 94061
*   415-368-8450
*   Internet: dkuhlman@netcom.com
*
* Copyright (c) 1994 G. David Kuhlman.
* All rights reserved.
*
* Permission is hereby granted, without written agreement and without
* license or royalty fees, to use, copy, modify, and distribute this
* software and its documentation for any purpose, provided that the
* above copyright notice and the following two paragraphs appear in
* all copies of this software.
* 
* IN NO EVENT SHALL G. DAVID KUHLMAN BE LIABLE TO ANY PARTY FOR
* DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
* OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF G. DAVID KUHLMAN
* HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* G. DAVID KUHLMAN SPECIFICALLY DISCLAIMS ANY WARRANTIES,
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
* ON AN "AS IS" BASIS, AND G. DAVID KUHLMAN HAS NO OBLIGATION TO
* PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

#ifndef dbdir_HPP
#define dbdir_HPP

#include <mdlsyn.hpp>

// #include "..\sup\reclink.hpp"

#include "mdl.hpp"
#include "..\sup\sup.hpp"
#include "db.hpp"
#include "dbblk.hpp"
#include "bt.h"

class DbDirModel;

class DbDirDirectory {
public:
    DbDirDirectory();
    DbDirDirectory(char * initName);
    ~DbDirDirectory();
    enum AddModelResponse {
        AddModelFailure = 0,
        AddModelSuccess,
        AddModelBadPath,
        AddModelSubDirNotExist,
        AddModelModelExists
    }; 
    enum PathNameType {
        PathNameError = 1,
        PathNameBadName,
        PathNameDirectoryEntry,
        PathNameModelEntry
    };
    AddModelResponse AddModel(char * path_name);

protected:
private:
    char * name;
    DbDirModel * models;
    DbDirDirectory * firstChild;
    DbDirDirectory * lastChild;
    DbDirDirectory * sibling;

    DbDirDirectory * findNextDirectory(char * subname);

    PathNameType nextName(char *pPin, char * pP1out, char * subname);

    // The copy constructor is private.  Don't call it.  It does NOT work.
    DbDirDirectory(const DbDirDirectory& rhs);
    // The assignment operator is private so we can't use it.  It does NOT work.
    DbDirDirectory& operator =(const DbDirDirectory& rhs);
}; // DbDirDirectory


class DbDirModel {
public:
    DbDirModel();
    DbDirModel(DbDirModel * initNext, char * initName, Db::RecNumber initRecno);
    ~DbDirModel();
    DbDirModel * findNextModel(char * subname);
protected:
private:
    char * name;
    Db::RecNumber firstBlock;
    DbDirModel * next;


    // The copy constructor is private.  Don't call it.  It does NOT work.
    DbDirModel(const DbDirModel& rhs);
    // The assignment operator is private so we can't use it.  It does NOT work.
    DbDirModel& operator =(const DbDirModel& rhs);
}; // DbDirModel

#endif // dbdir_HPP

