// kr.cpp    Member functions for Models

/*
*
* Send comments to:
*   Dave Kuhlman
*   Reify
*   426 Madison Ave #9
*   Redwood City, CA 94061
*   415-368-8450
*   Internet: dkuhlman@netcom.com
*
* Copyright (c) 1994 G. David Kuhlman.
* All rights reserved.
*
* Permission is hereby granted, without written agreement and without
* license or royalty fees, to use, copy, modify, and distribute this
* software and its documentation for any purpose, provided that the
* above copyright notice and the following two paragraphs appear in
* all copies of this software.
* 
* IN NO EVENT SHALL G. DAVID KUHLMAN BE LIABLE TO ANY PARTY FOR
* DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
* OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF G. DAVID KUHLMAN
* HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* G. DAVID KUHLMAN SPECIFICALLY DISCLAIMS ANY WARRANTIES,
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
* ON AN "AS IS" BASIS, AND G. DAVID KUHLMAN HAS NO OBLIGATION TO
* PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*
*/

// --------------------------------------------------------

#include "kr.hpp"

// --------------------------------------------------------

// Private function prototypes:

static void genIpfText(KrOstream & ofs, int level_, KrStringList * notes);

// ---------------------------------------------------------
// ---------------------------------------------------------

// Member functions for: Bibliography

// ---------------------------------------------------------

// Default constructor for class Bibliography//gen.icn/938
Bibliography::Bibliography()
    :
    id_(NULL),
    bibltitle(NULL),
    documents(new DocumentList()),
    authors(new AuthorList())
{
} // Bibliography::Bibliography

// ---------------------------------------------------------

// Copy constructor for class Bibliography//gen.icn/959
Bibliography::Bibliography(const Bibliography& rhs_)
    :
    id_(NULL),
    bibltitle(NULL),
    documents(new DocumentList()),
    authors(new AuthorList())
{
    if (((char *)*rhs_.id_) != NULL)
    {
        id_ = new KrString(*rhs_.id_);
    } // if
    // Copy member: bibltitle//gen.icn/1016
    bibltitle = new KrString(*rhs_.bibltitle);
    // Copy member: documents//gen.icn/1023
    documents = rhs_.documents->Copy();
    // Copy member: authors//gen.icn/1023
    authors = rhs_.authors->Copy();
} // Bibliography::Bibliography

// ---------------------------------------------------------

// Constructor to create Bibliography by loading from the database.//gen.icn/1316
Bibliography * Bibliography::Load(DbBlock * block_, Db::RecNumber recno_, Db::RecNumber & nextRecno_)
{
    Db::RecNumber convertedRecno_;

    if (recno_ == Db::DB_NULL_RECID)
    {
        nextRecno_ = Db::DB_NULL_RECID;
        return NULL;
    } // if
    if (DbBlock::IsDescriptor(recno_, convertedRecno_))
    {
        Db::DescriptorRec desc_;
        block_->GetRec(convertedRecno_, &desc_);
        char * type = block_->GetStr(desc_.className);
        recno_ = desc_.data;
        if (strcmp(type, "Bibliography") == 0)
        {
            Bibliography * pObj = Bibliography::Load(block_, recno_, nextRecno_);
            return pObj;
        } // if
    } // if
    else
    {
        DbBibliography dbBibliography;
        block_->GetRec(recno_, &dbBibliography);
        Bibliography * pObj = new Bibliography(block_, dbBibliography);
        nextRecno_ = dbBibliography.next_;
        return pObj;
    } // else
    return NULL;
} // Bibliography::Load

// ---------------------------------------------------------

// Constructor to create Bibliography by loading from the database.
Bibliography::Bibliography(DbBlock * block_, DbBibliography & dbBibliography_)
    :
    id_(NULL),
    bibltitle(NULL),
    documents(new DocumentList()),
    authors(new AuthorList())
{
    // Convert each field.
    BuildRec(block_, dbBibliography_);
} // Bibliography::Bibliography

// ---------------------------------------------------------

// Destructor for class Bibliography//gen.icn/1608
Bibliography::~Bibliography()
{

    // Destruct member: bibltitle//gen.icn/1641
    delete bibltitle;
    // Destruct member: documents//gen.icn/1653
    delete documents;
    // Destruct member: authors//gen.icn/1653
    delete authors;
} // Bibliography::~Bibliography

// ---------------------------------------------------------

// Build Bibliography record from DB record.//gen.icn/1680
void Bibliography::BuildRec(DbBlock * block_, DbBibliography& dbBibliography)
{
    Db::RecNumber recno_;
    if (dbBibliography.id_ != Db::DB_NULL_RECID)
    {
        id_ = new KrString(block_->GetStr(dbBibliography.id_));
    } // if
    // Convert each field in a Bibliography//gen.icn/1695
    // Load member: bibltitle//gen.icn/1740
    if (dbBibliography.bibltitle != Db::DB_NULL_RECID)
    {
        bibltitle = new KrString(block_->GetStr(dbBibliography.bibltitle));
    } // if
    // Load member: documents//gen.icn/1770
    documents = DocumentList::Load(block_, dbBibliography.documents);
    // Load member: authors//gen.icn/1770
    authors = AuthorList::Load(block_, dbBibliography.authors);
} // Bibliography::BuildRec

// ---------------------------------------------------------

// Assignment operator for class Bibliography//gen.icn/1971
Bibliography& Bibliography::operator=(const Bibliography& rhs_)
{
    if (this == &rhs_)
    {
        return *this;
    } // if
    // Copy member: bibltitle//gen.icn/1016
    bibltitle = new KrString(*rhs_.bibltitle);
    // Copy member: documents//gen.icn/1023
    documents = rhs_.documents->Copy();
    // Copy member: authors//gen.icn/1023
    authors = rhs_.authors->Copy();
    return *this;
} // Bibliography::operator=

// ---------------------------------------------------------

// Decompile member function for class Bibliography//gen.icn/2008
void Bibliography::Decompile(KrOstream& outf_, int level_, int & count_)
{
    char buf[1024];

    // Decompile object ID//gen.icn/2021
    if (id_ == NULL)
    {
        count_++;
        (outf_ <<= level_) << "(Bibliography bibliography" << count_ << endl;
    } // if
    else
    {
        (outf_ <<= level_) << "(Bibliography " << ((char *)*id_) << endl;
    } // else
    level_++;
    // Decompile member: bibltitle//gen.icn/2141
    if (bibltitle != NULL)
    {
        bibltitle->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(bibltitle \"" << buf << "\")" << endl;
    } // if
    // Decompile member: documents//gen.icn/2182
    DocumentList * pList2 = documents;
    if (pList2 != NULL)
    {
        DocumentLink * pLink2 = pList2->GetFirst();
        if (pLink2 != NULL)
        {
            (outf_ <<= level_) << "(documents" << endl;
            while (pLink2 != NULL)
            {
                Document * pDocument = pLink2->GetData();
                if (pDocument != NULL)
                {
                    pDocument->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink2 = pLink2->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    // Decompile member: authors//gen.icn/2182
    AuthorList * pList3 = authors;
    if (pList3 != NULL)
    {
        AuthorLink * pLink3 = pList3->GetFirst();
        if (pLink3 != NULL)
        {
            (outf_ <<= level_) << "(authors" << endl;
            while (pLink3 != NULL)
            {
                Author * pAuthor = pLink3->GetData();
                if (pAuthor != NULL)
                {
                    pAuthor->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink3 = pLink3->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    level_--;
    (outf_ <<= level_) << ")" << endl;
} // Bibliography::Decompile

// ---------------------------------------------------------

// Save the Bibliography in the database.

Db::RecNumber Bibliography::Save(DbBlock * block_, Db::RecNumber prevRecno_)
{
    Db::RecNumber recno_;
    DbBibliography dbBibliography;

    BuildDbRec(block_, dbBibliography);
    dbBibliography.next_ = prevRecno_;
    recno_ = block_->AddDbRec(&dbBibliography, sizeof(dbBibliography),
        True, "Bibliography"
        );
    return recno_;
} // Bibliography::Save

// ---------------------------------------------------------

// Build Bibliography DB record.//gen.icn/2261
void Bibliography::BuildDbRec(DbBlock * block_, DbBibliography& dbBibliography)
{
    char * str_;
    memset(&dbBibliography, 0, sizeof(dbBibliography));
    if (id_ != NULL)
    {
        str_ = (char *)*id_;
        dbBibliography.id_ = block_->AddDbString(str_);
    } // if

    // Save each field in a Bibliography//gen.icn/2277
    // Save member: bibltitle//gen.icn/2322
    if (bibltitle != NULL)
    {
        str_ = (char *)*bibltitle;
        dbBibliography.bibltitle = block_->AddDbString(str_);
    } // if
    // Save member: documents//gen.icn/2353
    dbBibliography.documents = documents->Save(block_);
    // Save member: authors//gen.icn/2353
    dbBibliography.authors = authors->Save(block_);
} // Bibliography::BuildDbRec

// --------------------------------------------------------

void BibliographyList::Append(Bibliography * initData) {
    BibliographyLink * pLink = new BibliographyLink(initData);
    if (first == NULL)
    {
        first = pLink;
        last = pLink;
    } // if
    else
    {
        last->SetNext(pLink);
        last = pLink;
    } // else
} // BibliographyList::Append


// --------------------------------------------------------

// Insert item into list at position pos.
//  If pos > size of list, insert at end of list.
void BibliographyList::Insert(Bibliography * initData, int pos)
{
    BibliographyLink * pLink = new BibliographyLink(initData);
    if (first == NULL)
    {
        first = pLink;
        last = pLink;
    } // if
    else if (pos == 0)
    {
        pLink->SetNext(first);
        first = pLink;
    } // else if
    else
    {
        BibliographyLink * pL = first;
        int idx = 0;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                // Add it to the end.
                last->SetNext(pLink);
                last = pLink;
                break;
            } // if
            if (idx == pos)
            {
                // We are at the desired position. Add it.
                pLink->SetNext(pL->GetNext());
                pL->SetNext(pLink);
                break;
            } // if
            idx++;
            pL = pL->GetNext();
        } // for
    } // else
} // BibliographyList::Insert

// --------------------------------------------------------

// Delete an item from list at position pos.
//  If pos > size of list, return false; else return true.
boolean BibliographyList::Delete(int pos)
{
    if (first == NULL)
    {
        return false;
    } // if
    else if (pos == 0)
    {
        BibliographyLink * pLink = first;
        first = first->GetNext();
        Bibliography * pBibliography = pLink->GetData();
        delete pLink;
        delete pBibliography;
        if (first == NULL)
        {
            last = NULL;
        } // if
    } // else if
    else
    {
        BibliographyLink * pL = first->GetNext();
        BibliographyLink * pPrevLink = first;
        int idx = 1;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                return false;
            } // if
            if (idx == pos)
            {
                // We are at the desired position. Delete it.
                pPrevLink->SetNext(pL->GetNext());
                if (pL == last)
                {
                    last = pPrevLink;
                } // if
                Bibliography * pBibliography = pL->GetData();
                delete pL;
                delete pBibliography;
                break;
            } // if
            idx++;
            pPrevLink = pL;
            pL = pL->GetNext();
        } // for
    } // else
    return true;
} // BibliographyList::Delete

// --------------------------------------------------------

// Return the item at position pos (relative to 0).
Bibliography * BibliographyList::operator [](int pos)
{
    if (first == NULL)
    {
        return NULL;
    } // if
    else if (pos == 0)
    {
        return first->GetData();
    } // else if
    else
    {
        BibliographyLink * pL = first;
        int idx = 0;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                return NULL;
            } // if
            if (idx == pos)
            {
                // We are at the desired position.
                return pL->GetData();
            } // if
            idx++;
            pL = pL->GetNext();
        } // for
    } // else
} // BibliographyList::operator []

// --------------------------------------------------------

BibliographyList * BibliographyList::Copy()
{
    if (first == NULL)
    {
        return NULL;
    } // if
    BibliographyList * pList = new BibliographyList;
    BibliographyLink * pLink = first;
    while (pLink != NULL)
    {
        Bibliography * pData1 = pLink->GetData();
        if (pData1 != NULL)
        {
            Bibliography * pData2 = new Bibliography(*pData1);
            pList->Append(pData2);
        } // if
        pLink = pLink->GetNext();
    } // while
    return pList;
} // BibliographyList::Copy

// --------------------------------------------------------

// Load a list of records from the database.
//  Each record contains the record number of the next record.
BibliographyList * BibliographyList::Load(DbBlock * block_, Db::RecNumber firstRecno_)
{
    Db::RecNumber recno_ = firstRecno_;
    Db::RecNumber nextRecno_;

    if (recno_ == Db::DB_NULL_RECID)
    {
        return NULL;
    } // if
    BibliographyList * pNewList_ = new BibliographyList;
    while (recno_ != Db::DB_NULL_RECID)
    {
        Bibliography * pBibliography = Bibliography::Load(block_, recno_, nextRecno_);
        pNewList_->Insert(pBibliography, 0);
        recno_ = nextRecno_;
    } // while
    return pNewList_;
} // BibliographyList::Load

// --------------------------------------------------------

// Save a list of records into the database.
//  Each record contains the record number of the next record.
Db::RecNumber BibliographyList::Save(DbBlock * block_)
{
    Db::RecNumber recno_ = Db::DB_NULL_RECID;
    Db::RecNumber prevRecno_ = Db::DB_NULL_RECID;

    BibliographyLink * pLink_ = first;
    while (pLink_ != NULL)
    {
        Bibliography * pB_ = pLink_->GetData();
        if (pB_ != NULL)
        {
            recno_ = pB_->Save(block_, prevRecno_);
            prevRecno_ = recno_;
        } // if
        pLink_ = pLink_->GetNext();
    } // while
    return recno_;
} // BibliographyList::Save

// ---------------------------------------------------------

// Generate .IPF text for model: Bibliography//gen.icn/1068
void Bibliography::GenIpf(KrOstream & ofs, int level_)
{
    if (level_ == 0)
    {
        ofs << ":userdoc." << endl;
        ofs << ":docprof toc=123456." << endl;
        ofs << ":title." << "Bibliography information" << endl;
    } // if

    if (level_ > 6)
    {
        KrReportError("Nesting too deep", KR_ERROR_LEVEL_1);
    } // if
    ofs << ":h" << level_ + 1 << ".Bibliography window" << endl
            << ":p." << endl;
    AuthorLink * pLink = authors->GetFirst();
    if (pLink != NULL)
    {
        while (pLink != NULL)
        {
            Author * pData = pLink->GetData();
            if (pData != NULL)
            {
                pData->GenIpf(ofs, level_ + 1);
            } // if
            pLink = pLink->GetNext();
        } // while
    } // if
    if (level_ == 0)
    {
        ofs << ":euserdoc." << endl;
    } // if
} // Bibliography::GenIpf


// ---------------------------------------------------------
// ---------------------------------------------------------

// Member functions for: Document

// ---------------------------------------------------------

// Default constructor for class Document//gen.icn/938
Document::Document()
    :
    id_(NULL),
    selectedItem(false),
    doctitle(NULL),
    authorid(NULL),
    pubdata(NULL),
    keywords(new KrStringList()),
    xrefs(new KrStringList()),
    docnotes(new KrStringList()),
    docnotesTitle(NULL)
{
} // Document::Document

// ---------------------------------------------------------

// Copy constructor for class Document//gen.icn/959
Document::Document(const Document& rhs_)
    :
    id_(NULL),
    selectedItem(false),
    doctitle(NULL),
    authorid(NULL),
    pubdata(NULL),
    keywords(new KrStringList()),
    xrefs(new KrStringList()),
    docnotes(new KrStringList()),
    docnotesTitle(NULL)
{
    if (((char *)*rhs_.id_) != NULL)
    {
        id_ = new KrString(*rhs_.id_);
    } // if
    // Copy member: selectedItem//gen.icn/1011
    selectedItem = rhs_.selectedItem;
    // Copy member: doctitle//gen.icn/1016
    doctitle = new KrString(*rhs_.doctitle);
    // Copy member: authorid//gen.icn/1016
    authorid = new KrString(*rhs_.authorid);
    // Copy member: pubdata//gen.icn/1016
    pubdata = new Pubs(*rhs_.pubdata);
    // Copy member: keywords//gen.icn/1023
    keywords = rhs_.keywords->Copy();
    // Copy member: xrefs//gen.icn/1023
    xrefs = rhs_.xrefs->Copy();
    // Copy member: docnotes//gen.icn/1023
    docnotes = rhs_.docnotes->Copy();
    // Copy member: docnotesTitle//gen.icn/1016
    docnotesTitle = new KrString(*rhs_.docnotesTitle);
} // Document::Document

// ---------------------------------------------------------

// Constructor to create Document by loading from the database.//gen.icn/1316
Document * Document::Load(DbBlock * block_, Db::RecNumber recno_, Db::RecNumber & nextRecno_)
{
    Db::RecNumber convertedRecno_;

    if (recno_ == Db::DB_NULL_RECID)
    {
        nextRecno_ = Db::DB_NULL_RECID;
        return NULL;
    } // if
    if (DbBlock::IsDescriptor(recno_, convertedRecno_))
    {
        Db::DescriptorRec desc_;
        block_->GetRec(convertedRecno_, &desc_);
        char * type = block_->GetStr(desc_.className);
        recno_ = desc_.data;
        if (strcmp(type, "Document") == 0)
        {
            Document * pObj = Document::Load(block_, recno_, nextRecno_);
            return pObj;
        } // if
    } // if
    else
    {
        DbDocument dbDocument;
        block_->GetRec(recno_, &dbDocument);
        Document * pObj = new Document(block_, dbDocument);
        nextRecno_ = dbDocument.next_;
        return pObj;
    } // else
    return NULL;
} // Document::Load

// ---------------------------------------------------------

// Constructor to create Document by loading from the database.
Document::Document(DbBlock * block_, DbDocument & dbDocument_)
    :
    id_(NULL),
    selectedItem(false),
    doctitle(NULL),
    authorid(NULL),
    pubdata(NULL),
    keywords(new KrStringList()),
    xrefs(new KrStringList()),
    docnotes(new KrStringList()),
    docnotesTitle(NULL)
{
    // Convert each field.
    BuildRec(block_, dbDocument_);
} // Document::Document

// ---------------------------------------------------------

// Destructor for class Document//gen.icn/1608
Document::~Document()
{

    // Destruct member: selectedItem//gen.icn/1641
    // Destruct member: doctitle//gen.icn/1641
    delete doctitle;
    // Destruct member: authorid//gen.icn/1641
    delete authorid;
    // Destruct member: pubdata//gen.icn/1641
    delete pubdata;
    // Destruct member: keywords//gen.icn/1653
    delete keywords;
    // Destruct member: xrefs//gen.icn/1653
    delete xrefs;
    // Destruct member: docnotes//gen.icn/1653
    delete docnotes;
    // Destruct member: docnotesTitle//gen.icn/1641
    delete docnotesTitle;
} // Document::~Document

// ---------------------------------------------------------

// Build Document record from DB record.//gen.icn/1680
void Document::BuildRec(DbBlock * block_, DbDocument& dbDocument)
{
    Db::RecNumber recno_;
    if (dbDocument.id_ != Db::DB_NULL_RECID)
    {
        id_ = new KrString(block_->GetStr(dbDocument.id_));
    } // if
    // Convert each field in a Document//gen.icn/1695
    // Load member: selectedItem//gen.icn/1732
    selectedItem = dbDocument.selectedItem;
    // Load member: doctitle//gen.icn/1740
    if (dbDocument.doctitle != Db::DB_NULL_RECID)
    {
        doctitle = new KrString(block_->GetStr(dbDocument.doctitle));
    } // if
    // Load member: authorid//gen.icn/1740
    if (dbDocument.authorid != Db::DB_NULL_RECID)
    {
        authorid = new KrString(block_->GetStr(dbDocument.authorid));
    } // if
    // Load member: pubdata//gen.icn/1754
    if (dbDocument.pubdata != Db::DB_NULL_RECID)
    {
        pubdata = Pubs::Load(block_, dbDocument.pubdata, recno_);
    } // if
    // Load member: keywords//gen.icn/1770
    keywords = KrStringList::Load(block_, dbDocument.keywords);
    // Load member: xrefs//gen.icn/1770
    xrefs = KrStringList::Load(block_, dbDocument.xrefs);
    // Load member: docnotes//gen.icn/1770
    docnotes = KrStringList::Load(block_, dbDocument.docnotes);
    // Load member: docnotesTitle//gen.icn/1740
    if (dbDocument.docnotesTitle != Db::DB_NULL_RECID)
    {
        docnotesTitle = new KrString(block_->GetStr(dbDocument.docnotesTitle));
    } // if
} // Document::BuildRec

// ---------------------------------------------------------

// Assignment operator for class Document//gen.icn/1971
Document& Document::operator=(const Document& rhs_)
{
    if (this == &rhs_)
    {
        return *this;
    } // if
    // Copy member: selectedItem//gen.icn/1011
    selectedItem = rhs_.selectedItem;
    // Copy member: doctitle//gen.icn/1016
    doctitle = new KrString(*rhs_.doctitle);
    // Copy member: authorid//gen.icn/1016
    authorid = new KrString(*rhs_.authorid);
    // Copy member: pubdata//gen.icn/1016
    pubdata = new Pubs(*rhs_.pubdata);
    // Copy member: keywords//gen.icn/1023
    keywords = rhs_.keywords->Copy();
    // Copy member: xrefs//gen.icn/1023
    xrefs = rhs_.xrefs->Copy();
    // Copy member: docnotes//gen.icn/1023
    docnotes = rhs_.docnotes->Copy();
    // Copy member: docnotesTitle//gen.icn/1016
    docnotesTitle = new KrString(*rhs_.docnotesTitle);
    return *this;
} // Document::operator=

// ---------------------------------------------------------

// Decompile member function for class Document//gen.icn/2008
void Document::Decompile(KrOstream& outf_, int level_, int & count_)
{
    char buf[1024];

    // Decompile object ID//gen.icn/2021
    if (id_ == NULL)
    {
        count_++;
        (outf_ <<= level_) << "(Document document" << count_ << endl;
    } // if
    else
    {
        (outf_ <<= level_) << "(Document " << ((char *)*id_) << endl;
    } // else
    level_++;
    // Decompile member: selectedItem//gen.icn/2099
    (outf_ <<= level_) << "(selectedItem ";
    if (selectedItem)
    {
        outf_ << "true";
    } // if
    else
    {
        outf_ << "false";
    } // else
    outf_ << ")" << endl;
    // Decompile member: doctitle//gen.icn/2141
    if (doctitle != NULL)
    {
        doctitle->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(doctitle \"" << buf << "\")" << endl;
    } // if
    // Decompile member: authorid//gen.icn/2141
    if (authorid != NULL)
    {
        authorid->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(authorid \"" << buf << "\")" << endl;
    } // if
    // Decompile member: pubdata//gen.icn/2154
    if (pubdata != NULL)
    {
        pubdata->Decompile(outf_, level_, count_);
    } // if
    // Decompile member: keywords//gen.icn/2182
    KrStringList * pList5 = keywords;
    if (pList5 != NULL)
    {
        KrStringLink * pLink5 = pList5->GetFirst();
        if (pLink5 != NULL)
        {
            (outf_ <<= level_) << "(keywords" << endl;
            while (pLink5 != NULL)
            {
                KrString * pKrString = pLink5->GetData();
                if (pKrString != NULL)
                {
                    pKrString->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink5 = pLink5->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    // Decompile member: xrefs//gen.icn/2182
    KrStringList * pList6 = xrefs;
    if (pList6 != NULL)
    {
        KrStringLink * pLink6 = pList6->GetFirst();
        if (pLink6 != NULL)
        {
            (outf_ <<= level_) << "(xrefs" << endl;
            while (pLink6 != NULL)
            {
                KrString * pKrString = pLink6->GetData();
                if (pKrString != NULL)
                {
                    pKrString->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink6 = pLink6->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    // Decompile member: docnotes//gen.icn/2182
    KrStringList * pList7 = docnotes;
    if (pList7 != NULL)
    {
        KrStringLink * pLink7 = pList7->GetFirst();
        if (pLink7 != NULL)
        {
            (outf_ <<= level_) << "(docnotes" << endl;
            while (pLink7 != NULL)
            {
                KrString * pKrString = pLink7->GetData();
                if (pKrString != NULL)
                {
                    pKrString->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink7 = pLink7->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    // Decompile member: docnotesTitle//gen.icn/2141
    if (docnotesTitle != NULL)
    {
        docnotesTitle->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(docnotesTitle \"" << buf << "\")" << endl;
    } // if
    level_--;
    (outf_ <<= level_) << ")" << endl;
} // Document::Decompile

// ---------------------------------------------------------

// Save the Document in the database.

Db::RecNumber Document::Save(DbBlock * block_, Db::RecNumber prevRecno_)
{
    Db::RecNumber recno_;
    DbDocument dbDocument;

    BuildDbRec(block_, dbDocument);
    dbDocument.next_ = prevRecno_;
    recno_ = block_->AddDbRec(&dbDocument, sizeof(dbDocument),
        True, "Document"
        );
    return recno_;
} // Document::Save

// ---------------------------------------------------------

// Build Document DB record.//gen.icn/2261
void Document::BuildDbRec(DbBlock * block_, DbDocument& dbDocument)
{
    char * str_;
    memset(&dbDocument, 0, sizeof(dbDocument));
    if (id_ != NULL)
    {
        str_ = (char *)*id_;
        dbDocument.id_ = block_->AddDbString(str_);
    } // if

    // Save each field in a Document//gen.icn/2277
    // Save member: selectedItem//gen.icn/2314
    dbDocument.selectedItem = selectedItem;
    // Save member: doctitle//gen.icn/2322
    if (doctitle != NULL)
    {
        str_ = (char *)*doctitle;
        dbDocument.doctitle = block_->AddDbString(str_);
    } // if
    // Save member: authorid//gen.icn/2322
    if (authorid != NULL)
    {
        str_ = (char *)*authorid;
        dbDocument.authorid = block_->AddDbString(str_);
    } // if
    // Save member: pubdata//gen.icn/2337
    if (pubdata != NULL)
    {
        dbDocument.pubdata = pubdata->Save(block_, Db::DB_NULL_RECID);
    } // if
    // Save member: keywords//gen.icn/2353
    dbDocument.keywords = keywords->Save(block_);
    // Save member: xrefs//gen.icn/2353
    dbDocument.xrefs = xrefs->Save(block_);
    // Save member: docnotes//gen.icn/2353
    dbDocument.docnotes = docnotes->Save(block_);
    // Save member: docnotesTitle//gen.icn/2322
    if (docnotesTitle != NULL)
    {
        str_ = (char *)*docnotesTitle;
        dbDocument.docnotesTitle = block_->AddDbString(str_);
    } // if
} // Document::BuildDbRec

// --------------------------------------------------------

void DocumentList::Append(Document * initData) {
    DocumentLink * pLink = new DocumentLink(initData);
    if (first == NULL)
    {
        first = pLink;
        last = pLink;
    } // if
    else
    {
        last->SetNext(pLink);
        last = pLink;
    } // else
} // DocumentList::Append


// --------------------------------------------------------

// Insert item into list at position pos.
//  If pos > size of list, insert at end of list.
void DocumentList::Insert(Document * initData, int pos)
{
    DocumentLink * pLink = new DocumentLink(initData);
    if (first == NULL)
    {
        first = pLink;
        last = pLink;
    } // if
    else if (pos == 0)
    {
        pLink->SetNext(first);
        first = pLink;
    } // else if
    else
    {
        DocumentLink * pL = first;
        int idx = 0;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                // Add it to the end.
                last->SetNext(pLink);
                last = pLink;
                break;
            } // if
            if (idx == pos)
            {
                // We are at the desired position. Add it.
                pLink->SetNext(pL->GetNext());
                pL->SetNext(pLink);
                break;
            } // if
            idx++;
            pL = pL->GetNext();
        } // for
    } // else
} // DocumentList::Insert

// --------------------------------------------------------

// Delete an item from list at position pos.
//  If pos > size of list, return false; else return true.
boolean DocumentList::Delete(int pos)
{
    if (first == NULL)
    {
        return false;
    } // if
    else if (pos == 0)
    {
        DocumentLink * pLink = first;
        first = first->GetNext();
        Document * pDocument = pLink->GetData();
        delete pLink;
        delete pDocument;
        if (first == NULL)
        {
            last = NULL;
        } // if
    } // else if
    else
    {
        DocumentLink * pL = first->GetNext();
        DocumentLink * pPrevLink = first;
        int idx = 1;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                return false;
            } // if
            if (idx == pos)
            {
                // We are at the desired position. Delete it.
                pPrevLink->SetNext(pL->GetNext());
                if (pL == last)
                {
                    last = pPrevLink;
                } // if
                Document * pDocument = pL->GetData();
                delete pL;
                delete pDocument;
                break;
            } // if
            idx++;
            pPrevLink = pL;
            pL = pL->GetNext();
        } // for
    } // else
    return true;
} // DocumentList::Delete

// --------------------------------------------------------

// Return the item at position pos (relative to 0).
Document * DocumentList::operator [](int pos)
{
    if (first == NULL)
    {
        return NULL;
    } // if
    else if (pos == 0)
    {
        return first->GetData();
    } // else if
    else
    {
        DocumentLink * pL = first;
        int idx = 0;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                return NULL;
            } // if
            if (idx == pos)
            {
                // We are at the desired position.
                return pL->GetData();
            } // if
            idx++;
            pL = pL->GetNext();
        } // for
    } // else
} // DocumentList::operator []

// --------------------------------------------------------

DocumentList * DocumentList::Copy()
{
    if (first == NULL)
    {
        return NULL;
    } // if
    DocumentList * pList = new DocumentList;
    DocumentLink * pLink = first;
    while (pLink != NULL)
    {
        Document * pData1 = pLink->GetData();
        if (pData1 != NULL)
        {
            Document * pData2 = new Document(*pData1);
            pList->Append(pData2);
        } // if
        pLink = pLink->GetNext();
    } // while
    return pList;
} // DocumentList::Copy

// --------------------------------------------------------

// Load a list of records from the database.
//  Each record contains the record number of the next record.
DocumentList * DocumentList::Load(DbBlock * block_, Db::RecNumber firstRecno_)
{
    Db::RecNumber recno_ = firstRecno_;
    Db::RecNumber nextRecno_;

    if (recno_ == Db::DB_NULL_RECID)
    {
        return NULL;
    } // if
    DocumentList * pNewList_ = new DocumentList;
    while (recno_ != Db::DB_NULL_RECID)
    {
        Document * pDocument = Document::Load(block_, recno_, nextRecno_);
        pNewList_->Insert(pDocument, 0);
        recno_ = nextRecno_;
    } // while
    return pNewList_;
} // DocumentList::Load

// --------------------------------------------------------

// Save a list of records into the database.
//  Each record contains the record number of the next record.
Db::RecNumber DocumentList::Save(DbBlock * block_)
{
    Db::RecNumber recno_ = Db::DB_NULL_RECID;
    Db::RecNumber prevRecno_ = Db::DB_NULL_RECID;

    DocumentLink * pLink_ = first;
    while (pLink_ != NULL)
    {
        Document * pB_ = pLink_->GetData();
        if (pB_ != NULL)
        {
            recno_ = pB_->Save(block_, prevRecno_);
            prevRecno_ = recno_;
        } // if
        pLink_ = pLink_->GetNext();
    } // while
    return recno_;
} // DocumentList::Save

// ---------------------------------------------------------

// Generate .IPF text for node: Document//gen.icn/1184
void Document::GenIpf(KrOstream & ofs, int level_)
{
    if (level_ == 0)
    {
        ofs << ":userdoc." << endl;
        ofs << ":docprof toc=123456." << endl;
        ofs << ":title." << "Document window" << endl;
    } // if

    if (level_ > 6)
    {
        KrReportError("Nesting too deep", KR_ERROR_LEVEL_1);
    } // if
    ofs << ":h" << level_ + 1 << ".Document information window" << endl
            << ":p." << endl;
    if (level_ == 0)
    {
        ofs << ":euserdoc." << endl;
    } // if
} // Document::GenIpf


// ---------------------------------------------------------
// ---------------------------------------------------------

// Member functions for: Author

// ---------------------------------------------------------

// Default constructor for class Author//gen.icn/938
Author::Author()
    :
    id_(NULL),
    name(NULL),
    publishedWorks(new DocumentList()),
    worksInProgress(new DocumentList()),
    authNotes(new KrStringList()),
    authnotesTitle(NULL)
{
} // Author::Author

// ---------------------------------------------------------

// Copy constructor for class Author//gen.icn/959
Author::Author(const Author& rhs_)
    :
    id_(NULL),
    name(NULL),
    publishedWorks(new DocumentList()),
    worksInProgress(new DocumentList()),
    authNotes(new KrStringList()),
    authnotesTitle(NULL)
{
    if (((char *)*rhs_.id_) != NULL)
    {
        id_ = new KrString(*rhs_.id_);
    } // if
    // Copy member: name//gen.icn/1016
    name = new KrString(*rhs_.name);
    // Copy member: publishedWorks//gen.icn/1023
    publishedWorks = rhs_.publishedWorks->Copy();
    // Copy member: worksInProgress//gen.icn/1023
    worksInProgress = rhs_.worksInProgress->Copy();
    // Copy member: authNotes//gen.icn/1023
    authNotes = rhs_.authNotes->Copy();
    // Copy member: authnotesTitle//gen.icn/1016
    authnotesTitle = new KrString(*rhs_.authnotesTitle);
} // Author::Author

// ---------------------------------------------------------

// Constructor to create Author by loading from the database.//gen.icn/1316
Author * Author::Load(DbBlock * block_, Db::RecNumber recno_, Db::RecNumber & nextRecno_)
{
    Db::RecNumber convertedRecno_;

    if (recno_ == Db::DB_NULL_RECID)
    {
        nextRecno_ = Db::DB_NULL_RECID;
        return NULL;
    } // if
    if (DbBlock::IsDescriptor(recno_, convertedRecno_))
    {
        Db::DescriptorRec desc_;
        block_->GetRec(convertedRecno_, &desc_);
        char * type = block_->GetStr(desc_.className);
        recno_ = desc_.data;
        if (strcmp(type, "HumorFictionAuthor") == 0)
        {
            HumorFictionAuthor * pObj = HumorFictionAuthor::Load(block_, recno_, nextRecno_);
            return pObj;
        } // if
        else if (strcmp(type, "NonfictionAuthor") == 0)
        {
            NonfictionAuthor * pObj = NonfictionAuthor::Load(block_, recno_, nextRecno_);
            return pObj;
        } // else if
        else if (strcmp(type, "FictionAuthor") == 0)
        {
            FictionAuthor * pObj = FictionAuthor::Load(block_, recno_, nextRecno_);
            return pObj;
        } // else if
        else if (strcmp(type, "RomanticFictionAuthor") == 0)
        {
            RomanticFictionAuthor * pObj = RomanticFictionAuthor::Load(block_, recno_, nextRecno_);
            return pObj;
        } // else if
        else if (strcmp(type, "Author") == 0)
        {
            Author * pObj = Author::Load(block_, recno_, nextRecno_);
            return pObj;
        } // else if
    } // if
    else
    {
        DbAuthor dbAuthor;
        block_->GetRec(recno_, &dbAuthor);
        Author * pObj = new Author(block_, dbAuthor);
        nextRecno_ = dbAuthor.next_;
        return pObj;
    } // else
    return NULL;
} // Author::Load

// ---------------------------------------------------------

// Constructor to create Author by loading from the database.
Author::Author(DbBlock * block_, DbAuthor & dbAuthor_)
    :
    id_(NULL),
    name(NULL),
    publishedWorks(new DocumentList()),
    worksInProgress(new DocumentList()),
    authNotes(new KrStringList()),
    authnotesTitle(NULL)
{
    // Convert each field.
    BuildRec(block_, dbAuthor_);
} // Author::Author

// ---------------------------------------------------------

// Destructor for class Author//gen.icn/1608
Author::~Author()
{

    // Destruct member: name//gen.icn/1641
    delete name;
    // Destruct member: publishedWorks//gen.icn/1653
    delete publishedWorks;
    // Destruct member: worksInProgress//gen.icn/1653
    delete worksInProgress;
    // Destruct member: authNotes//gen.icn/1653
    delete authNotes;
    // Destruct member: authnotesTitle//gen.icn/1641
    delete authnotesTitle;
} // Author::~Author

// ---------------------------------------------------------

// Build Author record from DB record.//gen.icn/1680
void Author::BuildRec(DbBlock * block_, DbAuthor& dbAuthor)
{
    Db::RecNumber recno_;
    if (dbAuthor.id_ != Db::DB_NULL_RECID)
    {
        id_ = new KrString(block_->GetStr(dbAuthor.id_));
    } // if
    // Convert each field in a Author//gen.icn/1695
    // Load member: name//gen.icn/1740
    if (dbAuthor.name != Db::DB_NULL_RECID)
    {
        name = new KrString(block_->GetStr(dbAuthor.name));
    } // if
    // Load member: publishedWorks//gen.icn/1770
    publishedWorks = DocumentList::Load(block_, dbAuthor.publishedWorks);
    // Load member: worksInProgress//gen.icn/1770
    worksInProgress = DocumentList::Load(block_, dbAuthor.worksInProgress);
    // Load member: authNotes//gen.icn/1770
    authNotes = KrStringList::Load(block_, dbAuthor.authNotes);
    // Load member: authnotesTitle//gen.icn/1740
    if (dbAuthor.authnotesTitle != Db::DB_NULL_RECID)
    {
        authnotesTitle = new KrString(block_->GetStr(dbAuthor.authnotesTitle));
    } // if
} // Author::BuildRec

// ---------------------------------------------------------

// Assignment operator for class Author//gen.icn/1971
Author& Author::operator=(const Author& rhs_)
{
    if (this == &rhs_)
    {
        return *this;
    } // if
    // Copy member: name//gen.icn/1016
    name = new KrString(*rhs_.name);
    // Copy member: publishedWorks//gen.icn/1023
    publishedWorks = rhs_.publishedWorks->Copy();
    // Copy member: worksInProgress//gen.icn/1023
    worksInProgress = rhs_.worksInProgress->Copy();
    // Copy member: authNotes//gen.icn/1023
    authNotes = rhs_.authNotes->Copy();
    // Copy member: authnotesTitle//gen.icn/1016
    authnotesTitle = new KrString(*rhs_.authnotesTitle);
    return *this;
} // Author::operator=

// ---------------------------------------------------------

// Decompile member function for class Author//gen.icn/2008
void Author::Decompile(KrOstream& outf_, int level_, int & count_)
{
    char buf[1024];

    // Decompile object ID//gen.icn/2021
    if (id_ == NULL)
    {
        count_++;
        (outf_ <<= level_) << "(Author author" << count_ << endl;
    } // if
    else
    {
        (outf_ <<= level_) << "(Author " << ((char *)*id_) << endl;
    } // else
    level_++;
    // Decompile member: name//gen.icn/2141
    if (name != NULL)
    {
        name->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(name \"" << buf << "\")" << endl;
    } // if
    // Decompile member: publishedWorks//gen.icn/2182
    DocumentList * pList2 = publishedWorks;
    if (pList2 != NULL)
    {
        DocumentLink * pLink2 = pList2->GetFirst();
        if (pLink2 != NULL)
        {
            (outf_ <<= level_) << "(publishedWorks" << endl;
            while (pLink2 != NULL)
            {
                Document * pDocument = pLink2->GetData();
                if (pDocument != NULL)
                {
                    pDocument->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink2 = pLink2->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    // Decompile member: worksInProgress//gen.icn/2182
    DocumentList * pList3 = worksInProgress;
    if (pList3 != NULL)
    {
        DocumentLink * pLink3 = pList3->GetFirst();
        if (pLink3 != NULL)
        {
            (outf_ <<= level_) << "(worksInProgress" << endl;
            while (pLink3 != NULL)
            {
                Document * pDocument = pLink3->GetData();
                if (pDocument != NULL)
                {
                    pDocument->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink3 = pLink3->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    // Decompile member: authNotes//gen.icn/2182
    KrStringList * pList4 = authNotes;
    if (pList4 != NULL)
    {
        KrStringLink * pLink4 = pList4->GetFirst();
        if (pLink4 != NULL)
        {
            (outf_ <<= level_) << "(authNotes" << endl;
            while (pLink4 != NULL)
            {
                KrString * pKrString = pLink4->GetData();
                if (pKrString != NULL)
                {
                    pKrString->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink4 = pLink4->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    // Decompile member: authnotesTitle//gen.icn/2141
    if (authnotesTitle != NULL)
    {
        authnotesTitle->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(authnotesTitle \"" << buf << "\")" << endl;
    } // if
    level_--;
    (outf_ <<= level_) << ")" << endl;
} // Author::Decompile

// ---------------------------------------------------------

// Save the Author in the database.

Db::RecNumber Author::Save(DbBlock * block_, Db::RecNumber prevRecno_)
{
    Db::RecNumber recno_;
    DbAuthor dbAuthor;

    BuildDbRec(block_, dbAuthor);
    dbAuthor.next_ = prevRecno_;
    recno_ = block_->AddDbRec(&dbAuthor, sizeof(dbAuthor),
        True, "Author"
        );
    return recno_;
} // Author::Save

// ---------------------------------------------------------

// Build Author DB record.//gen.icn/2261
void Author::BuildDbRec(DbBlock * block_, DbAuthor& dbAuthor)
{
    char * str_;
    memset(&dbAuthor, 0, sizeof(dbAuthor));
    if (id_ != NULL)
    {
        str_ = (char *)*id_;
        dbAuthor.id_ = block_->AddDbString(str_);
    } // if

    // Save each field in a Author//gen.icn/2277
    // Save member: name//gen.icn/2322
    if (name != NULL)
    {
        str_ = (char *)*name;
        dbAuthor.name = block_->AddDbString(str_);
    } // if
    // Save member: publishedWorks//gen.icn/2353
    dbAuthor.publishedWorks = publishedWorks->Save(block_);
    // Save member: worksInProgress//gen.icn/2353
    dbAuthor.worksInProgress = worksInProgress->Save(block_);
    // Save member: authNotes//gen.icn/2353
    dbAuthor.authNotes = authNotes->Save(block_);
    // Save member: authnotesTitle//gen.icn/2322
    if (authnotesTitle != NULL)
    {
        str_ = (char *)*authnotesTitle;
        dbAuthor.authnotesTitle = block_->AddDbString(str_);
    } // if
} // Author::BuildDbRec

// --------------------------------------------------------

void AuthorList::Append(Author * initData) {
    AuthorLink * pLink = new AuthorLink(initData);
    if (first == NULL)
    {
        first = pLink;
        last = pLink;
    } // if
    else
    {
        last->SetNext(pLink);
        last = pLink;
    } // else
} // AuthorList::Append


// --------------------------------------------------------

// Insert item into list at position pos.
//  If pos > size of list, insert at end of list.
void AuthorList::Insert(Author * initData, int pos)
{
    AuthorLink * pLink = new AuthorLink(initData);
    if (first == NULL)
    {
        first = pLink;
        last = pLink;
    } // if
    else if (pos == 0)
    {
        pLink->SetNext(first);
        first = pLink;
    } // else if
    else
    {
        AuthorLink * pL = first;
        int idx = 0;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                // Add it to the end.
                last->SetNext(pLink);
                last = pLink;
                break;
            } // if
            if (idx == pos)
            {
                // We are at the desired position. Add it.
                pLink->SetNext(pL->GetNext());
                pL->SetNext(pLink);
                break;
            } // if
            idx++;
            pL = pL->GetNext();
        } // for
    } // else
} // AuthorList::Insert

// --------------------------------------------------------

// Delete an item from list at position pos.
//  If pos > size of list, return false; else return true.
boolean AuthorList::Delete(int pos)
{
    if (first == NULL)
    {
        return false;
    } // if
    else if (pos == 0)
    {
        AuthorLink * pLink = first;
        first = first->GetNext();
        Author * pAuthor = pLink->GetData();
        delete pLink;
        delete pAuthor;
        if (first == NULL)
        {
            last = NULL;
        } // if
    } // else if
    else
    {
        AuthorLink * pL = first->GetNext();
        AuthorLink * pPrevLink = first;
        int idx = 1;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                return false;
            } // if
            if (idx == pos)
            {
                // We are at the desired position. Delete it.
                pPrevLink->SetNext(pL->GetNext());
                if (pL == last)
                {
                    last = pPrevLink;
                } // if
                Author * pAuthor = pL->GetData();
                delete pL;
                delete pAuthor;
                break;
            } // if
            idx++;
            pPrevLink = pL;
            pL = pL->GetNext();
        } // for
    } // else
    return true;
} // AuthorList::Delete

// --------------------------------------------------------

// Return the item at position pos (relative to 0).
Author * AuthorList::operator [](int pos)
{
    if (first == NULL)
    {
        return NULL;
    } // if
    else if (pos == 0)
    {
        return first->GetData();
    } // else if
    else
    {
        AuthorLink * pL = first;
        int idx = 0;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                return NULL;
            } // if
            if (idx == pos)
            {
                // We are at the desired position.
                return pL->GetData();
            } // if
            idx++;
            pL = pL->GetNext();
        } // for
    } // else
} // AuthorList::operator []

// --------------------------------------------------------

AuthorList * AuthorList::Copy()
{
    if (first == NULL)
    {
        return NULL;
    } // if
    AuthorList * pList = new AuthorList;
    AuthorLink * pLink = first;
    while (pLink != NULL)
    {
        Author * pData1 = pLink->GetData();
        if (pData1 != NULL)
        {
            Author * pData2 = new Author(*pData1);
            pList->Append(pData2);
        } // if
        pLink = pLink->GetNext();
    } // while
    return pList;
} // AuthorList::Copy

// --------------------------------------------------------

// Load a list of records from the database.
//  Each record contains the record number of the next record.
AuthorList * AuthorList::Load(DbBlock * block_, Db::RecNumber firstRecno_)
{
    Db::RecNumber recno_ = firstRecno_;
    Db::RecNumber nextRecno_;

    if (recno_ == Db::DB_NULL_RECID)
    {
        return NULL;
    } // if
    AuthorList * pNewList_ = new AuthorList;
    while (recno_ != Db::DB_NULL_RECID)
    {
        Author * pAuthor = Author::Load(block_, recno_, nextRecno_);
        pNewList_->Insert(pAuthor, 0);
        recno_ = nextRecno_;
    } // while
    return pNewList_;
} // AuthorList::Load

// --------------------------------------------------------

// Save a list of records into the database.
//  Each record contains the record number of the next record.
Db::RecNumber AuthorList::Save(DbBlock * block_)
{
    Db::RecNumber recno_ = Db::DB_NULL_RECID;
    Db::RecNumber prevRecno_ = Db::DB_NULL_RECID;

    AuthorLink * pLink_ = first;
    while (pLink_ != NULL)
    {
        Author * pB_ = pLink_->GetData();
        if (pB_ != NULL)
        {
            recno_ = pB_->Save(block_, prevRecno_);
            prevRecno_ = recno_;
        } // if
        pLink_ = pLink_->GetNext();
    } // while
    return recno_;
} // AuthorList::Save

// ---------------------------------------------------------

// Generate .IPF text for node: Author//gen.icn/1184
void Author::GenIpf(KrOstream & ofs, int level_)
{
    if (level_ == 0)
    {
        ofs << ":userdoc." << endl;
        ofs << ":docprof toc=123456." << endl;
        ofs << ":title." << "Author window" << endl;
    } // if

    if (level_ > 6)
    {
        KrReportError("Nesting too deep", KR_ERROR_LEVEL_1);
    } // if
    ofs << ":h" << level_ + 1 << ".Author information window" << endl
            << ":p." << endl;
    if (level_ == 0)
    {
        ofs << ":euserdoc." << endl;
    } // if
} // Author::GenIpf


// ---------------------------------------------------------
// ---------------------------------------------------------

// Member functions for: FictionAuthor

// ---------------------------------------------------------

// Default constructor for class FictionAuthor//gen.icn/938
FictionAuthor::FictionAuthor()
    :
    Author(),
    topicArea(NULL)
{
} // FictionAuthor::FictionAuthor

// ---------------------------------------------------------

// Copy constructor for class FictionAuthor//gen.icn/959
FictionAuthor::FictionAuthor(const FictionAuthor& rhs_)
    :
    Author(),
    topicArea(NULL)
{
    if (((char *)*rhs_.id_) != NULL)
    {
        id_ = new KrString(*rhs_.id_);
    } // if
    // Copy member: topicArea//gen.icn/1016
    topicArea = new KrString(*rhs_.topicArea);
    // Copy fields in superclass Author//gen.icn/981
    // Copy member: name//gen.icn/1016
    name = new KrString(*rhs_.name);
    // Copy member: publishedWorks//gen.icn/1023
    publishedWorks = rhs_.publishedWorks->Copy();
    // Copy member: worksInProgress//gen.icn/1023
    worksInProgress = rhs_.worksInProgress->Copy();
    // Copy member: authNotes//gen.icn/1023
    authNotes = rhs_.authNotes->Copy();
    // Copy member: authnotesTitle//gen.icn/1016
    authnotesTitle = new KrString(*rhs_.authnotesTitle);
} // FictionAuthor::FictionAuthor

// ---------------------------------------------------------

// Constructor to create FictionAuthor by loading from the database.//gen.icn/1316
FictionAuthor * FictionAuthor::Load(DbBlock * block_, Db::RecNumber recno_, Db::RecNumber & nextRecno_)
{
    Db::RecNumber convertedRecno_;

    if (recno_ == Db::DB_NULL_RECID)
    {
        nextRecno_ = Db::DB_NULL_RECID;
        return NULL;
    } // if
    if (DbBlock::IsDescriptor(recno_, convertedRecno_))
    {
        Db::DescriptorRec desc_;
        block_->GetRec(convertedRecno_, &desc_);
        char * type = block_->GetStr(desc_.className);
        recno_ = desc_.data;
        if (strcmp(type, "HumorFictionAuthor") == 0)
        {
            HumorFictionAuthor * pObj = HumorFictionAuthor::Load(block_, recno_, nextRecno_);
            return pObj;
        } // if
        else if (strcmp(type, "FictionAuthor") == 0)
        {
            FictionAuthor * pObj = FictionAuthor::Load(block_, recno_, nextRecno_);
            return pObj;
        } // else if
        else if (strcmp(type, "RomanticFictionAuthor") == 0)
        {
            RomanticFictionAuthor * pObj = RomanticFictionAuthor::Load(block_, recno_, nextRecno_);
            return pObj;
        } // else if
    } // if
    else
    {
        DbFictionAuthor dbFictionAuthor;
        block_->GetRec(recno_, &dbFictionAuthor);
        FictionAuthor * pObj = new FictionAuthor(block_, dbFictionAuthor);
        nextRecno_ = dbFictionAuthor.next_;
        return pObj;
    } // else
    return NULL;
} // FictionAuthor::Load

// ---------------------------------------------------------

// Constructor to create FictionAuthor by loading from the database.
FictionAuthor::FictionAuthor(DbBlock * block_, DbFictionAuthor & dbFictionAuthor_)
    :
    Author(),
    topicArea(NULL)
{
    // Convert each field.
    BuildRec(block_, dbFictionAuthor_);
} // FictionAuthor::FictionAuthor

// ---------------------------------------------------------

// Destructor for class FictionAuthor//gen.icn/1608
FictionAuthor::~FictionAuthor()
{

    // Destruct member: topicArea//gen.icn/1641
    delete topicArea;
    // Destruct member: name//gen.icn/1641
    delete name;
    // Destruct member: publishedWorks//gen.icn/1653
    delete publishedWorks;
    // Destruct member: worksInProgress//gen.icn/1653
    delete worksInProgress;
    // Destruct member: authNotes//gen.icn/1653
    delete authNotes;
    // Destruct member: authnotesTitle//gen.icn/1641
    delete authnotesTitle;
} // FictionAuthor::~FictionAuthor

// ---------------------------------------------------------

// Build FictionAuthor record from DB record.//gen.icn/1680
void FictionAuthor::BuildRec(DbBlock * block_, DbFictionAuthor& dbFictionAuthor)
{
    Db::RecNumber recno_;
    if (dbFictionAuthor.id_ != Db::DB_NULL_RECID)
    {
        id_ = new KrString(block_->GetStr(dbFictionAuthor.id_));
    } // if
    // Convert each field in a FictionAuthor//gen.icn/1695
    // Load member: topicArea//gen.icn/1740
    if (dbFictionAuthor.topicArea != Db::DB_NULL_RECID)
    {
        topicArea = new KrString(block_->GetStr(dbFictionAuthor.topicArea));
    } // if
    // Load member: name//gen.icn/1740
    if (dbFictionAuthor.name != Db::DB_NULL_RECID)
    {
        name = new KrString(block_->GetStr(dbFictionAuthor.name));
    } // if
    // Load member: publishedWorks//gen.icn/1770
    publishedWorks = DocumentList::Load(block_, dbFictionAuthor.publishedWorks);
    // Load member: worksInProgress//gen.icn/1770
    worksInProgress = DocumentList::Load(block_, dbFictionAuthor.worksInProgress);
    // Load member: authNotes//gen.icn/1770
    authNotes = KrStringList::Load(block_, dbFictionAuthor.authNotes);
    // Load member: authnotesTitle//gen.icn/1740
    if (dbFictionAuthor.authnotesTitle != Db::DB_NULL_RECID)
    {
        authnotesTitle = new KrString(block_->GetStr(dbFictionAuthor.authnotesTitle));
    } // if
} // FictionAuthor::BuildRec

// ---------------------------------------------------------

// Assignment operator for class FictionAuthor//gen.icn/1971
FictionAuthor& FictionAuthor::operator=(const FictionAuthor& rhs_)
{
    if (this == &rhs_)
    {
        return *this;
    } // if
    // Copy member: topicArea//gen.icn/1016
    topicArea = new KrString(*rhs_.topicArea);
    // Copy member: name//gen.icn/1016
    name = new KrString(*rhs_.name);
    // Copy member: publishedWorks//gen.icn/1023
    publishedWorks = rhs_.publishedWorks->Copy();
    // Copy member: worksInProgress//gen.icn/1023
    worksInProgress = rhs_.worksInProgress->Copy();
    // Copy member: authNotes//gen.icn/1023
    authNotes = rhs_.authNotes->Copy();
    // Copy member: authnotesTitle//gen.icn/1016
    authnotesTitle = new KrString(*rhs_.authnotesTitle);
    return *this;
} // FictionAuthor::operator=

// ---------------------------------------------------------

// Decompile member function for class FictionAuthor//gen.icn/2008
void FictionAuthor::Decompile(KrOstream& outf_, int level_, int & count_)
{
    char buf[1024];

    // Decompile object ID//gen.icn/2021
    if (id_ == NULL)
    {
        count_++;
        (outf_ <<= level_) << "(FictionAuthor fictionauthor" << count_ << endl;
    } // if
    else
    {
        (outf_ <<= level_) << "(FictionAuthor " << ((char *)*id_) << endl;
    } // else
    level_++;
    // Decompile member: topicArea//gen.icn/2141
    if (topicArea != NULL)
    {
        topicArea->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(topicArea \"" << buf << "\")" << endl;
    } // if
    // Decompile member: name//gen.icn/2141
    if (name != NULL)
    {
        name->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(name \"" << buf << "\")" << endl;
    } // if
    // Decompile member: publishedWorks//gen.icn/2182
    DocumentList * pList3 = publishedWorks;
    if (pList3 != NULL)
    {
        DocumentLink * pLink3 = pList3->GetFirst();
        if (pLink3 != NULL)
        {
            (outf_ <<= level_) << "(publishedWorks" << endl;
            while (pLink3 != NULL)
            {
                Document * pDocument = pLink3->GetData();
                if (pDocument != NULL)
                {
                    pDocument->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink3 = pLink3->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    // Decompile member: worksInProgress//gen.icn/2182
    DocumentList * pList4 = worksInProgress;
    if (pList4 != NULL)
    {
        DocumentLink * pLink4 = pList4->GetFirst();
        if (pLink4 != NULL)
        {
            (outf_ <<= level_) << "(worksInProgress" << endl;
            while (pLink4 != NULL)
            {
                Document * pDocument = pLink4->GetData();
                if (pDocument != NULL)
                {
                    pDocument->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink4 = pLink4->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    // Decompile member: authNotes//gen.icn/2182
    KrStringList * pList5 = authNotes;
    if (pList5 != NULL)
    {
        KrStringLink * pLink5 = pList5->GetFirst();
        if (pLink5 != NULL)
        {
            (outf_ <<= level_) << "(authNotes" << endl;
            while (pLink5 != NULL)
            {
                KrString * pKrString = pLink5->GetData();
                if (pKrString != NULL)
                {
                    pKrString->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink5 = pLink5->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    // Decompile member: authnotesTitle//gen.icn/2141
    if (authnotesTitle != NULL)
    {
        authnotesTitle->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(authnotesTitle \"" << buf << "\")" << endl;
    } // if
    level_--;
    (outf_ <<= level_) << ")" << endl;
} // FictionAuthor::Decompile

// ---------------------------------------------------------

// Save the FictionAuthor in the database.

Db::RecNumber FictionAuthor::Save(DbBlock * block_, Db::RecNumber prevRecno_)
{
    Db::RecNumber recno_;
    DbFictionAuthor dbFictionAuthor;

    BuildDbRec(block_, dbFictionAuthor);
    dbFictionAuthor.next_ = prevRecno_;
    recno_ = block_->AddDbRec(&dbFictionAuthor, sizeof(dbFictionAuthor),
        True, "FictionAuthor"
        );
    return recno_;
} // FictionAuthor::Save

// ---------------------------------------------------------

// Build FictionAuthor DB record.//gen.icn/2261
void FictionAuthor::BuildDbRec(DbBlock * block_, DbFictionAuthor& dbFictionAuthor)
{
    char * str_;
    memset(&dbFictionAuthor, 0, sizeof(dbFictionAuthor));
    if (id_ != NULL)
    {
        str_ = (char *)*id_;
        dbFictionAuthor.id_ = block_->AddDbString(str_);
    } // if

    // Save each field in a FictionAuthor//gen.icn/2277
    // Save member: topicArea//gen.icn/2322
    if (topicArea != NULL)
    {
        str_ = (char *)*topicArea;
        dbFictionAuthor.topicArea = block_->AddDbString(str_);
    } // if
    // Save member: name//gen.icn/2322
    if (name != NULL)
    {
        str_ = (char *)*name;
        dbFictionAuthor.name = block_->AddDbString(str_);
    } // if
    // Save member: publishedWorks//gen.icn/2353
    dbFictionAuthor.publishedWorks = publishedWorks->Save(block_);
    // Save member: worksInProgress//gen.icn/2353
    dbFictionAuthor.worksInProgress = worksInProgress->Save(block_);
    // Save member: authNotes//gen.icn/2353
    dbFictionAuthor.authNotes = authNotes->Save(block_);
    // Save member: authnotesTitle//gen.icn/2322
    if (authnotesTitle != NULL)
    {
        str_ = (char *)*authnotesTitle;
        dbFictionAuthor.authnotesTitle = block_->AddDbString(str_);
    } // if
} // FictionAuthor::BuildDbRec

// --------------------------------------------------------

void FictionAuthorList::Append(FictionAuthor * initData) {
    FictionAuthorLink * pLink = new FictionAuthorLink(initData);
    if (first == NULL)
    {
        first = pLink;
        last = pLink;
    } // if
    else
    {
        last->SetNext(pLink);
        last = pLink;
    } // else
} // FictionAuthorList::Append


// --------------------------------------------------------

// Insert item into list at position pos.
//  If pos > size of list, insert at end of list.
void FictionAuthorList::Insert(FictionAuthor * initData, int pos)
{
    FictionAuthorLink * pLink = new FictionAuthorLink(initData);
    if (first == NULL)
    {
        first = pLink;
        last = pLink;
    } // if
    else if (pos == 0)
    {
        pLink->SetNext(first);
        first = pLink;
    } // else if
    else
    {
        FictionAuthorLink * pL = first;
        int idx = 0;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                // Add it to the end.
                last->SetNext(pLink);
                last = pLink;
                break;
            } // if
            if (idx == pos)
            {
                // We are at the desired position. Add it.
                pLink->SetNext(pL->GetNext());
                pL->SetNext(pLink);
                break;
            } // if
            idx++;
            pL = pL->GetNext();
        } // for
    } // else
} // FictionAuthorList::Insert

// --------------------------------------------------------

// Delete an item from list at position pos.
//  If pos > size of list, return false; else return true.
boolean FictionAuthorList::Delete(int pos)
{
    if (first == NULL)
    {
        return false;
    } // if
    else if (pos == 0)
    {
        FictionAuthorLink * pLink = first;
        first = first->GetNext();
        FictionAuthor * pFictionAuthor = pLink->GetData();
        delete pLink;
        delete pFictionAuthor;
        if (first == NULL)
        {
            last = NULL;
        } // if
    } // else if
    else
    {
        FictionAuthorLink * pL = first->GetNext();
        FictionAuthorLink * pPrevLink = first;
        int idx = 1;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                return false;
            } // if
            if (idx == pos)
            {
                // We are at the desired position. Delete it.
                pPrevLink->SetNext(pL->GetNext());
                if (pL == last)
                {
                    last = pPrevLink;
                } // if
                FictionAuthor * pFictionAuthor = pL->GetData();
                delete pL;
                delete pFictionAuthor;
                break;
            } // if
            idx++;
            pPrevLink = pL;
            pL = pL->GetNext();
        } // for
    } // else
    return true;
} // FictionAuthorList::Delete

// --------------------------------------------------------

// Return the item at position pos (relative to 0).
FictionAuthor * FictionAuthorList::operator [](int pos)
{
    if (first == NULL)
    {
        return NULL;
    } // if
    else if (pos == 0)
    {
        return first->GetData();
    } // else if
    else
    {
        FictionAuthorLink * pL = first;
        int idx = 0;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                return NULL;
            } // if
            if (idx == pos)
            {
                // We are at the desired position.
                return pL->GetData();
            } // if
            idx++;
            pL = pL->GetNext();
        } // for
    } // else
} // FictionAuthorList::operator []

// --------------------------------------------------------

FictionAuthorList * FictionAuthorList::Copy()
{
    if (first == NULL)
    {
        return NULL;
    } // if
    FictionAuthorList * pList = new FictionAuthorList;
    FictionAuthorLink * pLink = first;
    while (pLink != NULL)
    {
        FictionAuthor * pData1 = pLink->GetData();
        if (pData1 != NULL)
        {
            FictionAuthor * pData2 = new FictionAuthor(*pData1);
            pList->Append(pData2);
        } // if
        pLink = pLink->GetNext();
    } // while
    return pList;
} // FictionAuthorList::Copy

// --------------------------------------------------------

// Load a list of records from the database.
//  Each record contains the record number of the next record.
FictionAuthorList * FictionAuthorList::Load(DbBlock * block_, Db::RecNumber firstRecno_)
{
    Db::RecNumber recno_ = firstRecno_;
    Db::RecNumber nextRecno_;

    if (recno_ == Db::DB_NULL_RECID)
    {
        return NULL;
    } // if
    FictionAuthorList * pNewList_ = new FictionAuthorList;
    while (recno_ != Db::DB_NULL_RECID)
    {
        FictionAuthor * pFictionAuthor = FictionAuthor::Load(block_, recno_, nextRecno_);
        pNewList_->Insert(pFictionAuthor, 0);
        recno_ = nextRecno_;
    } // while
    return pNewList_;
} // FictionAuthorList::Load

// --------------------------------------------------------

// Save a list of records into the database.
//  Each record contains the record number of the next record.
Db::RecNumber FictionAuthorList::Save(DbBlock * block_)
{
    Db::RecNumber recno_ = Db::DB_NULL_RECID;
    Db::RecNumber prevRecno_ = Db::DB_NULL_RECID;

    FictionAuthorLink * pLink_ = first;
    while (pLink_ != NULL)
    {
        FictionAuthor * pB_ = pLink_->GetData();
        if (pB_ != NULL)
        {
            recno_ = pB_->Save(block_, prevRecno_);
            prevRecno_ = recno_;
        } // if
        pLink_ = pLink_->GetNext();
    } // while
    return recno_;
} // FictionAuthorList::Save

// ---------------------------------------------------------

// Generate .IPF text for node: FictionAuthor//gen.icn/1184
void FictionAuthor::GenIpf(KrOstream & ofs, int level_)
{
    if (level_ == 0)
    {
        ofs << ":userdoc." << endl;
        ofs << ":docprof toc=123456." << endl;
        ofs << ":title." << "FictionAuthor window" << endl;
    } // if

    if (level_ > 6)
    {
        KrReportError("Nesting too deep", KR_ERROR_LEVEL_1);
    } // if
    ofs << ":h" << level_ + 1 << ".FictionAuthor information window" << endl
            << ":p." << endl;
    if (level_ == 0)
    {
        ofs << ":euserdoc." << endl;
    } // if
} // FictionAuthor::GenIpf


// ---------------------------------------------------------
// ---------------------------------------------------------

// Member functions for: HumorFictionAuthor

// ---------------------------------------------------------

// Default constructor for class HumorFictionAuthor//gen.icn/938
HumorFictionAuthor::HumorFictionAuthor()
    :
    FictionAuthor(),
    style(NULL)
{
} // HumorFictionAuthor::HumorFictionAuthor

// ---------------------------------------------------------

// Copy constructor for class HumorFictionAuthor//gen.icn/959
HumorFictionAuthor::HumorFictionAuthor(const HumorFictionAuthor& rhs_)
    :
    FictionAuthor(),
    style(NULL)
{
    if (((char *)*rhs_.id_) != NULL)
    {
        id_ = new KrString(*rhs_.id_);
    } // if
    // Copy member: style//gen.icn/1016
    style = new KrString(*rhs_.style);
    // Copy fields in superclass FictionAuthor//gen.icn/981
    // Copy member: topicArea//gen.icn/1016
    topicArea = new KrString(*rhs_.topicArea);
    // Copy fields in superclass Author//gen.icn/981
    // Copy member: name//gen.icn/1016
    name = new KrString(*rhs_.name);
    // Copy member: publishedWorks//gen.icn/1023
    publishedWorks = rhs_.publishedWorks->Copy();
    // Copy member: worksInProgress//gen.icn/1023
    worksInProgress = rhs_.worksInProgress->Copy();
    // Copy member: authNotes//gen.icn/1023
    authNotes = rhs_.authNotes->Copy();
    // Copy member: authnotesTitle//gen.icn/1016
    authnotesTitle = new KrString(*rhs_.authnotesTitle);
} // HumorFictionAuthor::HumorFictionAuthor

// ---------------------------------------------------------

// Constructor to create HumorFictionAuthor by loading from the database.//gen.icn/1316
HumorFictionAuthor * HumorFictionAuthor::Load(DbBlock * block_, Db::RecNumber recno_, Db::RecNumber & nextRecno_)
{
    Db::RecNumber convertedRecno_;

    if (recno_ == Db::DB_NULL_RECID)
    {
        nextRecno_ = Db::DB_NULL_RECID;
        return NULL;
    } // if
    if (DbBlock::IsDescriptor(recno_, convertedRecno_))
    {
        Db::DescriptorRec desc_;
        block_->GetRec(convertedRecno_, &desc_);
        char * type = block_->GetStr(desc_.className);
        recno_ = desc_.data;
        if (strcmp(type, "HumorFictionAuthor") == 0)
        {
            HumorFictionAuthor * pObj = HumorFictionAuthor::Load(block_, recno_, nextRecno_);
            return pObj;
        } // if
    } // if
    else
    {
        DbHumorFictionAuthor dbHumorFictionAuthor;
        block_->GetRec(recno_, &dbHumorFictionAuthor);
        HumorFictionAuthor * pObj = new HumorFictionAuthor(block_, dbHumorFictionAuthor);
        nextRecno_ = dbHumorFictionAuthor.next_;
        return pObj;
    } // else
    return NULL;
} // HumorFictionAuthor::Load

// ---------------------------------------------------------

// Constructor to create HumorFictionAuthor by loading from the database.
HumorFictionAuthor::HumorFictionAuthor(DbBlock * block_, DbHumorFictionAuthor & dbHumorFictionAuthor_)
    :
    FictionAuthor(),
    style(NULL)
{
    // Convert each field.
    BuildRec(block_, dbHumorFictionAuthor_);
} // HumorFictionAuthor::HumorFictionAuthor

// ---------------------------------------------------------

// Destructor for class HumorFictionAuthor//gen.icn/1608
HumorFictionAuthor::~HumorFictionAuthor()
{

    // Destruct member: style//gen.icn/1641
    delete style;
    // Destruct member: topicArea//gen.icn/1641
    delete topicArea;
    // Destruct member: name//gen.icn/1641
    delete name;
    // Destruct member: publishedWorks//gen.icn/1653
    delete publishedWorks;
    // Destruct member: worksInProgress//gen.icn/1653
    delete worksInProgress;
    // Destruct member: authNotes//gen.icn/1653
    delete authNotes;
    // Destruct member: authnotesTitle//gen.icn/1641
    delete authnotesTitle;
} // HumorFictionAuthor::~HumorFictionAuthor

// ---------------------------------------------------------

// Build HumorFictionAuthor record from DB record.//gen.icn/1680
void HumorFictionAuthor::BuildRec(DbBlock * block_, DbHumorFictionAuthor& dbHumorFictionAuthor)
{
    Db::RecNumber recno_;
    if (dbHumorFictionAuthor.id_ != Db::DB_NULL_RECID)
    {
        id_ = new KrString(block_->GetStr(dbHumorFictionAuthor.id_));
    } // if
    // Convert each field in a HumorFictionAuthor//gen.icn/1695
    // Load member: style//gen.icn/1740
    if (dbHumorFictionAuthor.style != Db::DB_NULL_RECID)
    {
        style = new KrString(block_->GetStr(dbHumorFictionAuthor.style));
    } // if
    // Load member: topicArea//gen.icn/1740
    if (dbHumorFictionAuthor.topicArea != Db::DB_NULL_RECID)
    {
        topicArea = new KrString(block_->GetStr(dbHumorFictionAuthor.topicArea));
    } // if
    // Load member: name//gen.icn/1740
    if (dbHumorFictionAuthor.name != Db::DB_NULL_RECID)
    {
        name = new KrString(block_->GetStr(dbHumorFictionAuthor.name));
    } // if
    // Load member: publishedWorks//gen.icn/1770
    publishedWorks = DocumentList::Load(block_, dbHumorFictionAuthor.publishedWorks);
    // Load member: worksInProgress//gen.icn/1770
    worksInProgress = DocumentList::Load(block_, dbHumorFictionAuthor.worksInProgress);
    // Load member: authNotes//gen.icn/1770
    authNotes = KrStringList::Load(block_, dbHumorFictionAuthor.authNotes);
    // Load member: authnotesTitle//gen.icn/1740
    if (dbHumorFictionAuthor.authnotesTitle != Db::DB_NULL_RECID)
    {
        authnotesTitle = new KrString(block_->GetStr(dbHumorFictionAuthor.authnotesTitle));
    } // if
} // HumorFictionAuthor::BuildRec

// ---------------------------------------------------------

// Assignment operator for class HumorFictionAuthor//gen.icn/1971
HumorFictionAuthor& HumorFictionAuthor::operator=(const HumorFictionAuthor& rhs_)
{
    if (this == &rhs_)
    {
        return *this;
    } // if
    // Copy member: style//gen.icn/1016
    style = new KrString(*rhs_.style);
    // Copy member: topicArea//gen.icn/1016
    topicArea = new KrString(*rhs_.topicArea);
    // Copy member: name//gen.icn/1016
    name = new KrString(*rhs_.name);
    // Copy member: publishedWorks//gen.icn/1023
    publishedWorks = rhs_.publishedWorks->Copy();
    // Copy member: worksInProgress//gen.icn/1023
    worksInProgress = rhs_.worksInProgress->Copy();
    // Copy member: authNotes//gen.icn/1023
    authNotes = rhs_.authNotes->Copy();
    // Copy member: authnotesTitle//gen.icn/1016
    authnotesTitle = new KrString(*rhs_.authnotesTitle);
    return *this;
} // HumorFictionAuthor::operator=

// ---------------------------------------------------------

// Decompile member function for class HumorFictionAuthor//gen.icn/2008
void HumorFictionAuthor::Decompile(KrOstream& outf_, int level_, int & count_)
{
    char buf[1024];

    // Decompile object ID//gen.icn/2021
    if (id_ == NULL)
    {
        count_++;
        (outf_ <<= level_) << "(HumorFictionAuthor humorfictionauthor" << count_ << endl;
    } // if
    else
    {
        (outf_ <<= level_) << "(HumorFictionAuthor " << ((char *)*id_) << endl;
    } // else
    level_++;
    // Decompile member: style//gen.icn/2141
    if (style != NULL)
    {
        style->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(style \"" << buf << "\")" << endl;
    } // if
    // Decompile member: topicArea//gen.icn/2141
    if (topicArea != NULL)
    {
        topicArea->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(topicArea \"" << buf << "\")" << endl;
    } // if
    // Decompile member: name//gen.icn/2141
    if (name != NULL)
    {
        name->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(name \"" << buf << "\")" << endl;
    } // if
    // Decompile member: publishedWorks//gen.icn/2182
    DocumentList * pList4 = publishedWorks;
    if (pList4 != NULL)
    {
        DocumentLink * pLink4 = pList4->GetFirst();
        if (pLink4 != NULL)
        {
            (outf_ <<= level_) << "(publishedWorks" << endl;
            while (pLink4 != NULL)
            {
                Document * pDocument = pLink4->GetData();
                if (pDocument != NULL)
                {
                    pDocument->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink4 = pLink4->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    // Decompile member: worksInProgress//gen.icn/2182
    DocumentList * pList5 = worksInProgress;
    if (pList5 != NULL)
    {
        DocumentLink * pLink5 = pList5->GetFirst();
        if (pLink5 != NULL)
        {
            (outf_ <<= level_) << "(worksInProgress" << endl;
            while (pLink5 != NULL)
            {
                Document * pDocument = pLink5->GetData();
                if (pDocument != NULL)
                {
                    pDocument->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink5 = pLink5->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    // Decompile member: authNotes//gen.icn/2182
    KrStringList * pList6 = authNotes;
    if (pList6 != NULL)
    {
        KrStringLink * pLink6 = pList6->GetFirst();
        if (pLink6 != NULL)
        {
            (outf_ <<= level_) << "(authNotes" << endl;
            while (pLink6 != NULL)
            {
                KrString * pKrString = pLink6->GetData();
                if (pKrString != NULL)
                {
                    pKrString->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink6 = pLink6->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    // Decompile member: authnotesTitle//gen.icn/2141
    if (authnotesTitle != NULL)
    {
        authnotesTitle->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(authnotesTitle \"" << buf << "\")" << endl;
    } // if
    level_--;
    (outf_ <<= level_) << ")" << endl;
} // HumorFictionAuthor::Decompile

// ---------------------------------------------------------

// Save the HumorFictionAuthor in the database.

Db::RecNumber HumorFictionAuthor::Save(DbBlock * block_, Db::RecNumber prevRecno_)
{
    Db::RecNumber recno_;
    DbHumorFictionAuthor dbHumorFictionAuthor;

    BuildDbRec(block_, dbHumorFictionAuthor);
    dbHumorFictionAuthor.next_ = prevRecno_;
    recno_ = block_->AddDbRec(&dbHumorFictionAuthor, sizeof(dbHumorFictionAuthor),
        True, "HumorFictionAuthor"
        );
    return recno_;
} // HumorFictionAuthor::Save

// ---------------------------------------------------------

// Build HumorFictionAuthor DB record.//gen.icn/2261
void HumorFictionAuthor::BuildDbRec(DbBlock * block_, DbHumorFictionAuthor& dbHumorFictionAuthor)
{
    char * str_;
    memset(&dbHumorFictionAuthor, 0, sizeof(dbHumorFictionAuthor));
    if (id_ != NULL)
    {
        str_ = (char *)*id_;
        dbHumorFictionAuthor.id_ = block_->AddDbString(str_);
    } // if

    // Save each field in a HumorFictionAuthor//gen.icn/2277
    // Save member: style//gen.icn/2322
    if (style != NULL)
    {
        str_ = (char *)*style;
        dbHumorFictionAuthor.style = block_->AddDbString(str_);
    } // if
    // Save member: topicArea//gen.icn/2322
    if (topicArea != NULL)
    {
        str_ = (char *)*topicArea;
        dbHumorFictionAuthor.topicArea = block_->AddDbString(str_);
    } // if
    // Save member: name//gen.icn/2322
    if (name != NULL)
    {
        str_ = (char *)*name;
        dbHumorFictionAuthor.name = block_->AddDbString(str_);
    } // if
    // Save member: publishedWorks//gen.icn/2353
    dbHumorFictionAuthor.publishedWorks = publishedWorks->Save(block_);
    // Save member: worksInProgress//gen.icn/2353
    dbHumorFictionAuthor.worksInProgress = worksInProgress->Save(block_);
    // Save member: authNotes//gen.icn/2353
    dbHumorFictionAuthor.authNotes = authNotes->Save(block_);
    // Save member: authnotesTitle//gen.icn/2322
    if (authnotesTitle != NULL)
    {
        str_ = (char *)*authnotesTitle;
        dbHumorFictionAuthor.authnotesTitle = block_->AddDbString(str_);
    } // if
} // HumorFictionAuthor::BuildDbRec

// --------------------------------------------------------

void HumorFictionAuthorList::Append(HumorFictionAuthor * initData) {
    HumorFictionAuthorLink * pLink = new HumorFictionAuthorLink(initData);
    if (first == NULL)
    {
        first = pLink;
        last = pLink;
    } // if
    else
    {
        last->SetNext(pLink);
        last = pLink;
    } // else
} // HumorFictionAuthorList::Append


// --------------------------------------------------------

// Insert item into list at position pos.
//  If pos > size of list, insert at end of list.
void HumorFictionAuthorList::Insert(HumorFictionAuthor * initData, int pos)
{
    HumorFictionAuthorLink * pLink = new HumorFictionAuthorLink(initData);
    if (first == NULL)
    {
        first = pLink;
        last = pLink;
    } // if
    else if (pos == 0)
    {
        pLink->SetNext(first);
        first = pLink;
    } // else if
    else
    {
        HumorFictionAuthorLink * pL = first;
        int idx = 0;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                // Add it to the end.
                last->SetNext(pLink);
                last = pLink;
                break;
            } // if
            if (idx == pos)
            {
                // We are at the desired position. Add it.
                pLink->SetNext(pL->GetNext());
                pL->SetNext(pLink);
                break;
            } // if
            idx++;
            pL = pL->GetNext();
        } // for
    } // else
} // HumorFictionAuthorList::Insert

// --------------------------------------------------------

// Delete an item from list at position pos.
//  If pos > size of list, return false; else return true.
boolean HumorFictionAuthorList::Delete(int pos)
{
    if (first == NULL)
    {
        return false;
    } // if
    else if (pos == 0)
    {
        HumorFictionAuthorLink * pLink = first;
        first = first->GetNext();
        HumorFictionAuthor * pHumorFictionAuthor = pLink->GetData();
        delete pLink;
        delete pHumorFictionAuthor;
        if (first == NULL)
        {
            last = NULL;
        } // if
    } // else if
    else
    {
        HumorFictionAuthorLink * pL = first->GetNext();
        HumorFictionAuthorLink * pPrevLink = first;
        int idx = 1;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                return false;
            } // if
            if (idx == pos)
            {
                // We are at the desired position. Delete it.
                pPrevLink->SetNext(pL->GetNext());
                if (pL == last)
                {
                    last = pPrevLink;
                } // if
                HumorFictionAuthor * pHumorFictionAuthor = pL->GetData();
                delete pL;
                delete pHumorFictionAuthor;
                break;
            } // if
            idx++;
            pPrevLink = pL;
            pL = pL->GetNext();
        } // for
    } // else
    return true;
} // HumorFictionAuthorList::Delete

// --------------------------------------------------------

// Return the item at position pos (relative to 0).
HumorFictionAuthor * HumorFictionAuthorList::operator [](int pos)
{
    if (first == NULL)
    {
        return NULL;
    } // if
    else if (pos == 0)
    {
        return first->GetData();
    } // else if
    else
    {
        HumorFictionAuthorLink * pL = first;
        int idx = 0;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                return NULL;
            } // if
            if (idx == pos)
            {
                // We are at the desired position.
                return pL->GetData();
            } // if
            idx++;
            pL = pL->GetNext();
        } // for
    } // else
} // HumorFictionAuthorList::operator []

// --------------------------------------------------------

HumorFictionAuthorList * HumorFictionAuthorList::Copy()
{
    if (first == NULL)
    {
        return NULL;
    } // if
    HumorFictionAuthorList * pList = new HumorFictionAuthorList;
    HumorFictionAuthorLink * pLink = first;
    while (pLink != NULL)
    {
        HumorFictionAuthor * pData1 = pLink->GetData();
        if (pData1 != NULL)
        {
            HumorFictionAuthor * pData2 = new HumorFictionAuthor(*pData1);
            pList->Append(pData2);
        } // if
        pLink = pLink->GetNext();
    } // while
    return pList;
} // HumorFictionAuthorList::Copy

// --------------------------------------------------------

// Load a list of records from the database.
//  Each record contains the record number of the next record.
HumorFictionAuthorList * HumorFictionAuthorList::Load(DbBlock * block_, Db::RecNumber firstRecno_)
{
    Db::RecNumber recno_ = firstRecno_;
    Db::RecNumber nextRecno_;

    if (recno_ == Db::DB_NULL_RECID)
    {
        return NULL;
    } // if
    HumorFictionAuthorList * pNewList_ = new HumorFictionAuthorList;
    while (recno_ != Db::DB_NULL_RECID)
    {
        HumorFictionAuthor * pHumorFictionAuthor = HumorFictionAuthor::Load(block_, recno_, nextRecno_);
        pNewList_->Insert(pHumorFictionAuthor, 0);
        recno_ = nextRecno_;
    } // while
    return pNewList_;
} // HumorFictionAuthorList::Load

// --------------------------------------------------------

// Save a list of records into the database.
//  Each record contains the record number of the next record.
Db::RecNumber HumorFictionAuthorList::Save(DbBlock * block_)
{
    Db::RecNumber recno_ = Db::DB_NULL_RECID;
    Db::RecNumber prevRecno_ = Db::DB_NULL_RECID;

    HumorFictionAuthorLink * pLink_ = first;
    while (pLink_ != NULL)
    {
        HumorFictionAuthor * pB_ = pLink_->GetData();
        if (pB_ != NULL)
        {
            recno_ = pB_->Save(block_, prevRecno_);
            prevRecno_ = recno_;
        } // if
        pLink_ = pLink_->GetNext();
    } // while
    return recno_;
} // HumorFictionAuthorList::Save

// ---------------------------------------------------------

// Generate .IPF text for node: HumorFictionAuthor//gen.icn/1184
void HumorFictionAuthor::GenIpf(KrOstream & ofs, int level_)
{
    if (level_ == 0)
    {
        ofs << ":userdoc." << endl;
        ofs << ":docprof toc=123456." << endl;
        ofs << ":title." << "HumorFictionAuthor window" << endl;
    } // if

    if (level_ > 6)
    {
        KrReportError("Nesting too deep", KR_ERROR_LEVEL_1);
    } // if
    ofs << ":h" << level_ + 1 << ".HumorFictionAuthor information window" << endl
            << ":p." << endl;
    if (level_ == 0)
    {
        ofs << ":euserdoc." << endl;
    } // if
} // HumorFictionAuthor::GenIpf


// ---------------------------------------------------------
// ---------------------------------------------------------

// Member functions for: RomanticFictionAuthor

// ---------------------------------------------------------

// Default constructor for class RomanticFictionAuthor//gen.icn/938
RomanticFictionAuthor::RomanticFictionAuthor()
    :
    FictionAuthor(),
    period(NULL)
{
} // RomanticFictionAuthor::RomanticFictionAuthor

// ---------------------------------------------------------

// Copy constructor for class RomanticFictionAuthor//gen.icn/959
RomanticFictionAuthor::RomanticFictionAuthor(const RomanticFictionAuthor& rhs_)
    :
    FictionAuthor(),
    period(NULL)
{
    if (((char *)*rhs_.id_) != NULL)
    {
        id_ = new KrString(*rhs_.id_);
    } // if
    // Copy member: period//gen.icn/1016
    period = new KrString(*rhs_.period);
    // Copy member: emotionalTemperature//gen.icn/1011
    emotionalTemperature = rhs_.emotionalTemperature;
    // Copy fields in superclass FictionAuthor//gen.icn/981
    // Copy member: topicArea//gen.icn/1016
    topicArea = new KrString(*rhs_.topicArea);
    // Copy fields in superclass Author//gen.icn/981
    // Copy member: name//gen.icn/1016
    name = new KrString(*rhs_.name);
    // Copy member: publishedWorks//gen.icn/1023
    publishedWorks = rhs_.publishedWorks->Copy();
    // Copy member: worksInProgress//gen.icn/1023
    worksInProgress = rhs_.worksInProgress->Copy();
    // Copy member: authNotes//gen.icn/1023
    authNotes = rhs_.authNotes->Copy();
    // Copy member: authnotesTitle//gen.icn/1016
    authnotesTitle = new KrString(*rhs_.authnotesTitle);
} // RomanticFictionAuthor::RomanticFictionAuthor

// ---------------------------------------------------------

// Constructor to create RomanticFictionAuthor by loading from the database.//gen.icn/1316
RomanticFictionAuthor * RomanticFictionAuthor::Load(DbBlock * block_, Db::RecNumber recno_, Db::RecNumber & nextRecno_)
{
    Db::RecNumber convertedRecno_;

    if (recno_ == Db::DB_NULL_RECID)
    {
        nextRecno_ = Db::DB_NULL_RECID;
        return NULL;
    } // if
    if (DbBlock::IsDescriptor(recno_, convertedRecno_))
    {
        Db::DescriptorRec desc_;
        block_->GetRec(convertedRecno_, &desc_);
        char * type = block_->GetStr(desc_.className);
        recno_ = desc_.data;
        if (strcmp(type, "RomanticFictionAuthor") == 0)
        {
            RomanticFictionAuthor * pObj = RomanticFictionAuthor::Load(block_, recno_, nextRecno_);
            return pObj;
        } // if
    } // if
    else
    {
        DbRomanticFictionAuthor dbRomanticFictionAuthor;
        block_->GetRec(recno_, &dbRomanticFictionAuthor);
        RomanticFictionAuthor * pObj = new RomanticFictionAuthor(block_, dbRomanticFictionAuthor);
        nextRecno_ = dbRomanticFictionAuthor.next_;
        return pObj;
    } // else
    return NULL;
} // RomanticFictionAuthor::Load

// ---------------------------------------------------------

// Constructor to create RomanticFictionAuthor by loading from the database.
RomanticFictionAuthor::RomanticFictionAuthor(DbBlock * block_, DbRomanticFictionAuthor & dbRomanticFictionAuthor_)
    :
    FictionAuthor(),
    period(NULL)
{
    // Convert each field.
    BuildRec(block_, dbRomanticFictionAuthor_);
} // RomanticFictionAuthor::RomanticFictionAuthor

// ---------------------------------------------------------

// Destructor for class RomanticFictionAuthor//gen.icn/1608
RomanticFictionAuthor::~RomanticFictionAuthor()
{

    // Destruct member: period//gen.icn/1641
    delete period;
    // Destruct member: emotionalTemperature//gen.icn/1641
    // Destruct member: topicArea//gen.icn/1641
    delete topicArea;
    // Destruct member: name//gen.icn/1641
    delete name;
    // Destruct member: publishedWorks//gen.icn/1653
    delete publishedWorks;
    // Destruct member: worksInProgress//gen.icn/1653
    delete worksInProgress;
    // Destruct member: authNotes//gen.icn/1653
    delete authNotes;
    // Destruct member: authnotesTitle//gen.icn/1641
    delete authnotesTitle;
} // RomanticFictionAuthor::~RomanticFictionAuthor

// ---------------------------------------------------------

// Build RomanticFictionAuthor record from DB record.//gen.icn/1680
void RomanticFictionAuthor::BuildRec(DbBlock * block_, DbRomanticFictionAuthor& dbRomanticFictionAuthor)
{
    Db::RecNumber recno_;
    if (dbRomanticFictionAuthor.id_ != Db::DB_NULL_RECID)
    {
        id_ = new KrString(block_->GetStr(dbRomanticFictionAuthor.id_));
    } // if
    // Convert each field in a RomanticFictionAuthor//gen.icn/1695
    // Load member: period//gen.icn/1740
    if (dbRomanticFictionAuthor.period != Db::DB_NULL_RECID)
    {
        period = new KrString(block_->GetStr(dbRomanticFictionAuthor.period));
    } // if
    // Load member: emotionalTemperature//gen.icn/1732
    emotionalTemperature = dbRomanticFictionAuthor.emotionalTemperature;
    // Load member: topicArea//gen.icn/1740
    if (dbRomanticFictionAuthor.topicArea != Db::DB_NULL_RECID)
    {
        topicArea = new KrString(block_->GetStr(dbRomanticFictionAuthor.topicArea));
    } // if
    // Load member: name//gen.icn/1740
    if (dbRomanticFictionAuthor.name != Db::DB_NULL_RECID)
    {
        name = new KrString(block_->GetStr(dbRomanticFictionAuthor.name));
    } // if
    // Load member: publishedWorks//gen.icn/1770
    publishedWorks = DocumentList::Load(block_, dbRomanticFictionAuthor.publishedWorks);
    // Load member: worksInProgress//gen.icn/1770
    worksInProgress = DocumentList::Load(block_, dbRomanticFictionAuthor.worksInProgress);
    // Load member: authNotes//gen.icn/1770
    authNotes = KrStringList::Load(block_, dbRomanticFictionAuthor.authNotes);
    // Load member: authnotesTitle//gen.icn/1740
    if (dbRomanticFictionAuthor.authnotesTitle != Db::DB_NULL_RECID)
    {
        authnotesTitle = new KrString(block_->GetStr(dbRomanticFictionAuthor.authnotesTitle));
    } // if
} // RomanticFictionAuthor::BuildRec

// ---------------------------------------------------------

// Assignment operator for class RomanticFictionAuthor//gen.icn/1971
RomanticFictionAuthor& RomanticFictionAuthor::operator=(const RomanticFictionAuthor& rhs_)
{
    if (this == &rhs_)
    {
        return *this;
    } // if
    // Copy member: period//gen.icn/1016
    period = new KrString(*rhs_.period);
    // Copy member: emotionalTemperature//gen.icn/1011
    emotionalTemperature = rhs_.emotionalTemperature;
    // Copy member: topicArea//gen.icn/1016
    topicArea = new KrString(*rhs_.topicArea);
    // Copy member: name//gen.icn/1016
    name = new KrString(*rhs_.name);
    // Copy member: publishedWorks//gen.icn/1023
    publishedWorks = rhs_.publishedWorks->Copy();
    // Copy member: worksInProgress//gen.icn/1023
    worksInProgress = rhs_.worksInProgress->Copy();
    // Copy member: authNotes//gen.icn/1023
    authNotes = rhs_.authNotes->Copy();
    // Copy member: authnotesTitle//gen.icn/1016
    authnotesTitle = new KrString(*rhs_.authnotesTitle);
    return *this;
} // RomanticFictionAuthor::operator=

// ---------------------------------------------------------

// Decompile member function for class RomanticFictionAuthor//gen.icn/2008
void RomanticFictionAuthor::Decompile(KrOstream& outf_, int level_, int & count_)
{
    char buf[1024];

    // Decompile object ID//gen.icn/2021
    if (id_ == NULL)
    {
        count_++;
        (outf_ <<= level_) << "(RomanticFictionAuthor romanticfictionauthor" << count_ << endl;
    } // if
    else
    {
        (outf_ <<= level_) << "(RomanticFictionAuthor " << ((char *)*id_) << endl;
    } // else
    level_++;
    // Decompile member: period//gen.icn/2141
    if (period != NULL)
    {
        period->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(period \"" << buf << "\")" << endl;
    } // if
    // Decompile member: emotionalTemperature//gen.icn/2087
    (outf_ <<= level_) << "(emotionalTemperature " << emotionalTemperature << ")" << endl;
    // Decompile member: topicArea//gen.icn/2141
    if (topicArea != NULL)
    {
        topicArea->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(topicArea \"" << buf << "\")" << endl;
    } // if
    // Decompile member: name//gen.icn/2141
    if (name != NULL)
    {
        name->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(name \"" << buf << "\")" << endl;
    } // if
    // Decompile member: publishedWorks//gen.icn/2182
    DocumentList * pList5 = publishedWorks;
    if (pList5 != NULL)
    {
        DocumentLink * pLink5 = pList5->GetFirst();
        if (pLink5 != NULL)
        {
            (outf_ <<= level_) << "(publishedWorks" << endl;
            while (pLink5 != NULL)
            {
                Document * pDocument = pLink5->GetData();
                if (pDocument != NULL)
                {
                    pDocument->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink5 = pLink5->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    // Decompile member: worksInProgress//gen.icn/2182
    DocumentList * pList6 = worksInProgress;
    if (pList6 != NULL)
    {
        DocumentLink * pLink6 = pList6->GetFirst();
        if (pLink6 != NULL)
        {
            (outf_ <<= level_) << "(worksInProgress" << endl;
            while (pLink6 != NULL)
            {
                Document * pDocument = pLink6->GetData();
                if (pDocument != NULL)
                {
                    pDocument->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink6 = pLink6->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    // Decompile member: authNotes//gen.icn/2182
    KrStringList * pList7 = authNotes;
    if (pList7 != NULL)
    {
        KrStringLink * pLink7 = pList7->GetFirst();
        if (pLink7 != NULL)
        {
            (outf_ <<= level_) << "(authNotes" << endl;
            while (pLink7 != NULL)
            {
                KrString * pKrString = pLink7->GetData();
                if (pKrString != NULL)
                {
                    pKrString->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink7 = pLink7->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    // Decompile member: authnotesTitle//gen.icn/2141
    if (authnotesTitle != NULL)
    {
        authnotesTitle->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(authnotesTitle \"" << buf << "\")" << endl;
    } // if
    level_--;
    (outf_ <<= level_) << ")" << endl;
} // RomanticFictionAuthor::Decompile

// ---------------------------------------------------------

// Save the RomanticFictionAuthor in the database.

Db::RecNumber RomanticFictionAuthor::Save(DbBlock * block_, Db::RecNumber prevRecno_)
{
    Db::RecNumber recno_;
    DbRomanticFictionAuthor dbRomanticFictionAuthor;

    BuildDbRec(block_, dbRomanticFictionAuthor);
    dbRomanticFictionAuthor.next_ = prevRecno_;
    recno_ = block_->AddDbRec(&dbRomanticFictionAuthor, sizeof(dbRomanticFictionAuthor),
        True, "RomanticFictionAuthor"
        );
    return recno_;
} // RomanticFictionAuthor::Save

// ---------------------------------------------------------

// Build RomanticFictionAuthor DB record.//gen.icn/2261
void RomanticFictionAuthor::BuildDbRec(DbBlock * block_, DbRomanticFictionAuthor& dbRomanticFictionAuthor)
{
    char * str_;
    memset(&dbRomanticFictionAuthor, 0, sizeof(dbRomanticFictionAuthor));
    if (id_ != NULL)
    {
        str_ = (char *)*id_;
        dbRomanticFictionAuthor.id_ = block_->AddDbString(str_);
    } // if

    // Save each field in a RomanticFictionAuthor//gen.icn/2277
    // Save member: period//gen.icn/2322
    if (period != NULL)
    {
        str_ = (char *)*period;
        dbRomanticFictionAuthor.period = block_->AddDbString(str_);
    } // if
    // Save member: emotionalTemperature//gen.icn/2314
    dbRomanticFictionAuthor.emotionalTemperature = emotionalTemperature;
    // Save member: topicArea//gen.icn/2322
    if (topicArea != NULL)
    {
        str_ = (char *)*topicArea;
        dbRomanticFictionAuthor.topicArea = block_->AddDbString(str_);
    } // if
    // Save member: name//gen.icn/2322
    if (name != NULL)
    {
        str_ = (char *)*name;
        dbRomanticFictionAuthor.name = block_->AddDbString(str_);
    } // if
    // Save member: publishedWorks//gen.icn/2353
    dbRomanticFictionAuthor.publishedWorks = publishedWorks->Save(block_);
    // Save member: worksInProgress//gen.icn/2353
    dbRomanticFictionAuthor.worksInProgress = worksInProgress->Save(block_);
    // Save member: authNotes//gen.icn/2353
    dbRomanticFictionAuthor.authNotes = authNotes->Save(block_);
    // Save member: authnotesTitle//gen.icn/2322
    if (authnotesTitle != NULL)
    {
        str_ = (char *)*authnotesTitle;
        dbRomanticFictionAuthor.authnotesTitle = block_->AddDbString(str_);
    } // if
} // RomanticFictionAuthor::BuildDbRec

// --------------------------------------------------------

void RomanticFictionAuthorList::Append(RomanticFictionAuthor * initData) {
    RomanticFictionAuthorLink * pLink = new RomanticFictionAuthorLink(initData);
    if (first == NULL)
    {
        first = pLink;
        last = pLink;
    } // if
    else
    {
        last->SetNext(pLink);
        last = pLink;
    } // else
} // RomanticFictionAuthorList::Append


// --------------------------------------------------------

// Insert item into list at position pos.
//  If pos > size of list, insert at end of list.
void RomanticFictionAuthorList::Insert(RomanticFictionAuthor * initData, int pos)
{
    RomanticFictionAuthorLink * pLink = new RomanticFictionAuthorLink(initData);
    if (first == NULL)
    {
        first = pLink;
        last = pLink;
    } // if
    else if (pos == 0)
    {
        pLink->SetNext(first);
        first = pLink;
    } // else if
    else
    {
        RomanticFictionAuthorLink * pL = first;
        int idx = 0;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                // Add it to the end.
                last->SetNext(pLink);
                last = pLink;
                break;
            } // if
            if (idx == pos)
            {
                // We are at the desired position. Add it.
                pLink->SetNext(pL->GetNext());
                pL->SetNext(pLink);
                break;
            } // if
            idx++;
            pL = pL->GetNext();
        } // for
    } // else
} // RomanticFictionAuthorList::Insert

// --------------------------------------------------------

// Delete an item from list at position pos.
//  If pos > size of list, return false; else return true.
boolean RomanticFictionAuthorList::Delete(int pos)
{
    if (first == NULL)
    {
        return false;
    } // if
    else if (pos == 0)
    {
        RomanticFictionAuthorLink * pLink = first;
        first = first->GetNext();
        RomanticFictionAuthor * pRomanticFictionAuthor = pLink->GetData();
        delete pLink;
        delete pRomanticFictionAuthor;
        if (first == NULL)
        {
            last = NULL;
        } // if
    } // else if
    else
    {
        RomanticFictionAuthorLink * pL = first->GetNext();
        RomanticFictionAuthorLink * pPrevLink = first;
        int idx = 1;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                return false;
            } // if
            if (idx == pos)
            {
                // We are at the desired position. Delete it.
                pPrevLink->SetNext(pL->GetNext());
                if (pL == last)
                {
                    last = pPrevLink;
                } // if
                RomanticFictionAuthor * pRomanticFictionAuthor = pL->GetData();
                delete pL;
                delete pRomanticFictionAuthor;
                break;
            } // if
            idx++;
            pPrevLink = pL;
            pL = pL->GetNext();
        } // for
    } // else
    return true;
} // RomanticFictionAuthorList::Delete

// --------------------------------------------------------

// Return the item at position pos (relative to 0).
RomanticFictionAuthor * RomanticFictionAuthorList::operator [](int pos)
{
    if (first == NULL)
    {
        return NULL;
    } // if
    else if (pos == 0)
    {
        return first->GetData();
    } // else if
    else
    {
        RomanticFictionAuthorLink * pL = first;
        int idx = 0;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                return NULL;
            } // if
            if (idx == pos)
            {
                // We are at the desired position.
                return pL->GetData();
            } // if
            idx++;
            pL = pL->GetNext();
        } // for
    } // else
} // RomanticFictionAuthorList::operator []

// --------------------------------------------------------

RomanticFictionAuthorList * RomanticFictionAuthorList::Copy()
{
    if (first == NULL)
    {
        return NULL;
    } // if
    RomanticFictionAuthorList * pList = new RomanticFictionAuthorList;
    RomanticFictionAuthorLink * pLink = first;
    while (pLink != NULL)
    {
        RomanticFictionAuthor * pData1 = pLink->GetData();
        if (pData1 != NULL)
        {
            RomanticFictionAuthor * pData2 = new RomanticFictionAuthor(*pData1);
            pList->Append(pData2);
        } // if
        pLink = pLink->GetNext();
    } // while
    return pList;
} // RomanticFictionAuthorList::Copy

// --------------------------------------------------------

// Load a list of records from the database.
//  Each record contains the record number of the next record.
RomanticFictionAuthorList * RomanticFictionAuthorList::Load(DbBlock * block_, Db::RecNumber firstRecno_)
{
    Db::RecNumber recno_ = firstRecno_;
    Db::RecNumber nextRecno_;

    if (recno_ == Db::DB_NULL_RECID)
    {
        return NULL;
    } // if
    RomanticFictionAuthorList * pNewList_ = new RomanticFictionAuthorList;
    while (recno_ != Db::DB_NULL_RECID)
    {
        RomanticFictionAuthor * pRomanticFictionAuthor = RomanticFictionAuthor::Load(block_, recno_, nextRecno_);
        pNewList_->Insert(pRomanticFictionAuthor, 0);
        recno_ = nextRecno_;
    } // while
    return pNewList_;
} // RomanticFictionAuthorList::Load

// --------------------------------------------------------

// Save a list of records into the database.
//  Each record contains the record number of the next record.
Db::RecNumber RomanticFictionAuthorList::Save(DbBlock * block_)
{
    Db::RecNumber recno_ = Db::DB_NULL_RECID;
    Db::RecNumber prevRecno_ = Db::DB_NULL_RECID;

    RomanticFictionAuthorLink * pLink_ = first;
    while (pLink_ != NULL)
    {
        RomanticFictionAuthor * pB_ = pLink_->GetData();
        if (pB_ != NULL)
        {
            recno_ = pB_->Save(block_, prevRecno_);
            prevRecno_ = recno_;
        } // if
        pLink_ = pLink_->GetNext();
    } // while
    return recno_;
} // RomanticFictionAuthorList::Save

// ---------------------------------------------------------

// Generate .IPF text for node: RomanticFictionAuthor//gen.icn/1184
void RomanticFictionAuthor::GenIpf(KrOstream & ofs, int level_)
{
    if (level_ == 0)
    {
        ofs << ":userdoc." << endl;
        ofs << ":docprof toc=123456." << endl;
        ofs << ":title." << "RomanticFictionAuthor window" << endl;
    } // if

    if (level_ > 6)
    {
        KrReportError("Nesting too deep", KR_ERROR_LEVEL_1);
    } // if
    ofs << ":h" << level_ + 1 << ".RomanticFictionAuthor information window" << endl
            << ":p." << endl;
    if (level_ == 0)
    {
        ofs << ":euserdoc." << endl;
    } // if
} // RomanticFictionAuthor::GenIpf


// ---------------------------------------------------------
// ---------------------------------------------------------

// Member functions for: NonfictionAuthor

// ---------------------------------------------------------

// Default constructor for class NonfictionAuthor//gen.icn/938
NonfictionAuthor::NonfictionAuthor()
    :
    Author(),
    specialFields(new KrStringList()),
    biases(new KrStringList())
{
} // NonfictionAuthor::NonfictionAuthor

// ---------------------------------------------------------

// Copy constructor for class NonfictionAuthor//gen.icn/959
NonfictionAuthor::NonfictionAuthor(const NonfictionAuthor& rhs_)
    :
    Author(),
    specialFields(new KrStringList()),
    biases(new KrStringList())
{
    if (((char *)*rhs_.id_) != NULL)
    {
        id_ = new KrString(*rhs_.id_);
    } // if
    // Copy member: docCount//gen.icn/1011
    docCount = rhs_.docCount;
    // Copy member: specialFields//gen.icn/1023
    specialFields = rhs_.specialFields->Copy();
    // Copy member: biases//gen.icn/1023
    biases = rhs_.biases->Copy();
    // Copy fields in superclass Author//gen.icn/981
    // Copy member: name//gen.icn/1016
    name = new KrString(*rhs_.name);
    // Copy member: publishedWorks//gen.icn/1023
    publishedWorks = rhs_.publishedWorks->Copy();
    // Copy member: worksInProgress//gen.icn/1023
    worksInProgress = rhs_.worksInProgress->Copy();
    // Copy member: authNotes//gen.icn/1023
    authNotes = rhs_.authNotes->Copy();
    // Copy member: authnotesTitle//gen.icn/1016
    authnotesTitle = new KrString(*rhs_.authnotesTitle);
} // NonfictionAuthor::NonfictionAuthor

// ---------------------------------------------------------

// Constructor to create NonfictionAuthor by loading from the database.//gen.icn/1316
NonfictionAuthor * NonfictionAuthor::Load(DbBlock * block_, Db::RecNumber recno_, Db::RecNumber & nextRecno_)
{
    Db::RecNumber convertedRecno_;

    if (recno_ == Db::DB_NULL_RECID)
    {
        nextRecno_ = Db::DB_NULL_RECID;
        return NULL;
    } // if
    if (DbBlock::IsDescriptor(recno_, convertedRecno_))
    {
        Db::DescriptorRec desc_;
        block_->GetRec(convertedRecno_, &desc_);
        char * type = block_->GetStr(desc_.className);
        recno_ = desc_.data;
        if (strcmp(type, "NonfictionAuthor") == 0)
        {
            NonfictionAuthor * pObj = NonfictionAuthor::Load(block_, recno_, nextRecno_);
            return pObj;
        } // if
    } // if
    else
    {
        DbNonfictionAuthor dbNonfictionAuthor;
        block_->GetRec(recno_, &dbNonfictionAuthor);
        NonfictionAuthor * pObj = new NonfictionAuthor(block_, dbNonfictionAuthor);
        nextRecno_ = dbNonfictionAuthor.next_;
        return pObj;
    } // else
    return NULL;
} // NonfictionAuthor::Load

// ---------------------------------------------------------

// Constructor to create NonfictionAuthor by loading from the database.
NonfictionAuthor::NonfictionAuthor(DbBlock * block_, DbNonfictionAuthor & dbNonfictionAuthor_)
    :
    Author(),
    specialFields(new KrStringList()),
    biases(new KrStringList())
{
    // Convert each field.
    BuildRec(block_, dbNonfictionAuthor_);
} // NonfictionAuthor::NonfictionAuthor

// ---------------------------------------------------------

// Destructor for class NonfictionAuthor//gen.icn/1608
NonfictionAuthor::~NonfictionAuthor()
{

    // Destruct member: docCount//gen.icn/1641
    // Destruct member: specialFields//gen.icn/1653
    delete specialFields;
    // Destruct member: biases//gen.icn/1653
    delete biases;
    // Destruct member: name//gen.icn/1641
    delete name;
    // Destruct member: publishedWorks//gen.icn/1653
    delete publishedWorks;
    // Destruct member: worksInProgress//gen.icn/1653
    delete worksInProgress;
    // Destruct member: authNotes//gen.icn/1653
    delete authNotes;
    // Destruct member: authnotesTitle//gen.icn/1641
    delete authnotesTitle;
} // NonfictionAuthor::~NonfictionAuthor

// ---------------------------------------------------------

// Build NonfictionAuthor record from DB record.//gen.icn/1680
void NonfictionAuthor::BuildRec(DbBlock * block_, DbNonfictionAuthor& dbNonfictionAuthor)
{
    Db::RecNumber recno_;
    if (dbNonfictionAuthor.id_ != Db::DB_NULL_RECID)
    {
        id_ = new KrString(block_->GetStr(dbNonfictionAuthor.id_));
    } // if
    // Convert each field in a NonfictionAuthor//gen.icn/1695
    // Load member: docCount//gen.icn/1732
    docCount = dbNonfictionAuthor.docCount;
    // Load member: specialFields//gen.icn/1770
    specialFields = KrStringList::Load(block_, dbNonfictionAuthor.specialFields);
    // Load member: biases//gen.icn/1770
    biases = KrStringList::Load(block_, dbNonfictionAuthor.biases);
    // Load member: name//gen.icn/1740
    if (dbNonfictionAuthor.name != Db::DB_NULL_RECID)
    {
        name = new KrString(block_->GetStr(dbNonfictionAuthor.name));
    } // if
    // Load member: publishedWorks//gen.icn/1770
    publishedWorks = DocumentList::Load(block_, dbNonfictionAuthor.publishedWorks);
    // Load member: worksInProgress//gen.icn/1770
    worksInProgress = DocumentList::Load(block_, dbNonfictionAuthor.worksInProgress);
    // Load member: authNotes//gen.icn/1770
    authNotes = KrStringList::Load(block_, dbNonfictionAuthor.authNotes);
    // Load member: authnotesTitle//gen.icn/1740
    if (dbNonfictionAuthor.authnotesTitle != Db::DB_NULL_RECID)
    {
        authnotesTitle = new KrString(block_->GetStr(dbNonfictionAuthor.authnotesTitle));
    } // if
} // NonfictionAuthor::BuildRec

// ---------------------------------------------------------

// Assignment operator for class NonfictionAuthor//gen.icn/1971
NonfictionAuthor& NonfictionAuthor::operator=(const NonfictionAuthor& rhs_)
{
    if (this == &rhs_)
    {
        return *this;
    } // if
    // Copy member: docCount//gen.icn/1011
    docCount = rhs_.docCount;
    // Copy member: specialFields//gen.icn/1023
    specialFields = rhs_.specialFields->Copy();
    // Copy member: biases//gen.icn/1023
    biases = rhs_.biases->Copy();
    // Copy member: name//gen.icn/1016
    name = new KrString(*rhs_.name);
    // Copy member: publishedWorks//gen.icn/1023
    publishedWorks = rhs_.publishedWorks->Copy();
    // Copy member: worksInProgress//gen.icn/1023
    worksInProgress = rhs_.worksInProgress->Copy();
    // Copy member: authNotes//gen.icn/1023
    authNotes = rhs_.authNotes->Copy();
    // Copy member: authnotesTitle//gen.icn/1016
    authnotesTitle = new KrString(*rhs_.authnotesTitle);
    return *this;
} // NonfictionAuthor::operator=

// ---------------------------------------------------------

// Decompile member function for class NonfictionAuthor//gen.icn/2008
void NonfictionAuthor::Decompile(KrOstream& outf_, int level_, int & count_)
{
    char buf[1024];

    // Decompile object ID//gen.icn/2021
    if (id_ == NULL)
    {
        count_++;
        (outf_ <<= level_) << "(NonfictionAuthor nonfictionauthor" << count_ << endl;
    } // if
    else
    {
        (outf_ <<= level_) << "(NonfictionAuthor " << ((char *)*id_) << endl;
    } // else
    level_++;
    // Decompile member: docCount//gen.icn/2087
    (outf_ <<= level_) << "(docCount " << docCount << ")" << endl;
    // Decompile member: specialFields//gen.icn/2182
    KrStringList * pList2 = specialFields;
    if (pList2 != NULL)
    {
        KrStringLink * pLink2 = pList2->GetFirst();
        if (pLink2 != NULL)
        {
            (outf_ <<= level_) << "(specialFields" << endl;
            while (pLink2 != NULL)
            {
                KrString * pKrString = pLink2->GetData();
                if (pKrString != NULL)
                {
                    pKrString->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink2 = pLink2->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    // Decompile member: biases//gen.icn/2182
    KrStringList * pList3 = biases;
    if (pList3 != NULL)
    {
        KrStringLink * pLink3 = pList3->GetFirst();
        if (pLink3 != NULL)
        {
            (outf_ <<= level_) << "(biases" << endl;
            while (pLink3 != NULL)
            {
                KrString * pKrString = pLink3->GetData();
                if (pKrString != NULL)
                {
                    pKrString->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink3 = pLink3->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    // Decompile member: name//gen.icn/2141
    if (name != NULL)
    {
        name->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(name \"" << buf << "\")" << endl;
    } // if
    // Decompile member: publishedWorks//gen.icn/2182
    DocumentList * pList5 = publishedWorks;
    if (pList5 != NULL)
    {
        DocumentLink * pLink5 = pList5->GetFirst();
        if (pLink5 != NULL)
        {
            (outf_ <<= level_) << "(publishedWorks" << endl;
            while (pLink5 != NULL)
            {
                Document * pDocument = pLink5->GetData();
                if (pDocument != NULL)
                {
                    pDocument->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink5 = pLink5->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    // Decompile member: worksInProgress//gen.icn/2182
    DocumentList * pList6 = worksInProgress;
    if (pList6 != NULL)
    {
        DocumentLink * pLink6 = pList6->GetFirst();
        if (pLink6 != NULL)
        {
            (outf_ <<= level_) << "(worksInProgress" << endl;
            while (pLink6 != NULL)
            {
                Document * pDocument = pLink6->GetData();
                if (pDocument != NULL)
                {
                    pDocument->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink6 = pLink6->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    // Decompile member: authNotes//gen.icn/2182
    KrStringList * pList7 = authNotes;
    if (pList7 != NULL)
    {
        KrStringLink * pLink7 = pList7->GetFirst();
        if (pLink7 != NULL)
        {
            (outf_ <<= level_) << "(authNotes" << endl;
            while (pLink7 != NULL)
            {
                KrString * pKrString = pLink7->GetData();
                if (pKrString != NULL)
                {
                    pKrString->Decompile(outf_, level_ + 1, count_);
                } // if
                pLink7 = pLink7->GetNext();
            } // while
            (outf_ <<= level_) << ")" << endl;
        } // if
    } // if
    // Decompile member: authnotesTitle//gen.icn/2141
    if (authnotesTitle != NULL)
    {
        authnotesTitle->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(authnotesTitle \"" << buf << "\")" << endl;
    } // if
    level_--;
    (outf_ <<= level_) << ")" << endl;
} // NonfictionAuthor::Decompile

// ---------------------------------------------------------

// Save the NonfictionAuthor in the database.

Db::RecNumber NonfictionAuthor::Save(DbBlock * block_, Db::RecNumber prevRecno_)
{
    Db::RecNumber recno_;
    DbNonfictionAuthor dbNonfictionAuthor;

    BuildDbRec(block_, dbNonfictionAuthor);
    dbNonfictionAuthor.next_ = prevRecno_;
    recno_ = block_->AddDbRec(&dbNonfictionAuthor, sizeof(dbNonfictionAuthor),
        True, "NonfictionAuthor"
        );
    return recno_;
} // NonfictionAuthor::Save

// ---------------------------------------------------------

// Build NonfictionAuthor DB record.//gen.icn/2261
void NonfictionAuthor::BuildDbRec(DbBlock * block_, DbNonfictionAuthor& dbNonfictionAuthor)
{
    char * str_;
    memset(&dbNonfictionAuthor, 0, sizeof(dbNonfictionAuthor));
    if (id_ != NULL)
    {
        str_ = (char *)*id_;
        dbNonfictionAuthor.id_ = block_->AddDbString(str_);
    } // if

    // Save each field in a NonfictionAuthor//gen.icn/2277
    // Save member: docCount//gen.icn/2314
    dbNonfictionAuthor.docCount = docCount;
    // Save member: specialFields//gen.icn/2353
    dbNonfictionAuthor.specialFields = specialFields->Save(block_);
    // Save member: biases//gen.icn/2353
    dbNonfictionAuthor.biases = biases->Save(block_);
    // Save member: name//gen.icn/2322
    if (name != NULL)
    {
        str_ = (char *)*name;
        dbNonfictionAuthor.name = block_->AddDbString(str_);
    } // if
    // Save member: publishedWorks//gen.icn/2353
    dbNonfictionAuthor.publishedWorks = publishedWorks->Save(block_);
    // Save member: worksInProgress//gen.icn/2353
    dbNonfictionAuthor.worksInProgress = worksInProgress->Save(block_);
    // Save member: authNotes//gen.icn/2353
    dbNonfictionAuthor.authNotes = authNotes->Save(block_);
    // Save member: authnotesTitle//gen.icn/2322
    if (authnotesTitle != NULL)
    {
        str_ = (char *)*authnotesTitle;
        dbNonfictionAuthor.authnotesTitle = block_->AddDbString(str_);
    } // if
} // NonfictionAuthor::BuildDbRec

// --------------------------------------------------------

void NonfictionAuthorList::Append(NonfictionAuthor * initData) {
    NonfictionAuthorLink * pLink = new NonfictionAuthorLink(initData);
    if (first == NULL)
    {
        first = pLink;
        last = pLink;
    } // if
    else
    {
        last->SetNext(pLink);
        last = pLink;
    } // else
} // NonfictionAuthorList::Append


// --------------------------------------------------------

// Insert item into list at position pos.
//  If pos > size of list, insert at end of list.
void NonfictionAuthorList::Insert(NonfictionAuthor * initData, int pos)
{
    NonfictionAuthorLink * pLink = new NonfictionAuthorLink(initData);
    if (first == NULL)
    {
        first = pLink;
        last = pLink;
    } // if
    else if (pos == 0)
    {
        pLink->SetNext(first);
        first = pLink;
    } // else if
    else
    {
        NonfictionAuthorLink * pL = first;
        int idx = 0;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                // Add it to the end.
                last->SetNext(pLink);
                last = pLink;
                break;
            } // if
            if (idx == pos)
            {
                // We are at the desired position. Add it.
                pLink->SetNext(pL->GetNext());
                pL->SetNext(pLink);
                break;
            } // if
            idx++;
            pL = pL->GetNext();
        } // for
    } // else
} // NonfictionAuthorList::Insert

// --------------------------------------------------------

// Delete an item from list at position pos.
//  If pos > size of list, return false; else return true.
boolean NonfictionAuthorList::Delete(int pos)
{
    if (first == NULL)
    {
        return false;
    } // if
    else if (pos == 0)
    {
        NonfictionAuthorLink * pLink = first;
        first = first->GetNext();
        NonfictionAuthor * pNonfictionAuthor = pLink->GetData();
        delete pLink;
        delete pNonfictionAuthor;
        if (first == NULL)
        {
            last = NULL;
        } // if
    } // else if
    else
    {
        NonfictionAuthorLink * pL = first->GetNext();
        NonfictionAuthorLink * pPrevLink = first;
        int idx = 1;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                return false;
            } // if
            if (idx == pos)
            {
                // We are at the desired position. Delete it.
                pPrevLink->SetNext(pL->GetNext());
                if (pL == last)
                {
                    last = pPrevLink;
                } // if
                NonfictionAuthor * pNonfictionAuthor = pL->GetData();
                delete pL;
                delete pNonfictionAuthor;
                break;
            } // if
            idx++;
            pPrevLink = pL;
            pL = pL->GetNext();
        } // for
    } // else
    return true;
} // NonfictionAuthorList::Delete

// --------------------------------------------------------

// Return the item at position pos (relative to 0).
NonfictionAuthor * NonfictionAuthorList::operator [](int pos)
{
    if (first == NULL)
    {
        return NULL;
    } // if
    else if (pos == 0)
    {
        return first->GetData();
    } // else if
    else
    {
        NonfictionAuthorLink * pL = first;
        int idx = 0;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                return NULL;
            } // if
            if (idx == pos)
            {
                // We are at the desired position.
                return pL->GetData();
            } // if
            idx++;
            pL = pL->GetNext();
        } // for
    } // else
} // NonfictionAuthorList::operator []

// --------------------------------------------------------

NonfictionAuthorList * NonfictionAuthorList::Copy()
{
    if (first == NULL)
    {
        return NULL;
    } // if
    NonfictionAuthorList * pList = new NonfictionAuthorList;
    NonfictionAuthorLink * pLink = first;
    while (pLink != NULL)
    {
        NonfictionAuthor * pData1 = pLink->GetData();
        if (pData1 != NULL)
        {
            NonfictionAuthor * pData2 = new NonfictionAuthor(*pData1);
            pList->Append(pData2);
        } // if
        pLink = pLink->GetNext();
    } // while
    return pList;
} // NonfictionAuthorList::Copy

// --------------------------------------------------------

// Load a list of records from the database.
//  Each record contains the record number of the next record.
NonfictionAuthorList * NonfictionAuthorList::Load(DbBlock * block_, Db::RecNumber firstRecno_)
{
    Db::RecNumber recno_ = firstRecno_;
    Db::RecNumber nextRecno_;

    if (recno_ == Db::DB_NULL_RECID)
    {
        return NULL;
    } // if
    NonfictionAuthorList * pNewList_ = new NonfictionAuthorList;
    while (recno_ != Db::DB_NULL_RECID)
    {
        NonfictionAuthor * pNonfictionAuthor = NonfictionAuthor::Load(block_, recno_, nextRecno_);
        pNewList_->Insert(pNonfictionAuthor, 0);
        recno_ = nextRecno_;
    } // while
    return pNewList_;
} // NonfictionAuthorList::Load

// --------------------------------------------------------

// Save a list of records into the database.
//  Each record contains the record number of the next record.
Db::RecNumber NonfictionAuthorList::Save(DbBlock * block_)
{
    Db::RecNumber recno_ = Db::DB_NULL_RECID;
    Db::RecNumber prevRecno_ = Db::DB_NULL_RECID;

    NonfictionAuthorLink * pLink_ = first;
    while (pLink_ != NULL)
    {
        NonfictionAuthor * pB_ = pLink_->GetData();
        if (pB_ != NULL)
        {
            recno_ = pB_->Save(block_, prevRecno_);
            prevRecno_ = recno_;
        } // if
        pLink_ = pLink_->GetNext();
    } // while
    return recno_;
} // NonfictionAuthorList::Save

// ---------------------------------------------------------

// Generate .IPF text for node: NonfictionAuthor//gen.icn/1184
void NonfictionAuthor::GenIpf(KrOstream & ofs, int level_)
{
    if (level_ == 0)
    {
        ofs << ":userdoc." << endl;
        ofs << ":docprof toc=123456." << endl;
        ofs << ":title." << "NonfictionAuthor window" << endl;
    } // if

    if (level_ > 6)
    {
        KrReportError("Nesting too deep", KR_ERROR_LEVEL_1);
    } // if
    ofs << ":h" << level_ + 1 << ".NonfictionAuthor information window" << endl
            << ":p." << endl;
    if (level_ == 0)
    {
        ofs << ":euserdoc." << endl;
    } // if
} // NonfictionAuthor::GenIpf


// ---------------------------------------------------------
// ---------------------------------------------------------

// Member functions for: Pubs

// ---------------------------------------------------------

// Default constructor for class Pubs//gen.icn/938
Pubs::Pubs()
    :
    id_(NULL),
    publisher(NULL),
    location(NULL),
    date(NULL)
{
} // Pubs::Pubs

// ---------------------------------------------------------

// Copy constructor for class Pubs//gen.icn/959
Pubs::Pubs(const Pubs& rhs_)
    :
    id_(NULL),
    publisher(NULL),
    location(NULL),
    date(NULL)
{
    if (((char *)*rhs_.id_) != NULL)
    {
        id_ = new KrString(*rhs_.id_);
    } // if
    // Copy member: publisher//gen.icn/1016
    publisher = new KrString(*rhs_.publisher);
    // Copy member: location//gen.icn/1016
    location = new KrString(*rhs_.location);
    // Copy member: date//gen.icn/1016
    date = new KrString(*rhs_.date);
} // Pubs::Pubs

// ---------------------------------------------------------

// Constructor to create Pubs by loading from the database.//gen.icn/1316
Pubs * Pubs::Load(DbBlock * block_, Db::RecNumber recno_, Db::RecNumber & nextRecno_)
{
    Db::RecNumber convertedRecno_;

    if (recno_ == Db::DB_NULL_RECID)
    {
        nextRecno_ = Db::DB_NULL_RECID;
        return NULL;
    } // if
    if (DbBlock::IsDescriptor(recno_, convertedRecno_))
    {
        Db::DescriptorRec desc_;
        block_->GetRec(convertedRecno_, &desc_);
        char * type = block_->GetStr(desc_.className);
        recno_ = desc_.data;
        if (strcmp(type, "Pubs") == 0)
        {
            Pubs * pObj = Pubs::Load(block_, recno_, nextRecno_);
            return pObj;
        } // if
    } // if
    else
    {
        DbPubs dbPubs;
        block_->GetRec(recno_, &dbPubs);
        Pubs * pObj = new Pubs(block_, dbPubs);
        nextRecno_ = dbPubs.next_;
        return pObj;
    } // else
    return NULL;
} // Pubs::Load

// ---------------------------------------------------------

// Constructor to create Pubs by loading from the database.
Pubs::Pubs(DbBlock * block_, DbPubs & dbPubs_)
    :
    id_(NULL),
    publisher(NULL),
    location(NULL),
    date(NULL)
{
    // Convert each field.
    BuildRec(block_, dbPubs_);
} // Pubs::Pubs

// ---------------------------------------------------------

// Destructor for class Pubs//gen.icn/1608
Pubs::~Pubs()
{

    // Destruct member: publisher//gen.icn/1641
    delete publisher;
    // Destruct member: location//gen.icn/1641
    delete location;
    // Destruct member: date//gen.icn/1641
    delete date;
} // Pubs::~Pubs

// ---------------------------------------------------------

// Build Pubs record from DB record.//gen.icn/1680
void Pubs::BuildRec(DbBlock * block_, DbPubs& dbPubs)
{
    Db::RecNumber recno_;
    if (dbPubs.id_ != Db::DB_NULL_RECID)
    {
        id_ = new KrString(block_->GetStr(dbPubs.id_));
    } // if
    // Convert each field in a Pubs//gen.icn/1695
    // Load member: publisher//gen.icn/1740
    if (dbPubs.publisher != Db::DB_NULL_RECID)
    {
        publisher = new KrString(block_->GetStr(dbPubs.publisher));
    } // if
    // Load member: location//gen.icn/1740
    if (dbPubs.location != Db::DB_NULL_RECID)
    {
        location = new KrString(block_->GetStr(dbPubs.location));
    } // if
    // Load member: date//gen.icn/1740
    if (dbPubs.date != Db::DB_NULL_RECID)
    {
        date = new KrString(block_->GetStr(dbPubs.date));
    } // if
} // Pubs::BuildRec

// ---------------------------------------------------------

// Assignment operator for class Pubs//gen.icn/1971
Pubs& Pubs::operator=(const Pubs& rhs_)
{
    if (this == &rhs_)
    {
        return *this;
    } // if
    // Copy member: publisher//gen.icn/1016
    publisher = new KrString(*rhs_.publisher);
    // Copy member: location//gen.icn/1016
    location = new KrString(*rhs_.location);
    // Copy member: date//gen.icn/1016
    date = new KrString(*rhs_.date);
    return *this;
} // Pubs::operator=

// ---------------------------------------------------------

// Decompile member function for class Pubs//gen.icn/2008
void Pubs::Decompile(KrOstream& outf_, int level_, int & count_)
{
    char buf[1024];

    // Decompile object ID//gen.icn/2021
    if (id_ == NULL)
    {
        count_++;
        (outf_ <<= level_) << "(Pubs pubs" << count_ << endl;
    } // if
    else
    {
        (outf_ <<= level_) << "(Pubs " << ((char *)*id_) << endl;
    } // else
    level_++;
    // Decompile member: publisher//gen.icn/2141
    if (publisher != NULL)
    {
        publisher->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(publisher \"" << buf << "\")" << endl;
    } // if
    // Decompile member: location//gen.icn/2141
    if (location != NULL)
    {
        location->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(location \"" << buf << "\")" << endl;
    } // if
    // Decompile member: date//gen.icn/2141
    if (date != NULL)
    {
        date->EscapeChars(buf, sizeof(buf));
        (outf_ <<= level_) << "(date \"" << buf << "\")" << endl;
    } // if
    level_--;
    (outf_ <<= level_) << ")" << endl;
} // Pubs::Decompile

// ---------------------------------------------------------

// Save the Pubs in the database.

Db::RecNumber Pubs::Save(DbBlock * block_, Db::RecNumber prevRecno_)
{
    Db::RecNumber recno_;
    DbPubs dbPubs;

    BuildDbRec(block_, dbPubs);
    dbPubs.next_ = prevRecno_;
    recno_ = block_->AddDbRec(&dbPubs, sizeof(dbPubs),
        True, "Pubs"
        );
    return recno_;
} // Pubs::Save

// ---------------------------------------------------------

// Build Pubs DB record.//gen.icn/2261
void Pubs::BuildDbRec(DbBlock * block_, DbPubs& dbPubs)
{
    char * str_;
    memset(&dbPubs, 0, sizeof(dbPubs));
    if (id_ != NULL)
    {
        str_ = (char *)*id_;
        dbPubs.id_ = block_->AddDbString(str_);
    } // if

    // Save each field in a Pubs//gen.icn/2277
    // Save member: publisher//gen.icn/2322
    if (publisher != NULL)
    {
        str_ = (char *)*publisher;
        dbPubs.publisher = block_->AddDbString(str_);
    } // if
    // Save member: location//gen.icn/2322
    if (location != NULL)
    {
        str_ = (char *)*location;
        dbPubs.location = block_->AddDbString(str_);
    } // if
    // Save member: date//gen.icn/2322
    if (date != NULL)
    {
        str_ = (char *)*date;
        dbPubs.date = block_->AddDbString(str_);
    } // if
} // Pubs::BuildDbRec

// --------------------------------------------------------

void PubsList::Append(Pubs * initData) {
    PubsLink * pLink = new PubsLink(initData);
    if (first == NULL)
    {
        first = pLink;
        last = pLink;
    } // if
    else
    {
        last->SetNext(pLink);
        last = pLink;
    } // else
} // PubsList::Append


// --------------------------------------------------------

// Insert item into list at position pos.
//  If pos > size of list, insert at end of list.
void PubsList::Insert(Pubs * initData, int pos)
{
    PubsLink * pLink = new PubsLink(initData);
    if (first == NULL)
    {
        first = pLink;
        last = pLink;
    } // if
    else if (pos == 0)
    {
        pLink->SetNext(first);
        first = pLink;
    } // else if
    else
    {
        PubsLink * pL = first;
        int idx = 0;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                // Add it to the end.
                last->SetNext(pLink);
                last = pLink;
                break;
            } // if
            if (idx == pos)
            {
                // We are at the desired position. Add it.
                pLink->SetNext(pL->GetNext());
                pL->SetNext(pLink);
                break;
            } // if
            idx++;
            pL = pL->GetNext();
        } // for
    } // else
} // PubsList::Insert

// --------------------------------------------------------

// Delete an item from list at position pos.
//  If pos > size of list, return false; else return true.
boolean PubsList::Delete(int pos)
{
    if (first == NULL)
    {
        return false;
    } // if
    else if (pos == 0)
    {
        PubsLink * pLink = first;
        first = first->GetNext();
        Pubs * pPubs = pLink->GetData();
        delete pLink;
        delete pPubs;
        if (first == NULL)
        {
            last = NULL;
        } // if
    } // else if
    else
    {
        PubsLink * pL = first->GetNext();
        PubsLink * pPrevLink = first;
        int idx = 1;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                return false;
            } // if
            if (idx == pos)
            {
                // We are at the desired position. Delete it.
                pPrevLink->SetNext(pL->GetNext());
                if (pL == last)
                {
                    last = pPrevLink;
                } // if
                Pubs * pPubs = pL->GetData();
                delete pL;
                delete pPubs;
                break;
            } // if
            idx++;
            pPrevLink = pL;
            pL = pL->GetNext();
        } // for
    } // else
    return true;
} // PubsList::Delete

// --------------------------------------------------------

// Return the item at position pos (relative to 0).
Pubs * PubsList::operator [](int pos)
{
    if (first == NULL)
    {
        return NULL;
    } // if
    else if (pos == 0)
    {
        return first->GetData();
    } // else if
    else
    {
        PubsLink * pL = first;
        int idx = 0;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                return NULL;
            } // if
            if (idx == pos)
            {
                // We are at the desired position.
                return pL->GetData();
            } // if
            idx++;
            pL = pL->GetNext();
        } // for
    } // else
} // PubsList::operator []

// --------------------------------------------------------

PubsList * PubsList::Copy()
{
    if (first == NULL)
    {
        return NULL;
    } // if
    PubsList * pList = new PubsList;
    PubsLink * pLink = first;
    while (pLink != NULL)
    {
        Pubs * pData1 = pLink->GetData();
        if (pData1 != NULL)
        {
            Pubs * pData2 = new Pubs(*pData1);
            pList->Append(pData2);
        } // if
        pLink = pLink->GetNext();
    } // while
    return pList;
} // PubsList::Copy

// --------------------------------------------------------

// Load a list of records from the database.
//  Each record contains the record number of the next record.
PubsList * PubsList::Load(DbBlock * block_, Db::RecNumber firstRecno_)
{
    Db::RecNumber recno_ = firstRecno_;
    Db::RecNumber nextRecno_;

    if (recno_ == Db::DB_NULL_RECID)
    {
        return NULL;
    } // if
    PubsList * pNewList_ = new PubsList;
    while (recno_ != Db::DB_NULL_RECID)
    {
        Pubs * pPubs = Pubs::Load(block_, recno_, nextRecno_);
        pNewList_->Insert(pPubs, 0);
        recno_ = nextRecno_;
    } // while
    return pNewList_;
} // PubsList::Load

// --------------------------------------------------------

// Save a list of records into the database.
//  Each record contains the record number of the next record.
Db::RecNumber PubsList::Save(DbBlock * block_)
{
    Db::RecNumber recno_ = Db::DB_NULL_RECID;
    Db::RecNumber prevRecno_ = Db::DB_NULL_RECID;

    PubsLink * pLink_ = first;
    while (pLink_ != NULL)
    {
        Pubs * pB_ = pLink_->GetData();
        if (pB_ != NULL)
        {
            recno_ = pB_->Save(block_, prevRecno_);
            prevRecno_ = recno_;
        } // if
        pLink_ = pLink_->GetNext();
    } // while
    return recno_;
} // PubsList::Save

// ---------------------------------------------------------

// Generate .IPF text for node: Pubs//gen.icn/1184
void Pubs::GenIpf(KrOstream & ofs, int level_)
{
    if (level_ == 0)
    {
        ofs << ":userdoc." << endl;
        ofs << ":docprof toc=123456." << endl;
        ofs << ":title." << "Pubs window" << endl;
    } // if

    if (level_ > 6)
    {
        KrReportError("Nesting too deep", KR_ERROR_LEVEL_1);
    } // if
    ofs << ":h" << level_ + 1 << ".Pubs information window" << endl
            << ":p." << endl;
    if (level_ == 0)
    {
        ofs << ":euserdoc." << endl;
    } // if
} // Pubs::GenIpf


// ---------------------------------------------------------

// Miscellaneous functions//gen.icn/287

// ---------------------------------------------------------

static void genIpfText(KrOstream & ofs, int level_, KrStringList * notes)
{
    KrString * pString;
    if (notes != NULL)
    {
        KrStringLink * pLink = notes->GetFirst();
        while (pLink != NULL)
        {
            pString = pLink->GetData();
            if (pString != NULL)
            {
                ofs << (const char *)*pString << endl;
            } // if
            pLink = pLink->GetNext();
        } // while
    } // if
} // genIpfText


// ---------------------------------------------------------

