/*
*
* Send comments to:
*   Dave Kuhlman
*   Reify
*   426 Madison Ave #9
*   Redwood City, CA 94061
*   415-368-8450
*   Internet: dkuhlman@netcom.com
*
* Copyright (c) 1994 G. David Kuhlman.
* All rights reserved.
*
* Permission is hereby granted, without written agreement and without
* license or royalty fees, to use, copy, modify, and distribute this
* software and its documentation for any purpose, provided that the
* above copyright notice and the following two paragraphs appear in
* all copies of this software.
* 
* IN NO EVENT SHALL G. DAVID KUHLMAN BE LIABLE TO ANY PARTY FOR
* DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
* OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF G. DAVID KUHLMAN
* HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* G. DAVID KUHLMAN SPECIFICALLY DISCLAIMS ANY WARRANTIES,
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
* ON AN "AS IS" BASIS, AND G. DAVID KUHLMAN HAS NO OBLIGATION TO
* PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*
*/

#header <<

#include "charptr.h"

#include "krg.h"
>>


#lexclass ACTION_STRINGS
#token STRING "\""  << zzmode(START); >>
#token "\\\""       << zzreplchar('\"'); zzmore(); >>
#token "\\n"        << zzreplchar('\n'); zzmore(); >>
#token "\\r"        << zzreplchar('\r'); zzmore(); >>
#token "\\t"        << zzreplchar('\t'); zzmore(); >>
#token "\\[1-9][0-9]*"
                    << zzreplchar((char)strtol(zzbegexpr,NULL,10)); zzmore(); >>
#token "\\0[0-7]*"  << zzreplchar((char)strtol(zzbegexpr,NULL,8)); zzmore(); >>
#token "\\0x[0-9]+" << zzreplchar((char)strtol(zzbegexpr,NULL,16)); zzmore(); >>
#token "\\~[\n\r]"  << zzmore(); >>
#token "[\n\r]"     << zzline++; zzmore(); /* print warning about \n in str */>>
#token "~[\"\n\r\\]+"<< zzmore(); >>


#lexclass ACTION_CPP_COMMENTS
#token "\n"                 << zzline++; zzmode(START); zzskip(); >>
#token "~[\n]+"             << zzskip(); >>

#lexclass ACTION_COMMENTS
#token "\*/"                << zzmode(START); zzskip(); >>
#token "\*"                 << zzskip(); >>
#token "\n"                 << zzline++; zzskip(); >>
#token "~[\n\*]+"           << zzskip(); >>



#lexclass START
#token "[\t\ ]+"    << zzskip(); >>                /* Ignore White */
#token "\n"         << zzline++; zzskip(); >>
// #token STRING "\"(~[\0-\0x1f\"\\]|(\\~[\0-\0x1f]))*\"" <<;>>
#token "\""                 << zzmode(ACTION_STRINGS); zzmore(); >>
#token "/\*"                << zzmode(ACTION_COMMENTS); zzskip(); >>
#token "\*/"                << warn("Missing /*; found dangling */"); zzskip(); >>
#token "//"                 << zzmode(ACTION_CPP_COMMENTS); zzskip(); >>


<<

// --------------------------------------------------------

#include "charptr.c"

#include "kr.h"

// --------------------------------------------------------

void create_ast(
    AST *node,
    Attrib *attr,
    int token,
    char *str
)
{
    int len;

    node->token = token;
    len = strlen(str);
    if (str[0] == '\"')
    {
        node->str = (char *)malloc(len - 1);
        strncpy(node->str, &str[1], len - 2);
        node->str[len - 2] = '\0';
    } // if
    else
    {
        node->str = (char *)malloc(len + 1);
        strcpy(node->str, str);
    } // else
} // create_ast


AST *
zzmk_ast(
    AST *node,
    int token,
    char *str
)
{
    int len;

    node->token = token;
    len = strlen(str);
    if (str[0] == '\"')
    {
        node->str = (char *)malloc(len - 1);
        strncpy(node->str, &str[1], len - 2);
        node->str[len - 2] = '\0';
    } // if
    else
    {
        node->str = (char *)malloc(len + 1);
        strcpy(node->str, str);
    } // else

    return node;
} // zzmk_ast


AST * appendAst(AST * left, AST * right)
{
    if (left == NULL)
    {
        return right;
    } // if
    left->right = right;
    return left;
} // appendAst


#ifdef OMIT_APPENDAST
// Append right to left.
// If left is NULL, return right.
AST * appendAst(AST * left, AST * right)
{
    if (left == NULL)
    {
        return right;
    } // if
    else
    {
        left->right = right;
        return left;
    } // else
} // appendAst
#endif // OMIT_APPENDAST

#define ErrHdr "%s, line %d:"

void warn(char * msg)
{
    fprintf(stderr, ErrHdr, InFileName, zzline);
    fprintf(stderr, " warning: %s\n", msg);
} // warn

void showLevel(int level)
{
    int idx;
    for (idx = 0; idx < level; idx++)
    {
        printf("    ");
    } // for
} // showLevel


void showTree(AST *tree, int level)
{
    if (tree == NULL)
    {
        return;
    } // if

    while (tree != NULL)
    {
        showLevel(level);
        printf("(");
        if (tree->str != NULL)
        {
            printf("%d/%s/\"%s\"", tree->token,
                TokenNames[tree->token - 1], tree->str);
        } // if
        else
        {
            printf("%d/%s", tree->token, "--");
        } // else
        printf(")\n");
        showTree(tree->down, level + 1);
        tree = tree->right;
    } // while
} // showTree


void deleteTree(AST *tree)
{
    AST * tree1;

    if (tree == NULL)
    {
        return;
    } // if

    while (tree != NULL)
    {
        if (tree->str != NULL)
        {
            free(tree->str);
        } // if
        deleteTree(tree->down);
        tree1 = tree->right;
        free(tree);
        tree = tree1;
    } // while
} // deleteTree

// --------------------------------------------------------

static AST * findNextSiblingByType(AST * pNode, int type)
{
    AST * pN = pNode->right;

    while (pN != NULL)
    {
        if (pN->token == type)
        {
            return pN;
        } // if
        pN = pN->right;
    } // while
    return NULL;
} // findNextSiblingByType

// --------------------------------------------------------

AST * Parse(char * infilename, int show)
{
    AST * root = NULL;

    InFileName = (char *)malloc(strlen(infilename) + 1);
    strcpy(InFileName, infilename);

    if ((InFile = fopen(InFileName, "r")) == NULL)
    {
        printf("Can't open input file: %s\n", InFileName);
        exit(-1);
    } // if

    ANTLR(file_top_gram(&root), InFile);

    if (show)
    {
        showTree(root, 0);
    } // if

    fclose(InFile);

    return root;
} // Parse

>>

// --------------------------------------------------------

file_top_gram! :
    << AST *ast1 = NULL; >>
    (bibliography_gram << ast1 = appendAst(#1, ast1); >> )* "@"
    << #0 = #(#[Tok_file_top_gram, "bibliography_file"], ast1); >>
    ;

bibliography_gram! :
    << AST *ast1 = NULL; >>
    "\(" "Bibliography" bibliography_id_gram (bibliography_item_gram << ast1 = appendAst(#1, ast1); >> )* "\)"
    << #0 = #(#[Tok_bibliography_gram, "bibliography_gram"], #3, ast1); >>
    ;

bibliography_item_gram! :
    << AST *ast1 = NULL; >>
    (
        bibltitle_gram << ast1 = #1; >> |
        documents_list_gram << ast1 = #1; >> |
        authors_list_gram << ast1 = #1; >>
    )
    << #0 = ast1; >>
    ;

bibliography_id_gram! :
    SYMBOL << #0 = #[Tok_bibliography_id_gram, $1]; >>
    ;

document_gram! :
    << AST *ast1 = NULL; >>
    "\(" "Document" document_id_gram (document_item_gram << ast1 = appendAst(#1, ast1); >> )* "\)"
    << #0 = #(#[Tok_document_gram, "document_gram"], #3, ast1); >>
    ;

document_item_gram! :
    << AST *ast1 = NULL; >>
    (
        selecteditem_gram << ast1 = #1; >> |
        doctitle_gram << ast1 = #1; >> |
        authorid_gram << ast1 = #1; >> |
        pubs_gram << ast1 = #1; >> |
        keywords_list_gram << ast1 = #1; >> |
        xrefs_list_gram << ast1 = #1; >> |
        docnotes_list_gram << ast1 = #1; >> |
        docnotestitle_gram << ast1 = #1; >>
    )
    << #0 = ast1; >>
    ;

document_id_gram! :
    SYMBOL << #0 = #[Tok_document_id_gram, $1]; >>
    ;

author_super_gram! :
    << AST *ast1 = NULL; >>
    (
        author_gram << ast1 = #1; >> |
        nonfictionauthor_gram << ast1 = #1; >> |
        fictionauthor_super_gram << ast1 = #1; >>
    )
    << #0 = ast1; >>
    ;

author_gram! :
    << AST *ast1 = NULL; >>
    "\(" "Author" author_id_gram (author_item_gram << ast1 = appendAst(#1, ast1); >> )* "\)"
    << #0 = #(#[Tok_author_gram, "author_gram"], #3, ast1); >>
    ;

author_item_gram! :
    << AST *ast1 = NULL; >>
    (
        name_gram << ast1 = #1; >> |
        publishedworks_list_gram << ast1 = #1; >> |
        worksinprogress_list_gram << ast1 = #1; >> |
        authnotes_list_gram << ast1 = #1; >> |
        authnotestitle_gram << ast1 = #1; >>
    )
    << #0 = ast1; >>
    ;

author_id_gram! :
    SYMBOL << #0 = #[Tok_author_id_gram, $1]; >>
    ;

fictionauthor_super_gram! :
    << AST *ast1 = NULL; >>
    (
        fictionauthor_gram << ast1 = #1; >> |
        humorfictionauthor_gram << ast1 = #1; >> |
        romanticfictionauthor_gram << ast1 = #1; >>
    )
    << #0 = ast1; >>
    ;

fictionauthor_gram! :
    << AST *ast1 = NULL; >>
    "\(" "FictionAuthor" fictionauthor_id_gram (fictionauthor_item_gram << ast1 = appendAst(#1, ast1); >> )* "\)"
    << #0 = #(#[Tok_fictionauthor_gram, "fictionauthor_gram"], #3, ast1); >>
    ;

fictionauthor_item_gram! :
    << AST *ast1 = NULL; >>
    (
        topicarea_gram << ast1 = #1; >> |
        name_gram << ast1 = #1; >> |
        publishedworks_list_gram << ast1 = #1; >> |
        worksinprogress_list_gram << ast1 = #1; >> |
        authnotes_list_gram << ast1 = #1; >> |
        authnotestitle_gram << ast1 = #1; >>
    )
    << #0 = ast1; >>
    ;

fictionauthor_id_gram! :
    SYMBOL << #0 = #[Tok_fictionauthor_id_gram, $1]; >>
    ;

humorfictionauthor_gram! :
    << AST *ast1 = NULL; >>
    "\(" "HumorFictionAuthor" humorfictionauthor_id_gram (humorfictionauthor_item_gram << ast1 = appendAst(#1, ast1); >> )* "\)"
    << #0 = #(#[Tok_humorfictionauthor_gram, "humorfictionauthor_gram"], #3, ast1); >>
    ;

humorfictionauthor_item_gram! :
    << AST *ast1 = NULL; >>
    (
        style_gram << ast1 = #1; >> |
        topicarea_gram << ast1 = #1; >> |
        name_gram << ast1 = #1; >> |
        publishedworks_list_gram << ast1 = #1; >> |
        worksinprogress_list_gram << ast1 = #1; >> |
        authnotes_list_gram << ast1 = #1; >> |
        authnotestitle_gram << ast1 = #1; >>
    )
    << #0 = ast1; >>
    ;

humorfictionauthor_id_gram! :
    SYMBOL << #0 = #[Tok_humorfictionauthor_id_gram, $1]; >>
    ;

romanticfictionauthor_gram! :
    << AST *ast1 = NULL; >>
    "\(" "RomanticFictionAuthor" romanticfictionauthor_id_gram (romanticfictionauthor_item_gram << ast1 = appendAst(#1, ast1); >> )* "\)"
    << #0 = #(#[Tok_romanticfictionauthor_gram, "romanticfictionauthor_gram"], #3, ast1); >>
    ;

romanticfictionauthor_item_gram! :
    << AST *ast1 = NULL; >>
    (
        period_gram << ast1 = #1; >> |
        emotionaltemperature_gram << ast1 = #1; >> |
        topicarea_gram << ast1 = #1; >> |
        name_gram << ast1 = #1; >> |
        publishedworks_list_gram << ast1 = #1; >> |
        worksinprogress_list_gram << ast1 = #1; >> |
        authnotes_list_gram << ast1 = #1; >> |
        authnotestitle_gram << ast1 = #1; >>
    )
    << #0 = ast1; >>
    ;

romanticfictionauthor_id_gram! :
    SYMBOL << #0 = #[Tok_romanticfictionauthor_id_gram, $1]; >>
    ;

nonfictionauthor_gram! :
    << AST *ast1 = NULL; >>
    "\(" "NonfictionAuthor" nonfictionauthor_id_gram (nonfictionauthor_item_gram << ast1 = appendAst(#1, ast1); >> )* "\)"
    << #0 = #(#[Tok_nonfictionauthor_gram, "nonfictionauthor_gram"], #3, ast1); >>
    ;

nonfictionauthor_item_gram! :
    << AST *ast1 = NULL; >>
    (
        doccount_gram << ast1 = #1; >> |
        specialfields_list_gram << ast1 = #1; >> |
        biases_list_gram << ast1 = #1; >> |
        name_gram << ast1 = #1; >> |
        publishedworks_list_gram << ast1 = #1; >> |
        worksinprogress_list_gram << ast1 = #1; >> |
        authnotes_list_gram << ast1 = #1; >> |
        authnotestitle_gram << ast1 = #1; >>
    )
    << #0 = ast1; >>
    ;

nonfictionauthor_id_gram! :
    SYMBOL << #0 = #[Tok_nonfictionauthor_id_gram, $1]; >>
    ;

pubs_gram! :
    << AST *ast1 = NULL; >>
    "\(" "Pubs" pubs_id_gram (pubs_item_gram << ast1 = appendAst(#1, ast1); >> )* "\)"
    << #0 = #(#[Tok_pubs_gram, "pubs_gram"], #3, ast1); >>
    ;

pubs_item_gram! :
    << AST *ast1 = NULL; >>
    (
        publisher_gram << ast1 = #1; >> |
        location_gram << ast1 = #1; >> |
        date_gram << ast1 = #1; >>
    )
    << #0 = ast1; >>
    ;

pubs_id_gram! :
    SYMBOL << #0 = #[Tok_pubs_id_gram, $1]; >>
    ;

date_gram! :
    "\(" "date" STRING "\)"
    << #0 = #(#[Tok_date_gram, $3]); >>
    ;

location_gram! :
    "\(" "location" STRING "\)"
    << #0 = #(#[Tok_location_gram, $3]); >>
    ;

publisher_gram! :
    "\(" "publisher" STRING "\)"
    << #0 = #(#[Tok_publisher_gram, $3]); >>
    ;

authnotestitle_gram! :
    "\(" "authnotesTitle" STRING "\)"
    << #0 = #(#[Tok_authnotestitle_gram, $3]); >>
    ;

name_gram! :
    "\(" "name" STRING "\)"
    << #0 = #(#[Tok_name_gram, $3]); >>
    ;

topicarea_gram! :
    "\(" "topicArea" STRING "\)"
    << #0 = #(#[Tok_topicarea_gram, $3]); >>
    ;

period_gram! :
    "\(" "period" STRING "\)"
    << #0 = #(#[Tok_period_gram, $3]); >>
    ;

style_gram! :
    "\(" "style" STRING "\)"
    << #0 = #(#[Tok_style_gram, $3]); >>
    ;

docnotestitle_gram! :
    "\(" "docnotesTitle" STRING "\)"
    << #0 = #(#[Tok_docnotestitle_gram, $3]); >>
    ;

authorid_gram! :
    "\(" "authorid" STRING "\)"
    << #0 = #(#[Tok_authorid_gram, $3]); >>
    ;

doctitle_gram! :
    "\(" "doctitle" STRING "\)"
    << #0 = #(#[Tok_doctitle_gram, $3]); >>
    ;

bibltitle_gram! :
    "\(" "bibltitle" STRING "\)"
    << #0 = #(#[Tok_bibltitle_gram, $3]); >>
    ;

selecteditem_gram! :
    << char buf[64]; >>
    "\(" "selectedItem" boolean_gram[buf] "\)"
    << #0 = #(#[Tok_selecteditem_gram, buf]); >>
    ;

doccount_gram! :
    << char buf[128]; >>
    "\(" "docCount" int_gram[buf] "\)"
    << #0 = #(#[Tok_doccount_gram, buf]); >>
    ;

emotionaltemperature_gram! :
    << char buf[128]; >>
    "\(" "emotionalTemperature" float_gram[buf] "\)"
    << #0 = #(#[Tok_emotionaltemperature_gram, buf]); >>
    ;

authnotes_list_gram! :
    << AST *ast1 = NULL; >>
    "\(" "authNotes" ( string_gram << ast1 = appendAst(#1, ast1); >> )* "\)"
    << #0 = #(#[Tok_authnotes_list_gram, "authnotes_list_gram"], ast1); >>
    ;

worksinprogress_list_gram! :
    << AST *ast1 = NULL; >>
    "\(" "worksInProgress" ( document_gram << ast1 = appendAst(#1, ast1); >> )* "\)"
    << #0 = #(#[Tok_worksinprogress_list_gram, "worksinprogress_list_gram"], ast1); >>
    ;

publishedworks_list_gram! :
    << AST *ast1 = NULL; >>
    "\(" "publishedWorks" ( document_gram << ast1 = appendAst(#1, ast1); >> )* "\)"
    << #0 = #(#[Tok_publishedworks_list_gram, "publishedworks_list_gram"], ast1); >>
    ;

biases_list_gram! :
    << AST *ast1 = NULL; >>
    "\(" "biases" ( string_gram << ast1 = appendAst(#1, ast1); >> )* "\)"
    << #0 = #(#[Tok_biases_list_gram, "biases_list_gram"], ast1); >>
    ;

specialfields_list_gram! :
    << AST *ast1 = NULL; >>
    "\(" "specialFields" ( string_gram << ast1 = appendAst(#1, ast1); >> )* "\)"
    << #0 = #(#[Tok_specialfields_list_gram, "specialfields_list_gram"], ast1); >>
    ;

docnotes_list_gram! :
    << AST *ast1 = NULL; >>
    "\(" "docnotes" ( string_gram << ast1 = appendAst(#1, ast1); >> )* "\)"
    << #0 = #(#[Tok_docnotes_list_gram, "docnotes_list_gram"], ast1); >>
    ;

xrefs_list_gram! :
    << AST *ast1 = NULL; >>
    "\(" "xrefs" ( string_gram << ast1 = appendAst(#1, ast1); >> )* "\)"
    << #0 = #(#[Tok_xrefs_list_gram, "xrefs_list_gram"], ast1); >>
    ;

keywords_list_gram! :
    << AST *ast1 = NULL; >>
    "\(" "keywords" ( string_gram << ast1 = appendAst(#1, ast1); >> )* "\)"
    << #0 = #(#[Tok_keywords_list_gram, "keywords_list_gram"], ast1); >>
    ;

authors_list_gram! :
    << AST *ast1 = NULL; >>
    "\(" "authors" ( author_super_gram << ast1 = appendAst(#1, ast1); >> )* "\)"
    << #0 = #(#[Tok_authors_list_gram, "authors_list_gram"], ast1); >>
    ;

documents_list_gram! :
    << AST *ast1 = NULL; >>
    "\(" "documents" ( document_gram << ast1 = appendAst(#1, ast1); >> )* "\)"
    << #0 = #(#[Tok_documents_list_gram, "documents_list_gram"], ast1); >>
    ;

boolean_gram[char * buf] :
    (
        "true" << strcpy(buf, "\"true\""); >> |
        "false" << strcpy(buf, "\"false\""); >> 
    )
    ;

string_gram! :
    STRING << #0 = #(#[Tok_string_gram, $1]); >>
    ;

int_gram[char * buf] :
    INTEGER << strcpy(buf, $1); >>
    ;

float_gram[char * buf] :
    FLOAT << strcpy(buf, $1); >>
    | INTEGER << strcpy(buf, $1); >>
    ;

#token SYMBOL "[a-zA-Z][a-zA-Z0-9_]*"
#token INTEGER "[0-9]+"
#token FLOAT "[0-9]+\.[0-9]+"

