// kr.hpp    Header file for models

/*
*
* Send comments to:
*   Dave Kuhlman
*   Reify
*   426 Madison Ave #9
*   Redwood City, CA 94061
*   415-368-8450
*   Internet: dkuhlman@netcom.com
*
* Copyright (c) 1994 G. David Kuhlman.
* All rights reserved.
*
* Permission is hereby granted, without written agreement and without
* license or royalty fees, to use, copy, modify, and distribute this
* software and its documentation for any purpose, provided that the
* above copyright notice and the following two paragraphs appear in
* all copies of this software.
* 
* IN NO EVENT SHALL G. DAVID KUHLMAN BE LIABLE TO ANY PARTY FOR
* DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
* OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF G. DAVID KUHLMAN
* HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* G. DAVID KUHLMAN SPECIFICALLY DISCLAIMS ANY WARRANTIES,
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
* ON AN "AS IS" BASIS, AND G. DAVID KUHLMAN HAS NO OBLIGATION TO
* PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*
*/

// kr.hpp    Header file for models

#ifndef kr_HPP
#define kr_HPP

#include <stdlib.h>
#include <fstream.h>
#include <iostream.h>
#include <mdlsyn.hpp>
#include <db.hpp>
#include <dbblk.hpp>
#include "krstrg.hpp"
#include "krstrm.hpp"

// ---------------------------------------------------------

class KrObject;
class Bibliography;
struct DbBibliography;
typedef Db::RecNumber DbBibliographyPtr;
class Document;
struct DbDocument;
typedef Db::RecNumber DbDocumentPtr;
class Author;
struct DbAuthor;
typedef Db::RecNumber DbAuthorPtr;
class FictionAuthor;
struct DbFictionAuthor;
typedef Db::RecNumber DbFictionAuthorPtr;
class HumorFictionAuthor;
struct DbHumorFictionAuthor;
typedef Db::RecNumber DbHumorFictionAuthorPtr;
class RomanticFictionAuthor;
struct DbRomanticFictionAuthor;
typedef Db::RecNumber DbRomanticFictionAuthorPtr;
class NonfictionAuthor;
struct DbNonfictionAuthor;
typedef Db::RecNumber DbNonfictionAuthorPtr;
class Pubs;
struct DbPubs;
typedef Db::RecNumber DbPubsPtr;

// ---------------------------------------------------------

class BibliographyLink {
public:
    BibliographyLink() : data(NULL), next(NULL) { }
    BibliographyLink(Bibliography * initData)
        : data(initData), next(NULL)
        { }
    BibliographyLink(Bibliography * initData, BibliographyLink * initNext)
        : data(initData), next(initNext)
        { }
    BibliographyLink(const BibliographyLink & rhs);
    ~BibliographyLink() { delete next; };
    Bibliography * GetData(void) { return data; };
    BibliographyLink * GetNext(void) { return next; };
    void SetNext(BibliographyLink * newNext) { next = newNext; }
    BibliographyLink * Add(Bibliography * newItem)
    {
        return new BibliographyLink(newItem, this);
    } // Add
    BibliographyLink & operator =(const BibliographyLink & rhs)
    {
        if (this == &rhs)
        {
            return *this;
        } // if

        data = rhs.data;
        next = rhs.next;
        return *this;
    } // BibliographyLink::operator =
protected:
private:
    Bibliography * data;
    BibliographyLink *next;
}; // BibliographyLink

class BibliographyList {
public:
    BibliographyList() : first(NULL), last(NULL) { }
    BibliographyList(Bibliography * initData) {
        first = new BibliographyLink(initData);
        last = first;
    }
    void Append(Bibliography * initData);
    void Insert(Bibliography * initData, int pos);
    boolean Delete(int pos);
    BibliographyLink * GetFirst() { return first; }
    BibliographyLink * GetLast() { return last; }
    Bibliography * operator [](int pos);
    BibliographyList * Copy();
    static BibliographyList * Load(DbBlock * block_, Db::RecNumber firstRecno_);
    Db::RecNumber Save(DbBlock * block_);
protected:
private:
    BibliographyLink *first;
    BibliographyLink *last;
}; // BibliographyList


class DocumentLink {
public:
    DocumentLink() : data(NULL), next(NULL) { }
    DocumentLink(Document * initData)
        : data(initData), next(NULL)
        { }
    DocumentLink(Document * initData, DocumentLink * initNext)
        : data(initData), next(initNext)
        { }
    DocumentLink(const DocumentLink & rhs);
    ~DocumentLink() { delete next; };
    Document * GetData(void) { return data; };
    DocumentLink * GetNext(void) { return next; };
    void SetNext(DocumentLink * newNext) { next = newNext; }
    DocumentLink * Add(Document * newItem)
    {
        return new DocumentLink(newItem, this);
    } // Add
    DocumentLink & operator =(const DocumentLink & rhs)
    {
        if (this == &rhs)
        {
            return *this;
        } // if

        data = rhs.data;
        next = rhs.next;
        return *this;
    } // DocumentLink::operator =
protected:
private:
    Document * data;
    DocumentLink *next;
}; // DocumentLink

class DocumentList {
public:
    DocumentList() : first(NULL), last(NULL) { }
    DocumentList(Document * initData) {
        first = new DocumentLink(initData);
        last = first;
    }
    void Append(Document * initData);
    void Insert(Document * initData, int pos);
    boolean Delete(int pos);
    DocumentLink * GetFirst() { return first; }
    DocumentLink * GetLast() { return last; }
    Document * operator [](int pos);
    DocumentList * Copy();
    static DocumentList * Load(DbBlock * block_, Db::RecNumber firstRecno_);
    Db::RecNumber Save(DbBlock * block_);
protected:
private:
    DocumentLink *first;
    DocumentLink *last;
}; // DocumentList


class AuthorLink {
public:
    AuthorLink() : data(NULL), next(NULL) { }
    AuthorLink(Author * initData)
        : data(initData), next(NULL)
        { }
    AuthorLink(Author * initData, AuthorLink * initNext)
        : data(initData), next(initNext)
        { }
    AuthorLink(const AuthorLink & rhs);
    ~AuthorLink() { delete next; };
    Author * GetData(void) { return data; };
    AuthorLink * GetNext(void) { return next; };
    void SetNext(AuthorLink * newNext) { next = newNext; }
    AuthorLink * Add(Author * newItem)
    {
        return new AuthorLink(newItem, this);
    } // Add
    AuthorLink & operator =(const AuthorLink & rhs)
    {
        if (this == &rhs)
        {
            return *this;
        } // if

        data = rhs.data;
        next = rhs.next;
        return *this;
    } // AuthorLink::operator =
protected:
private:
    Author * data;
    AuthorLink *next;
}; // AuthorLink

class AuthorList {
public:
    AuthorList() : first(NULL), last(NULL) { }
    AuthorList(Author * initData) {
        first = new AuthorLink(initData);
        last = first;
    }
    void Append(Author * initData);
    void Insert(Author * initData, int pos);
    boolean Delete(int pos);
    AuthorLink * GetFirst() { return first; }
    AuthorLink * GetLast() { return last; }
    Author * operator [](int pos);
    AuthorList * Copy();
    static AuthorList * Load(DbBlock * block_, Db::RecNumber firstRecno_);
    Db::RecNumber Save(DbBlock * block_);
protected:
private:
    AuthorLink *first;
    AuthorLink *last;
}; // AuthorList


class FictionAuthorLink {
public:
    FictionAuthorLink() : data(NULL), next(NULL) { }
    FictionAuthorLink(FictionAuthor * initData)
        : data(initData), next(NULL)
        { }
    FictionAuthorLink(FictionAuthor * initData, FictionAuthorLink * initNext)
        : data(initData), next(initNext)
        { }
    FictionAuthorLink(const FictionAuthorLink & rhs);
    ~FictionAuthorLink() { delete next; };
    FictionAuthor * GetData(void) { return data; };
    FictionAuthorLink * GetNext(void) { return next; };
    void SetNext(FictionAuthorLink * newNext) { next = newNext; }
    FictionAuthorLink * Add(FictionAuthor * newItem)
    {
        return new FictionAuthorLink(newItem, this);
    } // Add
    FictionAuthorLink & operator =(const FictionAuthorLink & rhs)
    {
        if (this == &rhs)
        {
            return *this;
        } // if

        data = rhs.data;
        next = rhs.next;
        return *this;
    } // FictionAuthorLink::operator =
protected:
private:
    FictionAuthor * data;
    FictionAuthorLink *next;
}; // FictionAuthorLink

class FictionAuthorList {
public:
    FictionAuthorList() : first(NULL), last(NULL) { }
    FictionAuthorList(FictionAuthor * initData) {
        first = new FictionAuthorLink(initData);
        last = first;
    }
    void Append(FictionAuthor * initData);
    void Insert(FictionAuthor * initData, int pos);
    boolean Delete(int pos);
    FictionAuthorLink * GetFirst() { return first; }
    FictionAuthorLink * GetLast() { return last; }
    FictionAuthor * operator [](int pos);
    FictionAuthorList * Copy();
    static FictionAuthorList * Load(DbBlock * block_, Db::RecNumber firstRecno_);
    Db::RecNumber Save(DbBlock * block_);
protected:
private:
    FictionAuthorLink *first;
    FictionAuthorLink *last;
}; // FictionAuthorList


class HumorFictionAuthorLink {
public:
    HumorFictionAuthorLink() : data(NULL), next(NULL) { }
    HumorFictionAuthorLink(HumorFictionAuthor * initData)
        : data(initData), next(NULL)
        { }
    HumorFictionAuthorLink(HumorFictionAuthor * initData, HumorFictionAuthorLink * initNext)
        : data(initData), next(initNext)
        { }
    HumorFictionAuthorLink(const HumorFictionAuthorLink & rhs);
    ~HumorFictionAuthorLink() { delete next; };
    HumorFictionAuthor * GetData(void) { return data; };
    HumorFictionAuthorLink * GetNext(void) { return next; };
    void SetNext(HumorFictionAuthorLink * newNext) { next = newNext; }
    HumorFictionAuthorLink * Add(HumorFictionAuthor * newItem)
    {
        return new HumorFictionAuthorLink(newItem, this);
    } // Add
    HumorFictionAuthorLink & operator =(const HumorFictionAuthorLink & rhs)
    {
        if (this == &rhs)
        {
            return *this;
        } // if

        data = rhs.data;
        next = rhs.next;
        return *this;
    } // HumorFictionAuthorLink::operator =
protected:
private:
    HumorFictionAuthor * data;
    HumorFictionAuthorLink *next;
}; // HumorFictionAuthorLink

class HumorFictionAuthorList {
public:
    HumorFictionAuthorList() : first(NULL), last(NULL) { }
    HumorFictionAuthorList(HumorFictionAuthor * initData) {
        first = new HumorFictionAuthorLink(initData);
        last = first;
    }
    void Append(HumorFictionAuthor * initData);
    void Insert(HumorFictionAuthor * initData, int pos);
    boolean Delete(int pos);
    HumorFictionAuthorLink * GetFirst() { return first; }
    HumorFictionAuthorLink * GetLast() { return last; }
    HumorFictionAuthor * operator [](int pos);
    HumorFictionAuthorList * Copy();
    static HumorFictionAuthorList * Load(DbBlock * block_, Db::RecNumber firstRecno_);
    Db::RecNumber Save(DbBlock * block_);
protected:
private:
    HumorFictionAuthorLink *first;
    HumorFictionAuthorLink *last;
}; // HumorFictionAuthorList


class RomanticFictionAuthorLink {
public:
    RomanticFictionAuthorLink() : data(NULL), next(NULL) { }
    RomanticFictionAuthorLink(RomanticFictionAuthor * initData)
        : data(initData), next(NULL)
        { }
    RomanticFictionAuthorLink(RomanticFictionAuthor * initData, RomanticFictionAuthorLink * initNext)
        : data(initData), next(initNext)
        { }
    RomanticFictionAuthorLink(const RomanticFictionAuthorLink & rhs);
    ~RomanticFictionAuthorLink() { delete next; };
    RomanticFictionAuthor * GetData(void) { return data; };
    RomanticFictionAuthorLink * GetNext(void) { return next; };
    void SetNext(RomanticFictionAuthorLink * newNext) { next = newNext; }
    RomanticFictionAuthorLink * Add(RomanticFictionAuthor * newItem)
    {
        return new RomanticFictionAuthorLink(newItem, this);
    } // Add
    RomanticFictionAuthorLink & operator =(const RomanticFictionAuthorLink & rhs)
    {
        if (this == &rhs)
        {
            return *this;
        } // if

        data = rhs.data;
        next = rhs.next;
        return *this;
    } // RomanticFictionAuthorLink::operator =
protected:
private:
    RomanticFictionAuthor * data;
    RomanticFictionAuthorLink *next;
}; // RomanticFictionAuthorLink

class RomanticFictionAuthorList {
public:
    RomanticFictionAuthorList() : first(NULL), last(NULL) { }
    RomanticFictionAuthorList(RomanticFictionAuthor * initData) {
        first = new RomanticFictionAuthorLink(initData);
        last = first;
    }
    void Append(RomanticFictionAuthor * initData);
    void Insert(RomanticFictionAuthor * initData, int pos);
    boolean Delete(int pos);
    RomanticFictionAuthorLink * GetFirst() { return first; }
    RomanticFictionAuthorLink * GetLast() { return last; }
    RomanticFictionAuthor * operator [](int pos);
    RomanticFictionAuthorList * Copy();
    static RomanticFictionAuthorList * Load(DbBlock * block_, Db::RecNumber firstRecno_);
    Db::RecNumber Save(DbBlock * block_);
protected:
private:
    RomanticFictionAuthorLink *first;
    RomanticFictionAuthorLink *last;
}; // RomanticFictionAuthorList


class NonfictionAuthorLink {
public:
    NonfictionAuthorLink() : data(NULL), next(NULL) { }
    NonfictionAuthorLink(NonfictionAuthor * initData)
        : data(initData), next(NULL)
        { }
    NonfictionAuthorLink(NonfictionAuthor * initData, NonfictionAuthorLink * initNext)
        : data(initData), next(initNext)
        { }
    NonfictionAuthorLink(const NonfictionAuthorLink & rhs);
    ~NonfictionAuthorLink() { delete next; };
    NonfictionAuthor * GetData(void) { return data; };
    NonfictionAuthorLink * GetNext(void) { return next; };
    void SetNext(NonfictionAuthorLink * newNext) { next = newNext; }
    NonfictionAuthorLink * Add(NonfictionAuthor * newItem)
    {
        return new NonfictionAuthorLink(newItem, this);
    } // Add
    NonfictionAuthorLink & operator =(const NonfictionAuthorLink & rhs)
    {
        if (this == &rhs)
        {
            return *this;
        } // if

        data = rhs.data;
        next = rhs.next;
        return *this;
    } // NonfictionAuthorLink::operator =
protected:
private:
    NonfictionAuthor * data;
    NonfictionAuthorLink *next;
}; // NonfictionAuthorLink

class NonfictionAuthorList {
public:
    NonfictionAuthorList() : first(NULL), last(NULL) { }
    NonfictionAuthorList(NonfictionAuthor * initData) {
        first = new NonfictionAuthorLink(initData);
        last = first;
    }
    void Append(NonfictionAuthor * initData);
    void Insert(NonfictionAuthor * initData, int pos);
    boolean Delete(int pos);
    NonfictionAuthorLink * GetFirst() { return first; }
    NonfictionAuthorLink * GetLast() { return last; }
    NonfictionAuthor * operator [](int pos);
    NonfictionAuthorList * Copy();
    static NonfictionAuthorList * Load(DbBlock * block_, Db::RecNumber firstRecno_);
    Db::RecNumber Save(DbBlock * block_);
protected:
private:
    NonfictionAuthorLink *first;
    NonfictionAuthorLink *last;
}; // NonfictionAuthorList


class PubsLink {
public:
    PubsLink() : data(NULL), next(NULL) { }
    PubsLink(Pubs * initData)
        : data(initData), next(NULL)
        { }
    PubsLink(Pubs * initData, PubsLink * initNext)
        : data(initData), next(initNext)
        { }
    PubsLink(const PubsLink & rhs);
    ~PubsLink() { delete next; };
    Pubs * GetData(void) { return data; };
    PubsLink * GetNext(void) { return next; };
    void SetNext(PubsLink * newNext) { next = newNext; }
    PubsLink * Add(Pubs * newItem)
    {
        return new PubsLink(newItem, this);
    } // Add
    PubsLink & operator =(const PubsLink & rhs)
    {
        if (this == &rhs)
        {
            return *this;
        } // if

        data = rhs.data;
        next = rhs.next;
        return *this;
    } // PubsLink::operator =
protected:
private:
    Pubs * data;
    PubsLink *next;
}; // PubsLink

class PubsList {
public:
    PubsList() : first(NULL), last(NULL) { }
    PubsList(Pubs * initData) {
        first = new PubsLink(initData);
        last = first;
    }
    void Append(Pubs * initData);
    void Insert(Pubs * initData, int pos);
    boolean Delete(int pos);
    PubsLink * GetFirst() { return first; }
    PubsLink * GetLast() { return last; }
    Pubs * operator [](int pos);
    PubsList * Copy();
    static PubsList * Load(DbBlock * block_, Db::RecNumber firstRecno_);
    Db::RecNumber Save(DbBlock * block_);
protected:
private:
    PubsLink *first;
    PubsLink *last;
}; // PubsList



// ---------------------------------------------------------

// Error reporting.
// User must define this function.

void KrReportError(char *msg, int level);

enum {
    KR_ERROR_LEVEL_1 = 1,
    KR_ERROR_LEVEL_2,
    KR_ERROR_LEVEL_3,
    KR_ERROR_LEVEL_4
};


// ---------------------------------------------------------

enum PARSE_NODE_TYPES {
    NT_firstjunk = 0,
    NT_Bibliography,
    NT_Document,
    NT_Author,
    NT_FictionAuthor,
    NT_HumorFictionAuthor,
    NT_RomanticFictionAuthor,
    NT_NonfictionAuthor,
    NT_Pubs,
    NT_lastjunk
};

// ---------------------------------------------------------

class KrObject {
public:
    void Set_bibltitle(KrString * initbibltitle_) {}
    KrString * Get_bibltitle() { return NULL; }
    void Set_documents(DocumentList * initdocuments_) {}
    DocumentList * Get_documents() { return NULL; }
    void Set_authors(AuthorList * initauthors_) {}
    AuthorList * Get_authors() { return NULL; }
    void Set_selectedItem(boolean initselectedItem) {}
    boolean Get_selectedItem() { return 0; }
    void Set_doctitle(KrString * initdoctitle_) {}
    KrString * Get_doctitle() { return NULL; }
    void Set_authorid(KrString * initauthorid_) {}
    KrString * Get_authorid() { return NULL; }
    void Set_pubdata(Pubs * initpubdata_) {}
    Pubs * Get_pubdata() { return NULL; }
    void Set_keywords(KrStringList * initkeywords_) {}
    KrStringList * Get_keywords() { return NULL; }
    void Set_xrefs(KrStringList * initxrefs_) {}
    KrStringList * Get_xrefs() { return NULL; }
    void Set_docnotes(KrStringList * initdocnotes_) {}
    KrStringList * Get_docnotes() { return NULL; }
    void Set_docnotesTitle(KrString * initdocnotesTitle_) {}
    KrString * Get_docnotesTitle() { return NULL; }
    void Set_name(KrString * initname_) {}
    KrString * Get_name() { return NULL; }
    void Set_publishedWorks(DocumentList * initpublishedWorks_) {}
    DocumentList * Get_publishedWorks() { return NULL; }
    void Set_worksInProgress(DocumentList * initworksInProgress_) {}
    DocumentList * Get_worksInProgress() { return NULL; }
    void Set_authNotes(KrStringList * initauthNotes_) {}
    KrStringList * Get_authNotes() { return NULL; }
    void Set_authnotesTitle(KrString * initauthnotesTitle_) {}
    KrString * Get_authnotesTitle() { return NULL; }
    void Set_topicArea(KrString * inittopicArea_) {}
    KrString * Get_topicArea() { return NULL; }
    void Set_style(KrString * initstyle_) {}
    KrString * Get_style() { return NULL; }
    void Set_period(KrString * initperiod_) {}
    KrString * Get_period() { return NULL; }
    void Set_emotionalTemperature(float initemotionalTemperature) {}
    float Get_emotionalTemperature() { return 0; }
    void Set_docCount(int initdocCount) {}
    int Get_docCount() { return 0; }
    void Set_specialFields(KrStringList * initspecialFields_) {}
    KrStringList * Get_specialFields() { return NULL; }
    void Set_biases(KrStringList * initbiases_) {}
    KrStringList * Get_biases() { return NULL; }
    void Set_publisher(KrString * initpublisher_) {}
    KrString * Get_publisher() { return NULL; }
    void Set_location(KrString * initlocation_) {}
    KrString * Get_location() { return NULL; }
    void Set_date(KrString * initdate_) {}
    KrString * Get_date() { return NULL; }
};

// ---------------------------------------------------------

class Bibliography {
public:
    Bibliography();
    Bibliography(const Bibliography& rhs_);
    virtual ~Bibliography();
    Bibliography& operator=(const Bibliography& rhs_);
    virtual void Decompile(KrOstream& outf_, int level_, int & count_);
    static Bibliography * Load(DbBlock * block_, Db::RecNumber recno_, Db::RecNumber & nextRecno_);
    virtual Db::RecNumber Save(DbBlock * block_, Db::RecNumber prevRecno_);
    void GenIpf(KrOstream & outf_, int level_);
// Data member access functions//gen.icn/450
    void Set_id(KrString * initid_) { id_ = initid_; }
    KrString * Get_id() { return id_; }
    void Set_bibltitle(KrString * initbibltitle_)
        { bibltitle = initbibltitle_; }
    KrString * Get_bibltitle() { return bibltitle; }
    void Set_documents(DocumentList * initdocuments_)
        { documents = initdocuments_; }
    DocumentList * Get_documents() { return documents; }
    void Set_authors(AuthorList * initauthors_)
        { authors = initauthors_; }
    AuthorList * Get_authors() { return authors; }
protected:
    KrString * id_;
    KrString * bibltitle;
    DocumentList * documents;
    AuthorList * authors;
private:
    Bibliography(DbBlock * block_, DbBibliography & dbBibliography_);
    void BuildDbRec(DbBlock * block_, DbBibliography& dbBibliography);
    void BuildRec(DbBlock * block_, DbBibliography& dbBibliography);
}; // Bibliography

// Database record:

struct DbBibliography {
    Db::RecNumber next_;
    Db::RecNumber id_;
    Db::RecNumber bibltitle;
    Db::RecNumber documents;
    Db::RecNumber authors;
}; // DbBibliography

// ---------------------------------------------------------

class Document {
public:
    Document();
    Document(const Document& rhs_);
    virtual ~Document();
    Document& operator=(const Document& rhs_);
    virtual void Decompile(KrOstream& outf_, int level_, int & count_);
    static Document * Load(DbBlock * block_, Db::RecNumber recno_, Db::RecNumber & nextRecno_);
    virtual Db::RecNumber Save(DbBlock * block_, Db::RecNumber prevRecno_);
    void GenIpf(KrOstream & outf_, int level_);
// Data member access functions//gen.icn/450
    void Set_id(KrString * initid_) { id_ = initid_; }
    KrString * Get_id() { return id_; }
    void Set_selectedItem(boolean initselectedItem)
        { selectedItem = initselectedItem; }
    boolean Get_selectedItem() { return selectedItem; }
    void Set_doctitle(KrString * initdoctitle_)
        { doctitle = initdoctitle_; }
    KrString * Get_doctitle() { return doctitle; }
    void Set_authorid(KrString * initauthorid_)
        { authorid = initauthorid_; }
    KrString * Get_authorid() { return authorid; }
    void Set_pubdata(Pubs * initpubdata_)
        { pubdata = initpubdata_; }
    Pubs * Get_pubdata() { return pubdata; }
    void Set_keywords(KrStringList * initkeywords_)
        { keywords = initkeywords_; }
    KrStringList * Get_keywords() { return keywords; }
    void Set_xrefs(KrStringList * initxrefs_)
        { xrefs = initxrefs_; }
    KrStringList * Get_xrefs() { return xrefs; }
    void Set_docnotes(KrStringList * initdocnotes_)
        { docnotes = initdocnotes_; }
    KrStringList * Get_docnotes() { return docnotes; }
    void Set_docnotesTitle(KrString * initdocnotesTitle_)
        { docnotesTitle = initdocnotesTitle_; }
    KrString * Get_docnotesTitle() { return docnotesTitle; }
protected:
    KrString * id_;
    boolean selectedItem;
    KrString * doctitle;
    KrString * authorid;
    Pubs * pubdata;
    KrStringList * keywords;
    KrStringList * xrefs;
    KrStringList * docnotes;
    KrString * docnotesTitle;
private:
    Document(DbBlock * block_, DbDocument & dbDocument_);
    void BuildDbRec(DbBlock * block_, DbDocument& dbDocument);
    void BuildRec(DbBlock * block_, DbDocument& dbDocument);
}; // Document

// Database record:

struct DbDocument {
    Db::RecNumber next_;
    Db::RecNumber id_;
    boolean selectedItem;
    Db::RecNumber doctitle;
    Db::RecNumber authorid;
    Db::RecNumber pubdata;
    Db::RecNumber keywords;
    Db::RecNumber xrefs;
    Db::RecNumber docnotes;
    Db::RecNumber docnotesTitle;
}; // DbDocument

// ---------------------------------------------------------

class Author {
public:
    Author();
    Author(const Author& rhs_);
    virtual ~Author();
    Author& operator=(const Author& rhs_);
    virtual void Decompile(KrOstream& outf_, int level_, int & count_);
    static Author * Load(DbBlock * block_, Db::RecNumber recno_, Db::RecNumber & nextRecno_);
    virtual Db::RecNumber Save(DbBlock * block_, Db::RecNumber prevRecno_);
    void GenIpf(KrOstream & outf_, int level_);
// Data member access functions//gen.icn/450
    void Set_id(KrString * initid_) { id_ = initid_; }
    KrString * Get_id() { return id_; }
    void Set_name(KrString * initname_)
        { name = initname_; }
    KrString * Get_name() { return name; }
    void Set_publishedWorks(DocumentList * initpublishedWorks_)
        { publishedWorks = initpublishedWorks_; }
    DocumentList * Get_publishedWorks() { return publishedWorks; }
    void Set_worksInProgress(DocumentList * initworksInProgress_)
        { worksInProgress = initworksInProgress_; }
    DocumentList * Get_worksInProgress() { return worksInProgress; }
    void Set_authNotes(KrStringList * initauthNotes_)
        { authNotes = initauthNotes_; }
    KrStringList * Get_authNotes() { return authNotes; }
    void Set_authnotesTitle(KrString * initauthnotesTitle_)
        { authnotesTitle = initauthnotesTitle_; }
    KrString * Get_authnotesTitle() { return authnotesTitle; }
protected:
    KrString * id_;
    KrString * name;
    DocumentList * publishedWorks;
    DocumentList * worksInProgress;
    KrStringList * authNotes;
    KrString * authnotesTitle;
private:
    Author(DbBlock * block_, DbAuthor & dbAuthor_);
    void BuildDbRec(DbBlock * block_, DbAuthor& dbAuthor);
    void BuildRec(DbBlock * block_, DbAuthor& dbAuthor);
}; // Author

// Database record:

struct DbAuthor {
    Db::RecNumber next_;
    Db::RecNumber id_;
    Db::RecNumber name;
    Db::RecNumber publishedWorks;
    Db::RecNumber worksInProgress;
    Db::RecNumber authNotes;
    Db::RecNumber authnotesTitle;
}; // DbAuthor

// ---------------------------------------------------------

class FictionAuthor : public Author {
public:
    FictionAuthor();
    FictionAuthor(const FictionAuthor& rhs_);
    virtual ~FictionAuthor();
    FictionAuthor& operator=(const FictionAuthor& rhs_);
    virtual void Decompile(KrOstream& outf_, int level_, int & count_);
    static FictionAuthor * Load(DbBlock * block_, Db::RecNumber recno_, Db::RecNumber & nextRecno_);
    virtual Db::RecNumber Save(DbBlock * block_, Db::RecNumber prevRecno_);
    void GenIpf(KrOstream & outf_, int level_);
// Data member access functions//gen.icn/450
    void Set_id(KrString * initid_) { id_ = initid_; }
    KrString * Get_id() { return id_; }
    void Set_topicArea(KrString * inittopicArea_)
        { topicArea = inittopicArea_; }
    KrString * Get_topicArea() { return topicArea; }
protected:
    KrString * topicArea;
private:
    FictionAuthor(DbBlock * block_, DbFictionAuthor & dbFictionAuthor_);
    void BuildDbRec(DbBlock * block_, DbFictionAuthor& dbFictionAuthor);
    void BuildRec(DbBlock * block_, DbFictionAuthor& dbFictionAuthor);
}; // FictionAuthor

// Database record:

struct DbFictionAuthor {
    Db::RecNumber next_;
    Db::RecNumber id_;
    Db::RecNumber topicArea;
    Db::RecNumber name;
    Db::RecNumber publishedWorks;
    Db::RecNumber worksInProgress;
    Db::RecNumber authNotes;
    Db::RecNumber authnotesTitle;
}; // DbFictionAuthor

// ---------------------------------------------------------

class HumorFictionAuthor : public FictionAuthor {
public:
    HumorFictionAuthor();
    HumorFictionAuthor(const HumorFictionAuthor& rhs_);
    virtual ~HumorFictionAuthor();
    HumorFictionAuthor& operator=(const HumorFictionAuthor& rhs_);
    virtual void Decompile(KrOstream& outf_, int level_, int & count_);
    static HumorFictionAuthor * Load(DbBlock * block_, Db::RecNumber recno_, Db::RecNumber & nextRecno_);
    virtual Db::RecNumber Save(DbBlock * block_, Db::RecNumber prevRecno_);
    void GenIpf(KrOstream & outf_, int level_);
// Data member access functions//gen.icn/450
    void Set_id(KrString * initid_) { id_ = initid_; }
    KrString * Get_id() { return id_; }
    void Set_style(KrString * initstyle_)
        { style = initstyle_; }
    KrString * Get_style() { return style; }
protected:
    KrString * style;
private:
    HumorFictionAuthor(DbBlock * block_, DbHumorFictionAuthor & dbHumorFictionAuthor_);
    void BuildDbRec(DbBlock * block_, DbHumorFictionAuthor& dbHumorFictionAuthor);
    void BuildRec(DbBlock * block_, DbHumorFictionAuthor& dbHumorFictionAuthor);
}; // HumorFictionAuthor

// Database record:

struct DbHumorFictionAuthor {
    Db::RecNumber next_;
    Db::RecNumber id_;
    Db::RecNumber style;
    Db::RecNumber topicArea;
    Db::RecNumber name;
    Db::RecNumber publishedWorks;
    Db::RecNumber worksInProgress;
    Db::RecNumber authNotes;
    Db::RecNumber authnotesTitle;
}; // DbHumorFictionAuthor

// ---------------------------------------------------------

class RomanticFictionAuthor : public FictionAuthor {
public:
    RomanticFictionAuthor();
    RomanticFictionAuthor(const RomanticFictionAuthor& rhs_);
    virtual ~RomanticFictionAuthor();
    RomanticFictionAuthor& operator=(const RomanticFictionAuthor& rhs_);
    virtual void Decompile(KrOstream& outf_, int level_, int & count_);
    static RomanticFictionAuthor * Load(DbBlock * block_, Db::RecNumber recno_, Db::RecNumber & nextRecno_);
    virtual Db::RecNumber Save(DbBlock * block_, Db::RecNumber prevRecno_);
    void GenIpf(KrOstream & outf_, int level_);
// Data member access functions//gen.icn/450
    void Set_id(KrString * initid_) { id_ = initid_; }
    KrString * Get_id() { return id_; }
    void Set_period(KrString * initperiod_)
        { period = initperiod_; }
    KrString * Get_period() { return period; }
    void Set_emotionalTemperature(float initemotionalTemperature)
        { emotionalTemperature = initemotionalTemperature; }
    float Get_emotionalTemperature() { return emotionalTemperature; }
protected:
    KrString * period;
    float emotionalTemperature;
private:
    RomanticFictionAuthor(DbBlock * block_, DbRomanticFictionAuthor & dbRomanticFictionAuthor_);
    void BuildDbRec(DbBlock * block_, DbRomanticFictionAuthor& dbRomanticFictionAuthor);
    void BuildRec(DbBlock * block_, DbRomanticFictionAuthor& dbRomanticFictionAuthor);
}; // RomanticFictionAuthor

// Database record:

struct DbRomanticFictionAuthor {
    Db::RecNumber next_;
    Db::RecNumber id_;
    Db::RecNumber period;
    float emotionalTemperature;
    Db::RecNumber topicArea;
    Db::RecNumber name;
    Db::RecNumber publishedWorks;
    Db::RecNumber worksInProgress;
    Db::RecNumber authNotes;
    Db::RecNumber authnotesTitle;
}; // DbRomanticFictionAuthor

// ---------------------------------------------------------

class NonfictionAuthor : public Author {
public:
    NonfictionAuthor();
    NonfictionAuthor(const NonfictionAuthor& rhs_);
    virtual ~NonfictionAuthor();
    NonfictionAuthor& operator=(const NonfictionAuthor& rhs_);
    virtual void Decompile(KrOstream& outf_, int level_, int & count_);
    static NonfictionAuthor * Load(DbBlock * block_, Db::RecNumber recno_, Db::RecNumber & nextRecno_);
    virtual Db::RecNumber Save(DbBlock * block_, Db::RecNumber prevRecno_);
    void GenIpf(KrOstream & outf_, int level_);
// Data member access functions//gen.icn/450
    void Set_id(KrString * initid_) { id_ = initid_; }
    KrString * Get_id() { return id_; }
    void Set_docCount(int initdocCount)
        { docCount = initdocCount; }
    int Get_docCount() { return docCount; }
    void Set_specialFields(KrStringList * initspecialFields_)
        { specialFields = initspecialFields_; }
    KrStringList * Get_specialFields() { return specialFields; }
    void Set_biases(KrStringList * initbiases_)
        { biases = initbiases_; }
    KrStringList * Get_biases() { return biases; }
protected:
    int docCount;
    KrStringList * specialFields;
    KrStringList * biases;
private:
    NonfictionAuthor(DbBlock * block_, DbNonfictionAuthor & dbNonfictionAuthor_);
    void BuildDbRec(DbBlock * block_, DbNonfictionAuthor& dbNonfictionAuthor);
    void BuildRec(DbBlock * block_, DbNonfictionAuthor& dbNonfictionAuthor);
}; // NonfictionAuthor

// Database record:

struct DbNonfictionAuthor {
    Db::RecNumber next_;
    Db::RecNumber id_;
    int docCount;
    Db::RecNumber specialFields;
    Db::RecNumber biases;
    Db::RecNumber name;
    Db::RecNumber publishedWorks;
    Db::RecNumber worksInProgress;
    Db::RecNumber authNotes;
    Db::RecNumber authnotesTitle;
}; // DbNonfictionAuthor

// ---------------------------------------------------------

class Pubs {
public:
    Pubs();
    Pubs(const Pubs& rhs_);
    virtual ~Pubs();
    Pubs& operator=(const Pubs& rhs_);
    virtual void Decompile(KrOstream& outf_, int level_, int & count_);
    static Pubs * Load(DbBlock * block_, Db::RecNumber recno_, Db::RecNumber & nextRecno_);
    virtual Db::RecNumber Save(DbBlock * block_, Db::RecNumber prevRecno_);
    void GenIpf(KrOstream & outf_, int level_);
// Data member access functions//gen.icn/450
    void Set_id(KrString * initid_) { id_ = initid_; }
    KrString * Get_id() { return id_; }
    void Set_publisher(KrString * initpublisher_)
        { publisher = initpublisher_; }
    KrString * Get_publisher() { return publisher; }
    void Set_location(KrString * initlocation_)
        { location = initlocation_; }
    KrString * Get_location() { return location; }
    void Set_date(KrString * initdate_)
        { date = initdate_; }
    KrString * Get_date() { return date; }
protected:
    KrString * id_;
    KrString * publisher;
    KrString * location;
    KrString * date;
private:
    Pubs(DbBlock * block_, DbPubs & dbPubs_);
    void BuildDbRec(DbBlock * block_, DbPubs& dbPubs);
    void BuildRec(DbBlock * block_, DbPubs& dbPubs);
}; // Pubs

// Database record:

struct DbPubs {
    Db::RecNumber next_;
    Db::RecNumber id_;
    Db::RecNumber publisher;
    Db::RecNumber location;
    Db::RecNumber date;
}; // DbPubs

// ---------------------------------------------------------

enum {
    NT_NULL = 0,
    NT_pubdata,
    NT_bibltitle,
    NT_doctitle,
    NT_biases,
    NT_authnotesTitle,
    NT_period,
    NT_authors,
    NT_topicArea,
    NT_date,
    NT_selectedItem,
    NT_style,
    NT_publisher,
    NT_docCount,
    NT_emotionalTemperature,
    NT_keywords,
    NT_authNotes,
    NT_authorid,
    NT_name,
    NT_publishedWorks,
    NT_worksInProgress,
    NT_docnotesTitle,
    NT_location,
    NT_xrefs,
    NT_specialFields,
    NT_documents,
    NT_docnotes,
    NT_junk
};

// ---------------------------------------------------------

// Function prototypes

Bibliography * Compile(char * infilename, int show);

// --------------------------------------------------------

#endif // kr_HPP

