#ifndef SORCERER_H
#define SORCERER_H

/*
 * sorcerer.h -- header for all sorcerer files
 *
 * SOFTWARE RIGHTS
 *
 * We reserve no LEGAL rights to SORCERER -- SORCERER is in the public
 * domain.  An individual or company may do whatever they wish with
 * source code distributed with SORCERER or the code generated by
 * SORCERER, including the incorporation of SORCERER, or its output, into
 * commerical software.
 * 
 * We encourage users to develop software with SORCERER.  However, we do
 * ask that credit is given to us for developing SORCERER.  By "credit",
 * we mean that if you incorporate our source code into one of your
 * programs (commercial product, research project, or otherwise) that you
 * acknowledge this fact somewhere in the documentation, research report,
 * etc...  If you like SORCERER and have developed a nice tool with the
 * output, please mention that you developed it using SORCERER.  In
 * addition, we ask that this header remain intact in our source code.
 * As long as these guidelines are kept, we expect to continue enhancing
 * this system and expect to make other tools available as they are
 * completed.
 *
 * SORCERER 1.00B
 * Terence Parr
 * AHPCRC, University of Minnesota
 * 1992-1994
 */

#ifdef __STDC__
#define __USE_PROTOS
#endif
#ifdef __cplusplus
#ifndef __USE_PROTOS
#define __USE_PROTOS
#endif
#endif

#include "setjmp.h"

/* SUPERCLASS AST (your tree must look at minimum like this)
typedef struct _node {
            struct _node *right, *down;			-- if not transform mode
            struct _node *right[N], *down[N];	-- if transform mode
            int token;
			-- user-defined stuff goes here
        } AST;
*/

/* Can be used sort of like inheritance to get the desired struct def */
#define AST_NONTRANSFORM_FIELDS \
            struct _node *right, *down; \
            int token;

#define AST_TRANSFORM_FIELDS \
            struct _node *right[2], *down[2]; \
            int token;

/* E.g.,
typedef struct _node {
			AST_TRANSFORM_STRUCT
			-- my fields
        } AST;
*/

#define _MATCH(tok)	if ( _t->token!=tok ) if ( _parser->guessing ) _GUESS_FAIL; else mismatched_token(_parser, tok, _t)
#define _MATCHRANGE(tok,tok2) \
		if ( _t->token<tok || _t->token>tok2 ) if ( _parser->guessing ) _GUESS_FAIL; else mismatched_range(_parser, tok, tok2, _t)

/* Normal DOWN and RIGHT */
#define _DOWN				_t=_t->down
#define _RIGHT				_t=_t->right

/* Transform DOWN and RIGHT */
#define _TDOWN				{if ( !_parser->guessing ) _tresult=&_t->ast_down(_parser->write); \
							 _t=_t->ast_down(_parser->read);}
#define _TRIGHT				{if ( !_parser->guessing ) _tresult=&_t->ast_right(_parser->write); \
							 _t=_t->ast_right(_parser->read);}

#define _SAVE				AST *_save=_t
#define _RESTORE			_t = _save
#define	_SETLABEL(u)		u=_t
#define _WILDCARD			if ( _t==NULL ) if ( _parser->guessing ) _GUESS_FAIL; else missing_wildcard(_parser)
#define _GUESS_BLOCK		STreeParser _st; int _gv; AST *_savet=NULL;
#define _GUESS				{_st = *_parser; \
							_savet = _t; \
							_parser->guessing = 1; \
							_gv = setjmp(_parser->startofguess.state);}
#define _GUESS_FAIL			longjmp(_parser->startofguess.state, 1)
#define _GUESS_DONE			{*_parser = _st; _t = _savet;}

#ifdef SORCERER_TRANSFORM
#ifndef ast_down
#define ast_down(rw)		down[rw]
#endif
#ifndef ast_right
#define ast_right(rw)		right[rw]
#endif

#else

#ifndef ast_down
#define ast_down(rw)		down
#endif
#ifndef ast_right
#define ast_right(rw)		right
#endif
#endif

/* OLD VERSION
#define STreeTry(p)			(p)->guessing = 1; \
							if ( setjmp((p)->startofguess.state)==0 )
*/

#define STreeTry(r,p,t)     \
			(p)->try_result = NULL;					\
            (p)->sjrv = setjmp((p)->startofguess);  \
            if ( !(p)->sjrv ) {						\
                rule(p,t,&try_result);				\
                (p)->try_ok = 1;					\
			}										\
            else {									\
                (p)->try_ok = 0;					\
			}										\
            if ( (p)->try_ok )


typedef struct _Sjmp_buf {
			jmp_buf state;
		} Sjmp_buf;

#ifndef _PARSER_VARS
#define _PARSER_VARS
#endif

#ifndef _REFVARS
#define _REFVARS
#endif

typedef struct _STreeParser {
			int try_ok, sjrv;	/* used by STreeTry macro */
			AST *try_result;	/* tree coming back from try */
			int guessing;
			Sjmp_buf startofguess;
			AST *t;
			int read, write; /* read/write from which ptr number (transform) */
			_REFVARS
			_PARSER_VARS
		} STreeParser;

#define STreeParserInit(_p) { (_p)->guessing = 0; _refvar_inits(_p); }


				/* S a n i t y  C h e c k i n g */

#ifndef require
#define require(expr, err) {if ( !(expr) ) sorcerer_panic(err);}
#endif


               /* T r a n s f o r m  M a c r o s */
#define ast_return(_t)	*_result = _t


#ifdef __USE_PROTOS
extern void mismatched_range(STreeParser *_parser, int looking_for, int upper_token, AST *found);
extern void missing_wildcard(STreeParser *_parser);
extern void mismatched_token(STreeParser *_parser, int looking_for, AST *found);
extern void no_viable_alt(STreeParser *_parser, char *rulename, AST *root);
extern AST **_nextresult(STreeParser *_parser, AST **t);
extern void sorcerer_panic(char *err);
extern void _refvars_inits(STreeParser *);
#else
extern void mismatched_range();
extern void missing_wildcard();
extern void mismatched_token();
extern void no_viable_alt();
extern AST **_nextresult();
extern void sorcerer_panic();
extern void _refvars_inits();
#endif

#endif
