// krstrg.cpp    Member functions for KrString

/*
* Copyright (c) 1994 G. David Kuhlman.
* All rights reserved.
*
* Permission is hereby granted, without written agreement and without
* license or royalty fees, to use, copy, modify, and distribute this
* software and its documentation for any purpose, provided that the
* above copyright notice and the following two paragraphs appear in
* all copies of this software.
* 
* IN NO EVENT SHALL G. DAVID KUHLMAN BE LIABLE TO ANY PARTY FOR
* DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
* OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF G. DAVID KUHLMAN
* HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* G. DAVID KUHLMAN SPECIFICALLY DISCLAIMS ANY WARRANTIES,
* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
* ON AN "AS IS" BASIS, AND G. DAVID KUHLMAN HAS NO OBLIGATION TO
* PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

#include "krstrg.hpp"

// --------------------------------------------------------

// Private constants:

// Private globals:

// Private function prototypes:

// --------------------------------------------------------

KrString::KrString()
#if defined(__WATCOMC__)
:   rep(new String())
#elif defined(__IBMCPP__)
:   rep(new IString())
#else // __WATCOMC__
:   rep(new string())
#endif // __WATCOMC__
{
} // KrString::KrString

// --------------------------------------------------------

KrString::KrString(char * initStr)
#if defined(__WATCOMC__)
:   rep(new String(initStr))
#elif defined(__IBMCPP__)
:   rep(new IString(initStr))
#else // __WATCOMC__
:   rep(new string(initStr))
#endif // __WATCOMC__
{
} // KrString::KrString

// --------------------------------------------------------

KrString::KrString(const KrString& rhs)
{
    if (this == &rhs)
    {
        return;
    } // if
#if defined(__WATCOMC__)
    rep = new String(*rhs.rep);
#elif defined(__IBMCPP__)
    rep = new IString(*rhs.rep);
#else // __WATCOMC__
    rep = new string(*rhs.rep);
#endif // __WATCOMC__

} // KrString::KrString

// ---------------------------------------------------------

// Constructor to create KrString by loading from the database.
KrString::KrString(DbBlock * block, Db::RecNumber recno)
{
    char * pStr;

    if (recno == Db::DB_NULL_RECID)
    {
        return;
    } // if

    pStr = block->GetStr(recno);
#if defined(__WATCOMC__)
    rep = new String(pStr);
#elif defined(__IBMCPP__)
    rep = new IString(pStr);
#else // __WATCOMC__
    rep = new string(pStr);
#endif // __WATCOMC__
} // KrString::KrString

// --------------------------------------------------------

KrString::~KrString()
{
    delete rep;
} // KrString::~KrString

// ---------------------------------------------------------

// Decompile member function for class KrString
void KrString::Decompile(KrOstream& outf, int level, int & count)
{
    char buf[1024];

    EscapeChars(buf, sizeof(buf));
    (outf <<= level) << "\"" << buf << "\"" << endl;
} // KrString::Decompile

// ---------------------------------------------------------

// Replace special characters with escaped (backslashed) characters.
void KrString::EscapeChars(char * dest, int maxBufLen)
{
    char * pSrc;
    char * pDest;

    for (
#if defined(__WATCOMC__)
            pSrc = ((char *)(const char *)*rep), pDest = dest;
#elif defined(__IBMCPP__)
            pSrc = ((char *)*rep), pDest = dest;
#else // __WATCOMC__
            pSrc = (char *)(rep->c_str()), pDest = dest;
#endif // __WATCOMC__
            (*pSrc != '\0') && ((pDest - dest + 1) < maxBufLen);
            pSrc++, pDest++
        )
    {
        switch (*pSrc)
        {
        case '\"':
            *pDest = '\\';
            pDest++;
            *pDest = '\"';
            break;
        case '\\':
            *pDest = '\\';
            pDest++;
            *pDest = '\\';
            break;
        case '\t':
            *pDest = '\\';
            pDest++;
            *pDest = 't';
            break;
        case '\n':
            *pDest = '\\';
            pDest++;
            *pDest = 'n';
            break;
        default:
            *pDest = *pSrc;
            break;
        } /* switch */
    } /* for */
    *pDest = '\0';
} // KrString::EscapeChars

// ---------------------------------------------------------

// Save the KrString in the database.
//
Db::RecNumber KrString::Save(DbBlock * block)
{
    Db::RecNumber recno;

#if defined(__WATCOMC__)
    recno = block->AddDbString((char *)(const char *)*rep);
#elif defined(__IBMCPP__)
    recno = block->AddDbString((char *)*rep);
#else // __WATCOMC__
    recno = block->AddDbString(rep->c_str());
#endif // __WATCOMC__
    return recno;
} // KrString::Save

// --------------------------------------------------------

KrString &KrString::operator =(const KrString& rhs)
{
    if (this == &rhs)
    {
        return *this;
    } // if
    rep = rhs.rep;

    return *this;
} // KrString::operator =

// --------------------------------------------------------

#ifdef INCL_CHARSTAR_OP

KrString::operator char *()
{
#if defined(__WATCOMC__)
    return (char *)(const char *)*rep;
#elif defined(__IBMCPP__)
    return (char *)*rep;
#else // __WATCOMC__
    return (char *)rep->c_str();
#endif // __WATCOMC__
} // KrString::operator char *

#endif // INCL_CHARSTAR_OP

// --------------------------------------------------------

void KrStringList::Append(KrString * initData)
{
    KrStringLink * pLink = new KrStringLink(initData);
    if (first == NULL)
    {
        first = pLink;
        last = pLink;
    } // if
    else
    {
        last->SetNext(pLink);
        last = pLink;
    } // else
} // KrStringList::Append

// --------------------------------------------------------

// Insert item into list at position pos.
//  If pos > size of list, insert at end of list.
void KrStringList::Insert(KrString * initData, int pos)
{
    KrStringLink * pLink = new KrStringLink(initData);
    if (first == NULL)
    {
        first = pLink;
        last = pLink;
    } // if
    else if (pos == 0)
    {
        pLink->SetNext(first);
        first = pLink;
    } // else if
    else
    {
        KrStringLink * pL = first;
        int idx = 0;
        while (true)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                // Add it to the end.
                last->SetNext(pLink);
                last = pLink;
                break;
            } // if
            if (idx == pos)
            {
                // We are at the desired position. Add it.
                pLink->SetNext(pL->GetNext());
                pL->SetNext(pLink);
                break;
            } // if
            idx++;
            pL = pL->GetNext();
        } // while
    } // else
} // KrStringList::Insert

// --------------------------------------------------------

// Delete an item from list at position pos.
//  If pos > size of list, return false; else return true.
boolean KrStringList::Delete(int pos)
{
    if (first == NULL)
    {
        return false;
    } // if
    else if (pos == 0)
    {
        KrStringLink * pLink = first;
        first = first->GetNext();
        KrString * pKrString = pLink->GetData();
        delete pLink;
        delete pKrString;
        if (first == NULL)
        {
            last = NULL;
        } // if
    } // else if
    else
    {
        KrStringLink * pL = first->GetNext();
        KrStringLink * pPrevLink = first;
        int idx = 1;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                return false;
            } // if
            if (idx == pos)
            {
                // We are at the desired position. Delete it.
                pPrevLink->SetNext(pL->GetNext());
                if (pL == last)
                {
                    last = pPrevLink;
                } // if
                KrString * pKrString = pL->GetData();
                delete pL;
                delete pKrString;
                break;
            } // if
            idx++;
            pPrevLink = pL;
            pL = pL->GetNext();
        } // for
    } // else
    return true;
} // KrStringList::Delete

// --------------------------------------------------------

// Return the item at position pos (relative to 0).
KrString * KrStringList::operator [](int pos)
{
    if (first == NULL)
    {
        return NULL;
    } // if
    else if (pos == 0)
    {
        return first->GetData();
    } // else if
    else
    {
        KrStringLink * pL = first;
        int idx = 0;
        for (;;)
        {
            if (pL == NULL)
            {
                // We ran off the end of the list.
                return NULL;
            } // if
            if (idx == pos)
            {
                // We are at the desired position.
                return pL->GetData();
            } // if
            idx++;
            pL = pL->GetNext();
        } // for
    } // else
} // KrStringList::operator []

// --------------------------------------------------------

KrStringList * KrStringList::Copy()
{
    if (first != NULL)
    {
        KrStringList * pList = new KrStringList;
        KrStringLink * pLink = first;
        while (pLink != NULL)
        {
            KrString * pData1 = pLink->GetData();
            if (pData1 != NULL)
            {
                KrString * pData2 = new KrString(*pData1);
                pList->Append(pData2);
            } // if
            pLink = pLink->GetNext();
        } // while
        return pList;
    } // if
    return NULL;
} // KrStringList::Copy

// --------------------------------------------------------

KrStringList * KrStringList::Load(DbBlock * block, Db::RecNumber firstRecno)
{
    Db::RecNumber recno = firstRecno;
    Db::RecNumber nextRecno;
    DbString link;

    if (recno != Db::DB_NULL_RECID)
    {
        KrStringList *pNewList = new KrStringList;
        while (recno != Db::DB_NULL_RECID)
        {
            block->GetRec(recno, &link);
            KrString * pKrString = new KrString(block, link.data);
            pNewList->Insert(pKrString, 0);
            recno = link.next;
        } // while
        return pNewList;
    } // if
    return NULL;
} // KrStringList::Load

// --------------------------------------------------------

Db::RecNumber KrStringList::Save(DbBlock * block)
{
    Db::RecNumber recno;
    Db::RecNumber prevRecno = Db::DB_NULL_RECID;
    Db::RecNumber linkRecno = Db::DB_NULL_RECID;
    DbString link;

    KrStringLink * pL = first;
    if (pL != NULL)
    {
        while (pL != NULL)
        {
            KrString * pS = pL->GetData();
            if (pS != NULL)
            {
                recno = pS->Save(block);

                link.next = prevRecno;
                link.data = recno;
                linkRecno = block->AddDbRec(&link, sizeof(link));

                prevRecno = linkRecno;
                pL = pL->GetNext();
            } // if
        } // while
    } // if
    return linkRecno;
} // KrStringList::Save

// --------------------------------------------------------

